#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "skiplist.h"
#include "hashtest.h"
#include "anchors.h"


char seq1name[255];
char seq2name[255];

float gapopen =0, gapcont=0;
int gapfreechunks = 0;
hll* parseCHAOS(int8_t* qr, int qrlen, int K, int* totnum);
hll* findBestChain(hptr* myarr, int arrsize);
void doOutput(hll* mylist);
hll* sortList(hll* mylist);

hll multi_matched_kmer;
static ds *dslist,*dslist_dgl;
static int dslist_size;
int anchors_setup(int dblen, int qrlen){
	dslist_size=dblen*qrlen;
	dslist=(ds*)calloc(dslist_size,sizeof(ds));
	dslist_dgl=(ds*)calloc(qrlen,sizeof(ds));
	multi_matched_kmer.scoreSoFar=-1;
	if(dslist){
		return 0;
	}else{
		return 1;
	}
}
static int hptrcomp (const void *p1, const void *p2) {
  int i = ((hptr*)p1)->number;
  int j = ((hptr*)p2)->number;
  int it = ((hptr*)p1)->isstart;
  int jt = ((hptr*)p2)->isstart;
  if (i > j)
    return (1);
  if (i < j)
    return (-1);
  if (it)
    return -1;
  else 
    return 1;
}
static int dscomp (const void *p1, const void *p2) {
  int i = ((ds*)p1)->diagonal_level;
  int j = ((ds*)p2)->diagonal_level;
  if (i > j){
    return (1);
  }else{
  //if (i < j)
    return (-1);
  }
}
static int dglcomp (const void *p1, const void *p2) {
  int i = ((ds*)p1)->db_start;
  int j = ((ds*)p2)->db_start;
  if (i > j){
    return (1);
  }else{
  //if (i < j)
    return (-1);
  }
}

hll* laganAnchors(int8_t* qr,int qrlen, int K,hll **mylist){

  hll *temp, *best;//*mylist
  int numhits, i=0;
  hptr* myptrs;

  *mylist = parseCHAOS(qr, qrlen, K, &numhits);

  //printf("numhits:%i\n",numhits);
  if (numhits==0){
    	return 0;
  }else if(numhits == -1){
  	return &multi_matched_kmer;
  }
  myptrs = (hptr*) malloc (sizeof(hptr) * numhits *2);
  if(!myptrs){
	fprintf(stderr,"myptrs alloc failed.\n");
	abort();
  }
  i = 0;
  for (temp = *mylist; temp; temp = temp->next) {
    //printf("%i:%i\n",i,temp->seq1start);
    myptrs[i].number  = temp->seq1start;
    //printf("%i:%i\n",i,temp->seq2start);
    myptrs[i].isstart = 1;
    myptrs[i].myhll = temp;
    myptrs[i+1].number  = temp->seq1end;
    myptrs[i+1].isstart = 0;
    myptrs[i+1].myhll = temp;
    i = i+2;
  }
  qsort(myptrs, numhits*2, sizeof(hptr), hptrcomp);
  best = findBestChain(myptrs, numhits*2);
  //doOutput(best);
  free(myptrs);


  return best;
}

int whRulez(hll* one, hll* two) {
  float gapdiff = ((float)(two->seq2end - one->seq2end)) * gapcont;
  return two->scoreSoFar-one->scoreSoFar-gapdiff > 0;
}

float gapPen(hll* next, hll* prev) {
  float j= ((float)(next->seq2start-prev->seq2end))*gapcont + gapopen;
  //  printf("%d (%f)*(%f) %f gap\n", next->seq2start-prev->seq2end, ((float)(next->seq2start-prev->seq2end)),gapcont,j);
  return j;
}

hll* findBestChain(hptr* array, int arrsize) {
  sklst* skipper = makeSkLst();
  sle* help, *bestptr;
  float best = -1;
  int i;
  for (i = 0; i < arrsize; i++) {
    if (array[i].isstart) {
      help = SLfind(skipper, array[i].myhll->seq2start);
      if (help->myelem && (gapPen(array[i].myhll, ((hll*)help->myelem)) + ((hll*)help->myelem)->scoreSoFar) > 0) {
    	  array[i].myhll->bkptr = (HitLocationList*)help->myelem;
    	  array[i].myhll->scoreSoFar = ((hll*)help->myelem)->scoreSoFar + array[i].myhll->score + gapPen(array[i].myhll, ((hll*)help->myelem));
      }
      else {
    	  array[i].myhll->bkptr = 0;
    	  array[i].myhll->scoreSoFar = array[i].myhll->score;
      }
    }else {
      help = SLfind(skipper, array[i].myhll->seq2end);

      if (help->myelem && whRulez(array[i].myhll,((hll*)help->myelem)))
    	  continue;
      SLinsertAfter(skipper, help, array[i].myhll->seq2end, array[i].myhll);
      help = help->next[0];

      while (help->next[0] && !whRulez(((hll*)help->myelem), ((hll*)help->next[0]->myelem)))
    	  SLremove(skipper, help->next[0]);
    }
  }
  help = skipper->sentinel->next[0];
  while (help) {
    if (((hll*)help->myelem)->scoreSoFar > best) {
      best = ((hll*)help->myelem)->scoreSoFar;
      bestptr = help;
    } 
    help = help->next[0];
  }
  delSkLst(skipper);
  free(skipper);
  return (hll*)bestptr->myelem;
}

void doOutput(hll* best) { 
  int len;

  hll *bestPtr=best, *temp;
  int chl=0, i, bestscore=-1;
  gfc* tmpgf;
  for (temp = bestPtr; temp; temp = temp->bkptr) {
    chl++;
  }


  for (temp = bestPtr; temp; temp = temp->bkptr) {
    len = temp->seq1end - temp->seq1start + 1 ;
    //if (!gapfreechunks || !temp->first) {
    if (!gapfreechunks ) {
      printf("(%d %d)=",temp->seq2start, temp->seq2end);
      printf("(%d %d) %f %f\n",temp->seq1start, temp->seq1end, temp->score,temp->scoreSoFar);
    }else {
    	/*
      for (tmpgf = temp->first; tmpgf ; tmpgf = tmpgf->next) {
    	  printf("(%d %d)=(%d %d) %d\n", tmpgf->y, tmpgf->y + tmpgf->length-1, tmpgf->x, tmpgf->x + tmpgf->length-1,
	      tmpgf->score);
      }*/
    }
  }
}

char* rolltonum(char* str) {
  char *got1=0, *got2=0;
  int in=0, i=0;
  while (1) {
    if (str[i] == 0) {
      break;
    }
    if (str[i] == ';' && got1 && got2){
      return got1;
    }
    if (isdigit(str[i])) {
      if (!in && (!i || isspace(str[i-1]))) { 
	if (got1) 
	  got2 = &str[i];
	else 
	  got1 = &str[i];
	in = 1;
      }
    }
    else if (in && (isspace(str[i]))) {
      if (got2) {
	got1 = got2; got2=0; in = 0;
      }
      in = 0;
    }

    else {
      in = 0;
      got1=got2=0;
    }
    i++;
  }
  return &str[i];
}

int getline(FILE* infile, hll* tt) {
  char temp[1024];
  char* help;
  int z, h;
  fgets(temp, 1024, infile);
   help = rolltonum(temp);
  z = sscanf(help, "%d %d;%n", &tt->seq2start, &tt->seq2end, &h);
  if (z<2)
    return 0;
  help = rolltonum(help+h);
  if (sscanf(help,"%d %d; score = %f (%*c)\n", &tt->seq1start,
	     &tt->seq1end,&tt->score)<3)
    return 0;
  return 1;
}



hll* parseCHAOS(int8_t* qr, int qrlen, int K, int* totnum) {
  hll *myres=0, *tt;
  gfc* temp;
  *totnum = 0;
  int8_t* q;
  int8_t tmp[K];
  bool multi_matched=false;
  int i,j;
  q=qr;
	struct nlist *ret;

	int dsindex=0;
	for(i=0;i<qrlen-K+1;i++){
		//memcpy(tmp,q++,K);
		ret = hashtable_getvalue(q,K);
		q++;
		if ( ret == NULL){
			//printf("not found\n");
		}else{
			if(!ret->multi_value){
				dslist[dsindex].diagonal_level=(i+1)-ret->value;
				dslist[dsindex].db_start=ret->value;
				dsindex++;
			}else{
				multi_matched=true;
				//printf("multi seeds.\n");
			}
		}

	}

	qsort(dslist, dsindex, sizeof(ds), dscomp);
	//for(j=0;j<dsindex;j++){
		//printf("ds1:%i,%i\n",dslist[j].diagonal_level,dslist[j].db_start);
	//}
	if(dsindex>0){
		int cur_diagonal;
		cur_diagonal=dslist[0].diagonal_level;
		for(j=0;j<dsindex;){
			int k=0;
			while(j+k<dsindex && dslist[j+k].diagonal_level==cur_diagonal){
				//printf("ds3:%i,%i\n",dslist[j+k].diagonal_level,dslist[j+k].db_start);
				k++;
			}

			memcpy(&dslist_dgl[0],&dslist[j],k*sizeof(ds));

			j+=k;
			int l,m=0,square_len,inc,square_end;
			qsort(dslist_dgl, k, sizeof(ds), dglcomp);
			square_len=K;
			for(l=0;l<k-1;l+=inc){
					square_len=K;
					inc=1;
					square_end=dslist_dgl[l].db_start+K-1;
					for(m=l+1;m<k;m++){
					//      if(dslist_dgl[m].db_start-dslist_dgl[l].db_start==inc){

					//              inc++;
					//              square_len++;
							if(square_end - dslist_dgl[m].db_start >= -1){
									inc++;
									square_end=dslist_dgl[m].db_start+K-1;
									square_len=square_end - dslist_dgl[l].db_start + 1;
							}else{

								break;
							}
					}

				//if( square_len<2*(K-1)){

					tt = (hll*) malloc(sizeof(hll));
					tt->seq1start=dslist_dgl[l].db_start;
					tt->seq1end=tt->seq1start+square_len-1;
					tt->seq2start=dslist_dgl[l].db_start+dslist_dgl[l].diagonal_level;
					tt->seq2end=tt->seq2start+square_len-1;
					tt->score=square_len;
					//printf("--tt %i,%i=%i,%i\n",tt->seq2start,tt->seq2end,tt->seq1start,tt->seq1end);
					tt->next = myres;
					tt->bkptr = 0;
					tt->scoreSoFar = 0;
					(*totnum)++;
					myres = tt;


				/*}else{
					tt = (hll*) malloc(sizeof(hll));
					tt->seq1start=dslist_dgl[l].db_start;
					tt->seq1end=tt->seq1start+ (K-1) -1 ;
					tt->seq2start=dslist_dgl[l].db_start+dslist_dgl[l].diagonal_level;
					tt->seq2end=tt->seq2start+ (K-1) -1 ;
					tt->score=(K-1);
					//printf("--tt1 %i,%i=%i,%i\n",tt->seq2start,tt->seq2end,tt->seq1start,tt->seq1end);
					tt->next = myres;
					tt->bkptr = 0;
					tt->scoreSoFar = 0;
					(*totnum)++;
					myres = tt;

					tt = (hll*) malloc(sizeof(hll));
					tt->seq1start=dslist_dgl[l].db_start;
					tt->seq1end=tt->seq1start+square_len-1 ;
					tt->seq2start=dslist_dgl[l].db_start+dslist_dgl[l].diagonal_level;
					tt->seq2end=tt->seq2start+square_len-1 ;
					tt->seq1start+=(K-1);
					tt->seq1end-=(K-1);
					tt->seq2start+=(K-1);
					tt->seq2end-=(K-1);
					tt->score=square_len - (K-1);
					if(tt->seq2start<=tt->seq2end){
						//printf("--tt2 %i,%i=%i,%i\n",tt->seq2start,tt->seq2end,tt->seq1start,tt->seq1end);
						tt->next = myres;
						tt->bkptr = 0;
						tt->scoreSoFar = 0;
						(*totnum)++;
						myres = tt;
					}else{
						free(tt);
					}

					tt = (hll*) malloc(sizeof(hll));
					//tt->seq1start=dslist_dgl[l].db_start;
					tt->seq1end=dslist_dgl[l].db_start+square_len-1 ;
					tt->seq1start=tt->seq1end - (K-1)+1;
					//tt->seq2start=dslist_dgl[l].db_start+dslist_dgl[l].diagonal_level;
					tt->seq2end=dslist_dgl[l].db_start+dslist_dgl[l].diagonal_level+square_len-1 ;
					tt->seq2start=tt->seq2end - (K-1)+1;
					tt->score=(K-1);
					//printf("--tt3 %i,%i=%i,%i\n",tt->seq2start,tt->seq2end,tt->seq1start,tt->seq1end);
					tt->next = myres;
					tt->bkptr = 0;
					tt->scoreSoFar = 0;
					(*totnum)++;
					myres = tt;
				}*/
			}
			if(l==0 || l<k-1){
				 l=k-1;
				 tt = (hll*) malloc(sizeof(hll));
				 tt->seq1start=dslist_dgl[l].db_start;
				 tt->seq1end=tt->seq1start+square_len-1;
				 tt->seq2start=dslist_dgl[l].db_start+dslist_dgl[l].diagonal_level;
				 tt->seq2end=tt->seq2start+square_len-1;
				 tt->score=square_len;
				 //printf("--tt %i,%i=%i,%i\n",tt->seq2start,tt->seq2end,tt->seq1start,tt->seq1end);
				 tt->next = myres;
				 tt->bkptr = 0;
				 tt->scoreSoFar = 0;
				 (*totnum)++;
				 myres = tt;

			}
			cur_diagonal=dslist[j].diagonal_level;

		}
	}
	if(*totnum==0 && multi_matched){
		*totnum=-1;
	}
  return myres;
}





