/*	$Id: sw-full-ls.c,v 1.16 2009/06/12 21:27:35 rumble Exp $	*/

#include <assert.h>
#include <ctype.h>
#include <errno.h>
#include <math.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>
#include <zlib.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>

#include "../common/fasta.h"
#include "../common/sw_split_ls.h"
#include "../common/util.h"
#include "../common/sw_split_common.h"
#include "hashtest.h"
#include "anchors.h"
#include "hashtest.h"
#define DEBUG_OUTPUT_LOG
struct max_t{
	int score;
	int pos;
	int multi;
};
struct swcell {
	int	score_north;
	int	score_west;
	int	score_northwest;
	int	score_northwest_jumped;
	int score_northwest_max;

	int8_t	back_north;
	int8_t	back_west;
	int8_t	back_northwest;
	int8_t	back_northwest_max;

	int pos;
	int multi_left;
	int multi_right;


};
/*
#define FROM_WEST 0x1
#define FROM_NORTHWEST 0x2
#define FROM_NORTH 0x3
#define FROM_NORTHWEST_MAX 0x4
*/
#define FROM_NORTH_NORTH		0x1
#define FROM_NORTH_NORTHWEST		0x2
#define	FROM_WEST_NORTHWEST		0x3
#define	FROM_WEST_WEST			0x4
#define FROM_NORTHWEST_NORTH		0x5
#define FROM_NORTHWEST_NORTHWEST	0x6
#define FROM_NORTHWEST_WEST		0x7

#define FROM_NORTHWEST_MAX_NORTH		0x08
#define FROM_NORTHWEST_MAX_NORTHWEST	0x09
#define FROM_NORTHWEST_MAX_WEST		0xA

#define FROM_NORTH_NORTHWEST_MAX	0xB
#define FROM_NORTHWEST_NORTHWEST_MAX	0xC
#define FROM_WEST_NORTHWEST_MAX	0xD

#define BACK_INSERTION			0x1
#define BACK_DELETION			0x2
#define BACK_MATCH_MISMATCH		0x3
#define BACK_N					0x4 //for splice cigar N
#define BACK_INSERTION_N		0x5

#define JUMP_LEN_THRESH			10
#define TAIL_LEN_THRESH_INS		10

#define MAX3(a,b,c) (( (a>b?a:b)>c)?(a>b?a:b):c)
#define MAX2(a,b) (a>b?a:b)
#define MINI_KMER_LEN			13

static int              initialised;
static int8_t	        *db, *qr;
static int		dblen, qrlen,rlen;
static int		a_gap_open, a_gap_ext;
static int		b_gap_open, b_gap_ext;
static int		split_penalty_ins, split_penalty_del_double,split_penalty_del_large,split_penalty_del;
static int		match, mismatch;
static struct swcell   *swmatrix;
static int8_t	       *backtrace;
static int	       *backtrace_pos;
static char	       *dbalign, *qralign;
static int 		kmer_len_array_len;
static int		kmer_len_array[3];


static struct max_t *max_w_cur,*max_w1,*max_w2, *max_n_cur,*max_n1,*max_n2;
static int gap_size;
#define HASHSIZE 4093

//static struct nlist *hashtable_kmer[HASHSIZE];
//static struct nlist *hashtable_merge[HASHSIZE];

/* statistics */
static uint64_t	swticks, swinvocs;


inline static void
init_cell(int idx) {
  
  swmatrix[idx].score_northwest = WORST_SCORE;
  swmatrix[idx].score_northwest_max = WORST_SCORE;
  swmatrix[idx].score_north = WORST_SCORE;
  swmatrix[idx].score_west = 0;
  swmatrix[idx].back_west = FROM_WEST_WEST;
}

void
print_swmatrix(int lena, int lenb){


	int i,j;
	/*
	return;
	for(i=0;i<lenb+1;i++){
	  for(j=0;j<lena+1;j++){

		  int score;
		  score=swmatrix[i * (lena + 1) + j].score_northwest;
		  if(score>0){
			  printf("%i,%i:",i,j);
			  printf("%i\n",score);
		  }else{
			  //printf("%s,","**");
		  }

	  }
	}
	return;
	*/
	return;
	printf("sw matrix:\n");
	  for(i=0;i<2*lenb+1;i++){
		  for(j=0;j<lena+1;j++){
			  printf("%i,%i:",i,j);
			  int score;
			  score=swmatrix[i * (lena + 1) + j].score_north;
			  if(score>-1000){
				  printf("%i,",score);
			  }else{
				  printf("%s,","**");
			  }
			  score=swmatrix[i * (lena + 1) + j].score_northwest;
			  if(score>-1000){
				  printf("%i,",score);
			  }else{
				  printf("%s,","**");
			  }
			  score=swmatrix[i * (lena + 1) + j].score_west;
			  if(score>-1000){
				  printf("%i,",score);
			  }else{
				  printf("%s,","**");
			  }
			  score=swmatrix[i * (lena + 1) + j].score_northwest_max;
			  if(score>-1000){
				  printf("%i,",score);
			  }else{
				  printf("%s,","**");
			  }
			  printf("%i\t",swmatrix[i * (lena + 1) + j].pos);
		  }
		  printf("\n");
	  }
}
void
realign_cigar(char *cigar, int genome_start,int read_start,int xy_gap_size){

		//printf("cigar:%s\n",cigar);
        int i,j;
        char *p,*startp;
        int m_len,d_len,i_len,mv_len;
        char *tmp_cigar;
        int gstart;
	int rstart;

        char realigned_cigar[200];
        realigned_cigar[0]='\0';
        m_len=d_len=i_len=0;
        gstart=genome_start;
	rstart=read_start;
        tmp_cigar=strdup(cigar);
        p=tmp_cigar;
	mv_len=0;
        startp=tmp_cigar;
		while(*p!='\0'){
			while(*p>='0' && *p<= '9'){
				p++;
			}
			if(*p=='M'){
				*p='\0';
				m_len=atoi(startp)+mv_len;
				if(m_len<=0){
                                        goto end;
                                }
				p++;
				startp=p;
			}else{
				//p++;
				//startp=p;
				//continue;
				goto end;
			}
			if(*p=='\0'){
				sprintf(realigned_cigar,"%s%iM",realigned_cigar,m_len);
				break;
			}
			while(*p>='0' && *p<= '9'){
				p++;
			}
			if(*p=='D' || *p=='N'){
				char a=*p;
				*p='\0';
				d_len=atoi(startp);
				if(a=='N'){
					d_len-=xy_gap_size;
				}
				p++;
				startp=p;

				mv_len=0;
				for(i=gstart+m_len-1,j=gstart+m_len+d_len-1;i>gstart && j>gstart;i--,j--){
					if(db[i]==db[j]){
						mv_len++;
					}else{
						break;
					}
				}
				gstart+=m_len+d_len;
				m_len-=mv_len;
				if(m_len<=0){
					goto end;
				}
				if(a=='N'){
					d_len+=xy_gap_size;
				}
				sprintf(realigned_cigar,"%s%iM%i%c",realigned_cigar,m_len,d_len,a);
			}else if(*p=='I'){
				*p='\0';
				i_len=atoi(startp);
				p++;
				startp=p;

				mv_len=0;
				for(i=rstart+m_len-1,j=rstart+m_len+i_len-1;i>rstart && j>rstart;i--,j--){
					if(qr[i]==qr[j]){
						mv_len++;
					}else{
						break;
					}
				}
				gstart+=m_len;
				m_len-=mv_len;

				sprintf(realigned_cigar,"%s%iM%iI",realigned_cigar,m_len,i_len);
			}else{
				goto end;
			}
			if(*p=='\0'){
				goto end;
			}
		}
		strcpy(cigar,realigned_cigar);
end:
		free(tmp_cigar);
		//printf("realigned_cigar:%s\n",realigned_cigar);

}
void
realign_cigar_inv(char *cigar, int genome_start,int read_start,int end_pos,int xy_gap_size,char *_tmp_cigar_left,
                char *_tmp_cigar_right,int *_head_len,int *_tail_len, int type){

        //printf("cigar:%s\thead_len:%i\ttail_len:%i\n",cigar,*_head_len,*_tail_len);
        int i,j;
        char *p,*startp;
        int m_len,d_len,i_len,mv_len;
        int gstart, rstart, gend;
        //int head_len, tail_len;
	
        char realigned_cigar[200];
	int head_len, tail_len;
	char tmp_cigar[200],tmp_cigar_left[200];

	head_len=*_head_len;
	tail_len=*_tail_len;
        realigned_cigar[0]='\0';
        m_len=d_len=i_len=0;
        gstart=genome_start;
        rstart=read_start;
        gend=end_pos;
	strcpy(tmp_cigar,cigar);
        p=tmp_cigar;
        startp=tmp_cigar;
	mv_len=0;

                while(*p!='\0'){
                        while(*p>='0' && *p<= '9'){
                                p++;
                        }
                        if(*p=='M'){
                                *p='\0';
                                m_len=atoi(startp)+mv_len;
				if(m_len<=0){
					goto end;
				}
                                p++;
                                startp=p;
                        }else{
                                //p++;
                                //startp=p;
                                //continue;
                                goto end;
                        }

                        if(*p=='\0'){
                                sprintf(realigned_cigar,"%s%iM",realigned_cigar,m_len);
                                break;
                        }

                        while(*p>='0' && *p<= '9'){
                                p++;
                        }
			if(*p=='D' || *p=='N'){
                                char a=*p;
                                *p='\0';
                                d_len=atoi(startp);
                                if(a=='N'){
                                //      d_len-=xy_gap_size;

                                }

                                p++;
                                startp=p;

                                mv_len=0;
                                switch(type){
                                case INVER_FW_OUT:
                                case INVER_BW_IN:
                                case SMALL_INVER_BW:
                                case DUP:
                                        for(i=gstart+m_len-1,j=gstart+m_len+d_len-1;i>gstart && j>gstart;i--,j--){
                                                if(db[i]==db[j]){
                                                        mv_len++;
                                                }else{
                                                        break;
                                                }
                                        }
                                        break;
                                case SMALL_INVER_FW:
                                case INVER_FW_IN:
                                case INVER_BW_OUT:
                                        //for(i=gstart+m_len-1,j=gstart+m_len+d_len-1;i>gstart && j>gstart;i--,j--){
                                        for(i=gstart+m_len,j=gstart+m_len+d_len;i<gend && j<gend;i++,j++){
                                                if(db[i]==db[j]){
                                                        mv_len--;
                                                }else{
                                                        break;
                                                }
                                        }
                                        break;
                                default:
                                        fprintf(stderr,"unknown inver type:%i\n",type);
                                        abort();
                                }
                                gstart+=m_len+d_len;
                                m_len-=mv_len;
				if(m_len<=0){
					goto end;
				}
				if(a=='N'){
                                //      d_len+=xy_gap_size;
                                        head_len-=mv_len;
                                        tail_len+=mv_len;
                                        sprintf(realigned_cigar,"%s%iM",realigned_cigar,m_len);
                                        strcpy(tmp_cigar_left,realigned_cigar);
                                        //printf("left:%s\n",tmp_cigar_left);
                                        realigned_cigar[0]='\0';
                                }else{
                                        sprintf(realigned_cigar,"%s%iM%i%c",realigned_cigar,m_len,d_len,a);
                                }
                        }else if(*p=='I'){
                                *p='\0';
                                i_len=atoi(startp);
                                p++;
                                startp=p;

                                mv_len=0;
                                for(i=rstart+m_len-1,j=rstart+m_len+i_len-1;i>rstart && j>rstart;i--,j--){
                                        if(qr[i]==qr[j]){
                                                mv_len++;
                                        }else{
                                                break;
                                        }
                                }
                                gstart+=m_len;
                                m_len-=mv_len;
				if(m_len<=0){
					goto end;
				}
                                sprintf(realigned_cigar,"%s%iM%iI",realigned_cigar,m_len,i_len);
                        }else{
                                goto end;
                        }
			if (*p=='\0'){
				goto end;
			}
                }
                strcpy(_tmp_cigar_left,tmp_cigar_left);
                strcpy(_tmp_cigar_right,realigned_cigar);
		*_head_len=head_len;
		*_tail_len=tail_len;
                //strcpy(cigar,realigned_cigar);
end:
                //free(tmp_cigar);
                //printf("realigned_cigar:%s\n",realigned_cigar);
        	//printf("cigar:%s\thead_len:%i\ttail_len:%i\n",cigar,*_head_len,*_tail_len);
		return;
}

static int
fill_diagonal(int lena, int lenb,int qr_start,int qr_end,int db_start,int db_end){
	int i;
	int start_score;
	int idx,idx2;
	idx=qr_start*(lena+1)+db_start;
	idx2=(qr_start+1)*(lena+1)+db_start+1;
	start_score=swmatrix[idx].score_northwest;
	swmatrix[idx2].back_northwest = FROM_NORTHWEST_NORTHWEST;
	swmatrix[idx2].score_north = WORST_SCORE;
	swmatrix[idx2].score_northwest_max =WORST_SCORE;

	swmatrix[idx2].score_west = WORST_SCORE;
	if(swmatrix[idx].score_northwest_max>start_score){
		start_score=swmatrix[idx].score_northwest_max;
		swmatrix[idx2].back_northwest = FROM_NORTHWEST_NORTHWEST_MAX;
	}

	if(swmatrix[idx].score_north>start_score){
		start_score=swmatrix[idx].score_north;
		swmatrix[idx2].back_northwest = FROM_NORTHWEST_NORTH;
	}
	if(swmatrix[idx].score_west>start_score){
		start_score=swmatrix[idx].score_west;
		swmatrix[idx2].back_northwest = FROM_NORTHWEST_WEST;
	}
	swmatrix[idx2].score_northwest = start_score+match;
	for(i=2;i<qr_end-qr_start+1;i++){
		idx=(qr_start+i)*(lena+1)+(db_start+i);
		swmatrix[idx].back_northwest = FROM_NORTHWEST_NORTHWEST;
		swmatrix[idx].score_north = WORST_SCORE;
		swmatrix[idx].score_northwest_max =WORST_SCORE;
		swmatrix[idx].score_northwest = start_score+match*i;
		swmatrix[idx].score_west = WORST_SCORE;
	}
	return 0;
}
static int
fill_sub_sw_block_x(int lena, int lenb, int x, int qr_start,int qr_end,int db_start,int db_end){
	int i,j;
	int ms,a_go, a_ge, b_go, b_ge, tmp;
	int8_t tmp2;
	int dbs,dbe,qrs,qre;
	int revcmpl=0;
	int top_score;

	a_go = a_gap_open;
	a_ge = a_gap_ext;
	b_go = b_gap_open;
	b_ge = b_gap_ext;
	dbs=db_start;
	dbe=db_end;
	qrs=qr_start;
	qre=qr_end;
	if(false){//dbs==0 || dbs==lenb){
		for (i = dbs; i < dbe + 1; i++) {
			init_cell(i);
		}
		for (i = qrs+1; i < qre + 1; i++) {
			int idx = i * (lena + 1 )+dbs;
			swmatrix[idx].back_north = FROM_NORTH_NORTH;
			swmatrix[idx].score_north = -a_go-(i-qrs)*a_ge;
			swmatrix[idx].score_northwest_max =WORST_SCORE;
			swmatrix[idx].score_northwest = WORST_SCORE;
			swmatrix[idx].score_west = WORST_SCORE;
		}
	}else{
		top_score=MAX3((swmatrix[qrs*(lena + 1)+dbs].score_north),(swmatrix[qrs*(lena + 1)+dbs].score_northwest),
				(swmatrix[qrs*(lena + 1)+dbs].score_west));
		top_score=MAX2(top_score,swmatrix[qrs*(lena + 1)+dbs].score_northwest_max);
		if(qrs>0){
			for (i = dbs+1; i < dbe + 1; i++) {
				int idx=qrs*(lena + 1)+i;
				swmatrix[idx].score_northwest = WORST_SCORE;
				swmatrix[idx].score_north = WORST_SCORE;
				swmatrix[idx].score_west = top_score-a_go-(i-dbs)*a_ge;
				if(i>dbs+1){
					swmatrix[idx].back_west = FROM_WEST_WEST;
				}else{
					swmatrix[idx].back_west = FROM_WEST_NORTHWEST;
				}
				swmatrix[idx].score_northwest_max =WORST_SCORE;
			}
		}
		if(dbs!=0 && dbs != x){
			for (i = qrs+1; i < qre + 1; i++) {
				int idx = i * (lena + 1 )+dbs;
				if(i>qrs+1){
					swmatrix[idx].back_north = FROM_NORTH_NORTH;
				}else{
					swmatrix[idx].back_north = FROM_NORTH_NORTHWEST;
				}
				swmatrix[idx].score_north = top_score-a_go-(i-qrs)*a_ge;
				swmatrix[idx].score_northwest_max =WORST_SCORE;
				swmatrix[idx].score_northwest = WORST_SCORE;
				swmatrix[idx].score_west = WORST_SCORE;
			}
		}
	}
	for (i = qrs; i < qre; i++) {
		for(j=dbs;j<dbe;j++){
			struct swcell *cell_nw, *cell_n, *cell_w, *cell_cur;
			cell_nw  = &swmatrix[i * (lena + 1) + j];
			cell_n   = cell_nw + 1;
			cell_w   = cell_nw + (lena + 1);
			cell_cur = cell_w + 1;

			ms = (db[j] == qr[i]) ? match : mismatch;

			/*
			 * northwest
			 */

			if (!revcmpl) {

				tmp  = cell_nw->score_northwest + ms;
				tmp2 = FROM_NORTHWEST_NORTHWEST;

				if (cell_nw->score_north + ms > tmp) {
					tmp  = cell_nw->score_north + ms;
					tmp2 = FROM_NORTHWEST_NORTH;

				}

				if (cell_nw->score_west + ms > tmp) {
					tmp  = cell_nw->score_west + ms;
					tmp2 = FROM_NORTHWEST_WEST;

				}
			} else {

				  tmp  = cell_nw->score_west + ms;
				  tmp2 = FROM_NORTHWEST_WEST;


				if (cell_nw->score_north + ms > tmp) {
				  tmp  = cell_nw->score_north + ms;
				  tmp2 = FROM_NORTHWEST_NORTH;

				}

				if (cell_nw->score_northwest + ms > tmp) {
				  tmp  = cell_nw->score_northwest + ms;
				  tmp2 = FROM_NORTHWEST_NORTHWEST;

				}
			}

			cell_cur->score_northwest = tmp;
			cell_cur->back_northwest  = tmp2;

			/**
			 * norhtwest max
			 */
			cell_cur->score_northwest_max = WORST_SCORE;
			cell_cur->back_northwest_max  = 0;

			/*
			 * north
			 */

			if (!revcmpl) {
				tmp  = cell_n->score_northwest - b_go - b_ge;
				tmp2 = FROM_NORTH_NORTHWEST;

				if (cell_n->score_north - b_ge > tmp) {
				  tmp  = cell_n->score_north - b_ge;
				  tmp2 = FROM_NORTH_NORTH;

				}
			} else {
				tmp  = cell_n->score_north - b_ge;
				tmp2 = FROM_NORTH_NORTH;


				if (cell_n->score_northwest - b_go - b_ge > tmp) {
				  tmp  = cell_n->score_northwest - b_go - b_ge;
				  tmp2 = FROM_NORTH_NORTHWEST;

				}
			}


			cell_cur->score_north = tmp;
			cell_cur->back_north  = tmp2;

			/*
			 * west
			 */

			if (!revcmpl) {
				tmp  = cell_w->score_northwest - a_go - a_ge;
				tmp2 = FROM_WEST_NORTHWEST;

				if (cell_w->score_west - a_ge > tmp) {
				  tmp  = cell_w->score_west - a_ge;
				  tmp2 = FROM_WEST_WEST;

				}
			} else {
				tmp  = cell_w->score_west - a_ge;
				tmp2 = FROM_WEST_WEST;

				if (cell_w->score_northwest - a_go - a_ge > tmp) {
				  tmp  = cell_w->score_northwest - a_go - a_ge;
				  tmp2 = FROM_WEST_NORTHWEST;
				}
			}

			cell_cur->score_west = tmp;
			cell_cur->back_west  = tmp2;
		}
	}
	return 0;
}
static int
fill_sub_sw_block_xy_del(int lena, int lenb, int qr_start,int qr_end,int db_start,int db_end,int dbsl, int dbel,
		int overlap_start,int x, bool is_in_overlap_region){
	int i,j;
	int dbs,dbe,qrs,qre;
	int ms,a_go, a_ge, b_go, b_ge, tmp;
	int8_t tmp2;
	int revcmpl=0;

	//int split_penalty=split_penalty_del_large;


	a_go = a_gap_open;
	a_ge = a_gap_ext;
	b_go = b_gap_open;
	b_ge = b_gap_ext;


	dbs=db_start;
	dbe=db_end;
	qrs=qr_start;
	qre=qr_end;
	if(is_in_overlap_region){
		int top_score;
		top_score=swmatrix[qrs*(lena + 1)+dbs].score_northwest;
		if(qrs!=0){
			for (i = dbs+1; i < dbe + 1; i++) {
				int idx=qrs*(lena + 1)+i;
				swmatrix[idx].score_northwest = WORST_SCORE;
				swmatrix[idx].score_north = WORST_SCORE;
				swmatrix[idx].score_west = top_score-a_go-(i-dbs)*a_ge;
				swmatrix[idx].score_northwest_max =WORST_SCORE;
				//swmatrix[idx].back_west = FROM_WEST_WEST;
				if(i>dbs+1){
					swmatrix[idx].back_west = FROM_WEST_WEST;
				}else{
					swmatrix[idx].back_west = FROM_WEST_NORTHWEST;
				}
			}
		}
		if(dbs!=x){
			for (i = qrs+1; i < qre + 1; i++) {
				int idx = i * (lena + 1 )+dbs;
				//swmatrix[idx].back_north = FROM_NORTH_NORTH;
				if(i>qrs+1){
					swmatrix[idx].back_north = FROM_NORTH_NORTH;
				}else{
					swmatrix[idx].back_north = FROM_NORTH_NORTHWEST;
				}
				swmatrix[idx].score_north =top_score -a_go-(i-qrs)*a_ge;
				swmatrix[idx].score_northwest_max =WORST_SCORE;
				swmatrix[idx].score_northwest = WORST_SCORE;
				swmatrix[idx].score_west = WORST_SCORE;
			}
		}
	}else{
		if(qrs!=0){
			for (i = dbs; i < dbe + 1; i++) {
				int idx=qrs*(lena + 1)+i;
				swmatrix[idx].score_northwest = WORST_SCORE;
				swmatrix[idx].score_north = WORST_SCORE;
				swmatrix[idx].score_west = WORST_SCORE;
				swmatrix[idx].score_northwest_max =WORST_SCORE;
			}
		}
		if(dbs!=x){
			for (i = qrs+1; i < qre + 1; i++) {
				int idx = i * (lena + 1 )+dbs;
				swmatrix[idx].score_northwest = WORST_SCORE;
				swmatrix[idx].score_north = WORST_SCORE;
				swmatrix[idx].score_west = WORST_SCORE;
				swmatrix[idx].score_northwest_max =WORST_SCORE;
			}
		}
	}
	struct max_t max_w;
	for (i = qrs; i < qre; i++) {
	  max_w.score=WORST_SCORE;
	  max_w.pos=-1;
	  max_w.multi=1;
	  //for(m=0;m<x+1;m++){
	  int m;
	  for(m=dbsl;m<(dbs-x)+overlap_start-1 && m <=dbel;m++){
	  //for(m=dbsl;m<overlap_start-2 && m <=dbel;m++){
	  //for(m=dbsl;m<overlap_start+(dbs-x)-1;m++){
		if(max_w.score<(&swmatrix[(i) * (lena + 1) + m])->score_northwest){
			max_w.score=(&swmatrix[(i) * (lena + 1) + m])->score_northwest;
			max_w.pos=m;
		}
	  }
	  for (j = dbs; j < dbe; j++) {
		struct swcell *cell_nw, *cell_n, *cell_w, *cell_cur, *cell_max_w;
		cell_nw  = &swmatrix[i * (lena + 1) + j];
		cell_n   = cell_nw + 1;
		cell_w   = cell_nw + (lena + 1);
		cell_cur = cell_w + 1;
		ms = (db[j] == qr[i]) ? match : mismatch;
		if((j-x)+overlap_start-1>0 && (j-x)+overlap_start<=dbel){//in overlap region
			cell_max_w=(&swmatrix[i * (lena + 1) + (j-x)+overlap_start-1]);
			if(cell_max_w->score_northwest >= max_w.score){
				//max_w.score=(&swmatrix[(i) * (lena + 1) + (j-x)+overlap_start-2])->score_northwest;
				//max_w.pos=j-x+overlap_start-2;
				if(cell_max_w->score_northwest == max_w.score){
					max_w.multi++;
				}else{
					max_w.multi=1;
				}
				max_w.score=(&swmatrix[(i) * (lena + 1) + (j-x)+overlap_start-1])->score_northwest;
				max_w.pos=j-x+overlap_start-1;
			}
		}
		/*
		 * northwest
		 */
		if (!revcmpl) {
			tmp  = cell_nw->score_northwest + ms;
			tmp2 = FROM_NORTHWEST_NORTHWEST;
			if ((i<lenb-JUMP_LEN_THRESH) && (cell_nw->score_northwest_max + ms > tmp)) {
				tmp  = cell_nw->score_northwest_max + ms;
				tmp2 = FROM_NORTHWEST_NORTHWEST_MAX;

			}
			if (cell_nw->score_north + ms > tmp) {
				tmp  = cell_nw->score_north + ms;
				tmp2 = FROM_NORTHWEST_NORTH;
			}
			if (cell_nw->score_west + ms > tmp) {
				tmp  = cell_nw->score_west + ms;
				tmp2 = FROM_NORTHWEST_WEST;
			}
		} else {

			tmp  = cell_nw->score_west + ms;
			tmp2 = FROM_NORTHWEST_WEST;

			if (cell_nw->score_north + ms > tmp) {
			  tmp  = cell_nw->score_north + ms;
			  tmp2 = FROM_NORTHWEST_NORTH;

			}
			if (cell_nw->score_northwest_max + ms > tmp) {
				tmp  = cell_nw->score_northwest_max + ms;
				tmp2 = FROM_NORTHWEST_NORTHWEST_MAX;

			}

			if (cell_nw->score_northwest + ms > tmp) {
			  tmp  = cell_nw->score_northwest + ms;
			  tmp2 = FROM_NORTHWEST_NORTHWEST;

			}

		}

		cell_cur->score_northwest = tmp;
		cell_cur->back_northwest  = tmp2;
		cell_cur->pos = -1;

		/*
		 * northwest_max
		 */
		  if (!revcmpl) {
			if(i<lenb-JUMP_LEN_THRESH){
				//tmp  = max_w_cur[i].score + ms + split_penalty;//+ 1;
				tmp = max_w.score + ms + split_penalty_del;
				tmp2 = FROM_NORTHWEST_MAX_NORTHWEST;
			}else{
				tmp =WORST_SCORE;
			}

		  } else {//waiting for modify
			 tmp  = max_w_cur[i].score + ms + split_penalty_del;//1;//3*(mismatch-match);
			 tmp2 = FROM_NORTHWEST_MAX_NORTHWEST;
		  }


		  cell_cur->score_northwest_max = tmp;
		  cell_cur->back_northwest_max  = tmp2;
		  //cell_cur->pos = max_w_cur[i].pos;
		  cell_cur->pos = max_w.pos;
		  cell_cur->multi_left = max_w.multi;


		  /*
		   * north
		   */
		  if (!revcmpl) {

			  tmp  = cell_n->score_northwest - b_go - b_ge;
			  tmp2 = FROM_NORTH_NORTHWEST;


			  if (cell_n->score_north - b_ge > tmp) {
				  tmp  = cell_n->score_north - b_ge;
				  tmp2 = FROM_NORTH_NORTH;

			  }


		  }else {
			  tmp  = cell_n->score_north - b_ge;
			  tmp2 = FROM_NORTH_NORTH;


			  if (cell_n->score_northwest - b_go - b_ge > tmp) {
				  tmp  = cell_n->score_northwest - b_go - b_ge;
				  tmp2 = FROM_NORTH_NORTHWEST;
			  }


		  }

		  cell_cur->score_north = tmp;
		  cell_cur->back_north  = tmp2;

		  /*
		   * west
		   */
		 if (!revcmpl) {

			tmp  = cell_w->score_northwest - a_go - a_ge;
			tmp2 = FROM_WEST_NORTHWEST;

			if (cell_w->score_west - a_ge > tmp) {
			  tmp  = cell_w->score_west - a_ge;
			  tmp2 = FROM_WEST_WEST;

			}

		 } else {
			tmp  = cell_w->score_west - a_ge;
			tmp2 = FROM_WEST_WEST;


			if (cell_w->score_northwest - a_go - a_ge > tmp) {
			  tmp  = cell_w->score_northwest - a_go - a_ge;
			  tmp2 = FROM_WEST_NORTHWEST;

			}

		 }
		 cell_cur->score_west = tmp;
		 cell_cur->back_west  = tmp2;

	  }// for j

	}
	return 0;
}
static int
fill_sub_sw_block_xy_ins(int lena, int lenb, int qr_start,int qr_end,int db_start,int db_end,int dbsl, int dbel,
		int overlap_start,int x){
	int i,j;
	int dbs,dbe,qrs,qre;
	int ms,a_go, a_ge, b_go, b_ge, tmp;
	int8_t tmp2;
	int revcmpl=0;

	//int split_penalty=split_penalty_ins;


	a_go = a_gap_open;
	a_ge = a_gap_ext;
	b_go = b_gap_open;
	b_ge = b_gap_ext;


	dbs=db_start;
	dbe=db_end;
	qrs=qr_start;
	qre=qr_end;

	int top_score;
	top_score=swmatrix[qrs*(lena + 1)+dbs].score_northwest;
	//init row
	if(qrs!=lenb){
		for (i = dbs+1; i < dbe + 1; i++) {
			int idx=qrs*(lena + 1)+i;
			swmatrix[idx].score_northwest = WORST_SCORE;
			swmatrix[idx].score_north = WORST_SCORE;
			swmatrix[idx].score_west = top_score-a_go-(i-dbs)*a_ge;
			if(i>dbs+1){
				swmatrix[idx].back_west = FROM_WEST_WEST;
			}else{
				swmatrix[idx].back_west = FROM_WEST_NORTHWEST;
			}

			swmatrix[idx].score_northwest_max =WORST_SCORE;
		}
	}
	//init colum
	if(dbs!=0){
		for (i = qrs+1; i < qre + 1; i++) {
			int idx = i * (lena + 1 )+dbs;
			if(i>qrs+1){
				swmatrix[idx].back_north = FROM_NORTH_NORTH;
			}else{
				swmatrix[idx].back_north = FROM_NORTH_NORTHWEST;
			}
			swmatrix[idx].score_north = top_score-a_go-(i-qrs)*a_ge;
			swmatrix[idx].score_northwest_max =WORST_SCORE;
			swmatrix[idx].score_northwest = WORST_SCORE;
			swmatrix[idx].score_west = WORST_SCORE;
		}
	}
	//initial max_n_cur matritx
	for(i=0;i<lena+1;i++){
		max_n_cur[i].score=WORST_SCORE;//max score of left part
		//max_n_cur[i].score_from=0;//max score from
		max_n_cur[i].pos=0;//max cell position
	}

	//fill in the second part of the matrix
	for (i = qrs; i < qre; i++) {
		for (j = dbs; j < dbe; j++) {

		struct swcell *cell_nw, *cell_n, *cell_w, *cell_cur;

		cell_nw  = &swmatrix[i * (lena + 1) + j];
		cell_n   = cell_nw + 1;
		cell_w   = cell_nw + (lena + 1);
		cell_cur = cell_w + 1;

		if(i>lenb+JUMP_LEN_THRESH && i > qrs){
			if(max_n_cur[j].score<(&swmatrix[(i-lenb-1) * (lena + 1) + j])->score_northwest){
				max_n_cur[j].score=(&swmatrix[(i-lenb-1) * (lena + 1) + j])->score_northwest;
				//max_n_cur[i].score_from=FROM_NORTHWEST_MAX_NORTHWEST;
				max_n_cur[j].pos=i-lenb-1;
			}
		}

		ms = (db[j] == qr[i]) ? match : mismatch;
		if(1){//i & j in the loop fills the uninitialized part of swmatrix. so here is
				  // x-1, not x

		/*
		 * northwest
		 */

		if (!revcmpl) {

			tmp  = cell_nw->score_northwest + ms;
			tmp2 = FROM_NORTHWEST_NORTHWEST;

			if (cell_nw->score_north + ms > tmp) {
				tmp  = cell_nw->score_north + ms;
				tmp2 = FROM_NORTHWEST_NORTH;

			}

			if (cell_nw->score_west + ms > tmp) {
				tmp  = cell_nw->score_west + ms;
				tmp2 = FROM_NORTHWEST_WEST;

			}

			if (cell_nw->score_northwest_max + ms > tmp) {
				tmp  = cell_nw->score_northwest_max + ms;
				tmp2 = FROM_NORTHWEST_NORTHWEST_MAX;

			}

		} else {

		  tmp  = cell_nw->score_west + ms;
		  tmp2 = FROM_NORTHWEST_WEST;

		  if (cell_nw->score_north + ms > tmp) {
			  tmp  = cell_nw->score_north + ms;
			  tmp2 = FROM_NORTHWEST_NORTH;

		  }

		  if (cell_nw->score_northwest + ms > tmp) {
			  tmp  = cell_nw->score_northwest + ms;
			  tmp2 = FROM_NORTHWEST_NORTHWEST;

		  }

		  if (cell_nw->score_northwest_max + ms > tmp) {
			  tmp  = cell_nw->score_northwest_max + ms;
			  tmp2 = FROM_NORTHWEST_NORTHWEST_MAX;

		  }
		}

		cell_cur->score_northwest = tmp;
		cell_cur->back_northwest  = tmp2;
		cell_cur->pos = -1;

		/*
		 * northwest_max
		 */
		  if (!revcmpl) {
			 //if(i < 2*lenb-JUMP_LEN_THRESH){
			 if(i > lenb){
				int jump_len;
				/*
				for(k=lenb+2;k<i;k++){
					cell_tmp=(&swmatrix[(k-lenb-1) * (lena + 1) + j]);
					jump_len= i - k +1;
					tmp = cell_tmp->score_northwest + ms + jump_len * (match-1)+split_penalty;
					//if(max_n_cur[j].score<(&swmatrix[(i-lenb-1) * (lena + 1) + j])->score_northwest){
					if(tmp > max_n_cur[j].score){
									//max_n_cur[j].score=(&swmatrix[(i-lenb-1) * (lena + 1) + j])->score_northwest;
										//max_n_cur[i].score_from=FROM_NORTHWEST_MAX_NORTHWEST;
						max_n_cur[j].score = tmp;
										max_n_cur[j].pos=k-lenb-1;
							}
				}*/
				jump_len= i - lenb - max_n_cur[j].pos;
				tmp  = max_n_cur[j].score + ms + jump_len * (match - 5 )+split_penalty_ins;// + split_penalty;// 2*(mismatch-match);
				//tmp  = max_n_cur[j].score;
				tmp2 = FROM_NORTHWEST_MAX_NORTHWEST;
			 }else{
				tmp  = WORST_SCORE;
				tmp2 = 0 ;
			 }

		  } else {//waiting for modify
			 tmp  = max_n_cur[j].score + ms + split_penalty_ins;//2*(mismatch-match);
			 tmp2 = FROM_NORTHWEST_MAX_NORTHWEST;
		  }

		  cell_cur->score_northwest_max = tmp;
		  cell_cur->back_northwest_max  = tmp2;
		  cell_cur->pos = max_n_cur[j].pos;

		/*
		 * north
		 */

		if (!revcmpl) {
			tmp  = cell_n->score_northwest - b_go - b_ge;
			tmp2 = FROM_NORTH_NORTHWEST;

			if (cell_n->score_north - b_ge > tmp) {
			  tmp  = cell_n->score_north - b_ge;
			  tmp2 = FROM_NORTH_NORTH;

			}
		} else {
			tmp  = cell_n->score_north - b_ge;
			tmp2 = FROM_NORTH_NORTH;

			if (cell_n->score_northwest - b_go - b_ge > tmp) {
			  tmp  = cell_n->score_northwest - b_go - b_ge;
			  tmp2 = FROM_NORTH_NORTHWEST;

			}
		}

		cell_cur->score_north = tmp;
		cell_cur->back_north  = tmp2;

		/*
		 * west
		 */
		if (!revcmpl) {
			tmp  = cell_w->score_northwest - a_go - a_ge;
			tmp2 = FROM_WEST_NORTHWEST;

			if (cell_w->score_west - a_ge > tmp) {
			  tmp  = cell_w->score_west - a_ge;
			  tmp2 = FROM_WEST_WEST;

			}
		}else{
			tmp  = cell_w->score_west - a_ge;
			tmp2 = FROM_WEST_WEST;

			if (cell_w->score_northwest - a_go - a_ge > tmp) {
			  tmp  = cell_w->score_northwest - a_go - a_ge;
			  tmp2 = FROM_WEST_NORTHWEST;

			}
		}

		cell_cur->score_west = tmp;
		cell_cur->back_west  = tmp2;
		}
	  }// for j
	}// for i
	return 0;
}
int
find_max_score(int lena, int lenb, int *iret, int *jret,int *backtrace_pos_len,int dbs, int dbe){
	int i,j,score,multi_aln_pos_num=0,q=0;
	struct swcell *cell_cur, *cell_return=NULL;
	score=2*WORST_SCORE;
	for(i=lenb,j=dbs;j<=dbe;j++){
		cell_cur  = &swmatrix[i * (lena + 1) + j];

		if(cell_cur->score_west>score){// && cell_cur->w_jumped){
			  score=cell_cur->score_west;
			  q=0;
			  jret[q++]=j;

			  multi_aln_pos_num=1;
			  cell_return=cell_cur;
		}else if (cell_cur->score_west==score){
			  multi_aln_pos_num++;

		}

		if(cell_cur->score_northwest>score){//  && cell_cur->nw_jumped){
			  score=cell_cur->score_northwest;
			  q=0;
			  jret[q++]=j;

			  multi_aln_pos_num=1;
			  cell_return=cell_cur;
		}else if (cell_cur->score_northwest==score){
			multi_aln_pos_num++;
		}

		if(cell_cur->score_north>score){//  && cell_cur->n_jumped){
			  score=cell_cur->score_north;
			  q=0;
			  jret[q++]=j;

			  multi_aln_pos_num=1;
			  cell_return=cell_cur;
		}else if (cell_cur->score_north==score){
			multi_aln_pos_num++;

		}
		cell_return->multi_right=multi_aln_pos_num;
	}
	*iret=lenb;
	*backtrace_pos_len=q;
	return score;

}
static int
split_sw_indel_square(hll* best1, int lena,int lenb, int *iret, int *jret, int *backtrace_pos_len, bool revcmpl, int x, int overlap_start){

	  int i, j;

	  int score_ins,score_del,a_go, a_ge, b_go, b_ge;

	  split_penalty_del=split_penalty_del_double;

	  score_ins = WORST_SCORE;
	  score_del = WORST_SCORE;
	  a_go = a_gap_open;
	  a_ge = a_gap_ext;
	  b_go = b_gap_open;
	  b_ge = b_gap_ext;
/*
	  for(i=0;i<(lena+1)*(2*lenb+1);i++){
	  	  swmatrix[i].score_northwest_max =WORST_SCORE;
	  	  swmatrix[i].score_northwest = WORST_SCORE;
	  	  swmatrix[i].score_north = WORST_SCORE;
	  	  swmatrix[i].score_west = WORST_SCORE;
	  }
*/
	   //clean up the last row of del_matrix
          for(i=x+1;i<=lena;i++){
                int idx;
                idx=lenb*(lena+1)+i;
                swmatrix[idx].score_northwest_max =WORST_SCORE;
                swmatrix[idx].score_northwest = WORST_SCORE;
                swmatrix[idx].score_north = WORST_SCORE;
                swmatrix[idx].score_west = WORST_SCORE;
          }
          //clean up the last row of del_matrix
          for(i=1;i<x;i++){
                 int idx;
                 idx=2*lenb*(lena+1)+i;
                 swmatrix[idx].score_northwest_max =WORST_SCORE;
                 swmatrix[idx].score_northwest = WORST_SCORE;
                 swmatrix[idx].score_north = WORST_SCORE;
                 swmatrix[idx].score_west = WORST_SCORE;
           }

	  //initialize the first row

	  for (j = 0; j < lena + 1; j++) {
		  init_cell(j);
	  }

	  //initialize the first column
	  for (i = 1; i < lenb; i++) {

		int idx = i * (lena + 1);
		swmatrix[idx].back_north = FROM_NORTH_NORTH;
		swmatrix[idx].score_north = -b_go-i*b_ge;
		swmatrix[idx].score_northwest = WORST_SCORE;
		swmatrix[idx].score_northwest_max = WORST_SCORE;
		swmatrix[idx].score_west = WORST_SCORE;
	  }

	  for(j = 0; j < lena + 1; j++){
		  init_cell(lenb*(lena+1)+j);
	  }
	  for(i = 1; i < lenb + 1; i++){
		int idx = i * (lena + 1)+x;
		swmatrix[idx].back_north = FROM_NORTH_NORTH;
		swmatrix[idx].score_north = -b_go-i*b_ge;
		swmatrix[idx].score_northwest_max =WORST_SCORE;
		swmatrix[idx].score_northwest = WORST_SCORE;
		swmatrix[idx].score_west = WORST_SCORE;
	  }
	  for(i = lenb+1; i < 2*lenb+1; i++){
		  int idx = i * (lena + 1);
		  swmatrix[idx].back_north = FROM_NORTH_NORTH;
		  swmatrix[idx].score_northwest_max = WORST_SCORE;
		  swmatrix[idx].score_north = -b_go-(i-lenb)*b_ge;
		  swmatrix[idx].score_northwest = WORST_SCORE;
		  swmatrix[idx].score_west = WORST_SCORE;
	  }

	  //return 0;
	  /*
	   * Figure out our band.
	   *   We can actually skip computation of a significant number of
	   *   cells, which could never be part of an alignment corresponding
	   *   to our threshhold score.
	   */
	  hll* temp;
	  int qrs,dbs,qre,dbe,qrs_ins,dbs_del,qre_ins,dbe_del;
	  int dbsl,dbel,qrsl,qrel;
	  bool switch1=false,switch2=false;
	  switch2=true;
	  switch1=false;
	  /*fill part1*/
	  if(best1){
		  temp=best1;
		  qrs=0;
		  qre=temp->seq2start;
		  //dbs=dbs-lenb>0?dbs-lenb:0;
		  dbs=0;//(temp->seq1start-lenb)>0?(temp->seq1start-lenb):0;
		  dbe=temp->seq1start;
	  }else{
		  return 0;// no aligned kmer
	  }
	  fill_sub_sw_block_x(lena,lenb,x,qrs,qre,dbs,dbe);

	  qrsl=qrs;
	  qrel=qre;
	  dbsl=dbs;
	  dbel=dbe;
	  /*fill part2*/

	  dbe=best1->seq1start;
	  dbs=0;//dbe-lenb>0?dbe-lenb:0;
	  qrs_ins=qrs+lenb;
	  qre_ins=qre+lenb;
	  fill_sub_sw_block_xy_ins(lena,lenb,qrs_ins,qre_ins,dbs,dbe,qrsl,qrel,overlap_start,x);
	  /*fill part3*/
	  dbs_del=dbs+x;
	  dbe_del=dbe+x;
	  fill_sub_sw_block_xy_del(lena,lenb,qrs,qre,dbs_del,dbe_del,dbsl,dbel,overlap_start,x,true);

	  //print_swmatrix( lena, lenb);
	  for (temp = best1; temp; temp = temp->nextptr) {
#ifdef DEBUG_OUTPUT_LOG
	  	      printf("(%d %d)=",temp->seq2start, temp->seq2end);
	  	      printf("(%d %d) %f %f\n",temp->seq1start, temp->seq1end, temp->score,temp->scoreSoFar);
#endif
	  	      /*fill diagonal part1*/
	  	      dbs=temp->seq1start;
	  		  dbe=temp->seq1end;
	  		  qrs=temp->seq2start;
	  		  qre=temp->seq2end;

	  	      fill_diagonal(lena,lenb,qrs,qre,dbs,dbe);
	  	      //print_swmatrix( lena, lenb);

	  	      /*fill diagonal part2*/
	  	      qrs_ins=qrs+lenb;
	  	      qre_ins=qre+lenb;
	  	      fill_diagonal(lena,lenb,qrs_ins,qre_ins,dbs,dbe);

	  	      /*fill diagonal part3*/
	  	      dbs_del=dbs+x;
	  	      dbe_del=dbe+x;
	  	      fill_diagonal(lena,lenb,qrs,qre,dbs_del,dbe_del);
	  	      //print_swmatrix( lena, lenb);
	  	      if(temp->nextptr){
	  	    	  /*fill part1*/
	  	    	  dbs=temp->seq1end;
				  dbe=temp->nextptr->seq1start;
				  qrs=temp->seq2end;
				  qre=temp->nextptr->seq2start;
				  qrsl=qrs;
				  qrel=qre;
				  dbsl=dbs;
				  dbel=dbe;
				  fill_sub_sw_block_x(lena,lenb,x,qrs,qre,dbs,dbe);
				  /*fill part2*/
				  qrs_ins=qrs+lenb;
				  qre_ins=qre+lenb;
				  fill_sub_sw_block_xy_ins(lena,lenb,qrs_ins,qre_ins,dbs,dbe,qrsl,qrel,overlap_start,x);

				  /*fill part3*/
				  dbs_del=dbs+x;
				  dbe_del=dbe+x;
				  fill_sub_sw_block_xy_del(lena,lenb,qrs,qre,dbs_del,dbe_del,dbsl,dbel,overlap_start,x,true);

	  	      }else{//last block
	  	    	  if(temp->seq2end<lenb){
	  	    		  /*fill part1*/
					  qrs=temp->seq2end;
					  qre=lenb;
					  dbs=temp->seq1end;
					  dbe=x;//dbs+lenb<x?dbs+lenb:x;
					  qrsl=qrs;
					  qrel=qre;
					  dbsl=dbs;
					  dbel=dbe;
					  fill_sub_sw_block_x(lena,lenb,x,qrs,qre,dbs,dbe);
					  /*fill part2*/
					  qrs_ins=qrs+lenb;
					  qre_ins=qre+lenb;
					  //print_swmatrix( lena, lenb);
					  fill_sub_sw_block_xy_ins(lena,lenb,qrs_ins,qre_ins,dbs,dbe,qrsl,qrel,overlap_start,x);
					  //print_swmatrix( lena, lenb);
					  /*fill part3*/
					  dbs_del=dbs+x;
					  dbe_del=dbe+x;
					  fill_sub_sw_block_xy_del(lena,lenb,qrs,qre,dbs_del,dbe_del,dbsl,dbel,overlap_start,x,true);
	  	    	  }
	  	      }

	  }
	  //print_swmatrix( lena, lenb);
	  int iret_ins=0, jret_ins=0, iret_del=0, jret_del=0, bpl_ins=0,bpl_del=0;
	  score_ins=find_max_score(lena, 2*lenb,&iret_ins,&jret_ins,&bpl_ins,dbs,dbe);
	  score_del=find_max_score(lena, lenb,&iret_del,&jret_del,&bpl_del,dbs_del,dbe_del);
	  if(score_ins>score_del){
		  *iret=iret_ins;
		  *jret=jret_ins;
		  *backtrace_pos_len=bpl_ins;
	  }else{
		  *iret=iret_del;
		  *jret=jret_del;
		  *backtrace_pos_len=bpl_del;
	  }

	  return score_ins>score_del?score_ins:score_del;
}
static int
split_sw_del_square(hll* best1,hll* best2, int lena,int lenb, int *iret, int *jret, int *backtrace_pos_len, bool revcmpl,
		int x, int overlap_start, bool cluster_supported){

	  int i, j;

	  int score, a_go, a_ge, b_go, b_ge;

	  //int split_penalty;
	  int overlap_len=x-overlap_start+1;

	  if(cluster_supported){
	  	split_penalty_del=split_penalty_del_large;
	  }else{
		split_penalty_del = split_penalty_del_double;
	  }
	  score = WORST_SCORE;
	  a_go = a_gap_open;
	  a_ge = a_gap_ext;
	  b_go = b_gap_open;
	  b_ge = b_gap_ext;
/*
	  for(i=0;i<(lena+1)*(lenb+1);i++){
	  	  swmatrix[i].score_northwest_max =WORST_SCORE;
	  	  swmatrix[i].score_northwest = WORST_SCORE;
	  	  swmatrix[i].score_north = WORST_SCORE;
	  	  swmatrix[i].score_west = WORST_SCORE;
	  }
*/
	  //clean up the last row of del_matrix
          for(i=1;i<=lena;i++){
                int idx;
                idx=lenb*(lena+1)+i;
                swmatrix[idx].score_northwest_max =WORST_SCORE;
                swmatrix[idx].score_northwest = WORST_SCORE;
                swmatrix[idx].score_north = WORST_SCORE;
                swmatrix[idx].score_west = WORST_SCORE;
          }
	  //initialize the first row

	  for (j = 0; j < lena + 1; j++) {
	    init_cell(j);
	  }

	  //initialize the first column
	  for (i = 1; i < lenb + 1; i++) {
	    int idx = i * (lena + 1);
	    swmatrix[idx].back_north = FROM_NORTH_NORTH;
	    swmatrix[idx].score_north = -a_go-i*a_ge;
	    swmatrix[idx].score_northwest_max =WORST_SCORE;
	    swmatrix[idx].score_northwest = WORST_SCORE;
	    swmatrix[idx].score_west = WORST_SCORE;
	  }
/*
	  for (i = 1; i < lenb + 1; i++) {
		int idx = i * (lena + 1)+x;
		swmatrix[idx].back_north = FROM_NORTH_NORTH;
		swmatrix[idx].score_north = -a_go-i*a_ge;
		swmatrix[idx].score_northwest_max =WORST_SCORE;
		swmatrix[idx].score_northwest = WORST_SCORE;
		swmatrix[idx].score_west = WORST_SCORE;
	  }
*/

	  /*
	   * Figure out our band.
	   *   We can actually skip computation of a significant number of
	   *   cells, which could never be part of an alignment corresponding
	   *   to our threshhold score.
	   */
	  hll* temp;
	  int qrs,dbs,qre,dbe;
	  int dbsl,dbel,qrsl;
	  bool switch1=false,switch2=false;
	  switch2=true;
	  switch1=false;
	  /*fill x-region*/
	  if(best1){
		  temp=best1;
		  qrs=0;
		  qre=temp->seq2start;
		  dbs=(temp->seq1start-lenb)>0?(temp->seq1start-lenb):0;
		  dbe=temp->seq1start;
	  }else if(best2){
		  temp=best2;
		  qrs=0;
		  qre=temp->seq2start;
		  dbs=0;
		  dbe=x;
	  }else{
		  return 0;// no aligned kmer
	  }
	  fill_sub_sw_block_x(lena,lenb,x,qrs,qre,dbs,dbe);
	  //print_swmatrix( lena, lenb);
	  dbsl=dbs;
	  dbel=dbe;
	  qrsl=qre;

	  /*Smith Waterman*/

	  for (temp = best1; temp; temp = temp->nextptr) {
		  //printf("(%d %d)=",temp->seq2start, temp->seq2end);
	     // printf("(%d %d) %f %f\n",temp->seq1start, temp->seq1end, temp->score,temp->scoreSoFar);

	      dbs=temp->seq1start;
		  dbe=temp->seq1end;
		  qrs=temp->seq2start;
		  qre=temp->seq2end;
		  dbsl=dbs;
	      fill_diagonal(lena,lenb,qrs,qre,dbs,dbe);
	      if(switch1 && dbsl>=overlap_start){//this cannot happen
			  dbs+=overlap_len;
			  dbe+=overlap_len;
			  fill_diagonal(lena,lenb,qrs,qre,dbs,dbe);
		  }
	      //print_swmatrix( lena, lenb);
	      if(temp->nextptr){
			  dbs=temp->seq1end;
			  dbe=temp->nextptr->seq1start;
			  qrs=temp->seq2end;
			  qre=temp->nextptr->seq2start;
			  fill_sub_sw_block_x(lena,lenb,x,qrs,qre,dbs,dbe);
			  dbsl=dbs;
			  dbel=dbe;
			  qrsl=qrs;
			  /*
			  if(dbe<=x){
				  fill_sub_sw_block_x(lena,lenb,qrs,qre,dbs,dbe,false);
			  }else if(dbs>x){
				  fill_sub_sw_block_x(lena,lenb,qrs,qre,dbs,dbe,false);
			  }else{
				  fill_sub_sw_block_xy_del(lena,lenb,qrs,qre,dbs,dbe,overlap_start,x);
				  print_swmatrix( lena, lenb);
			  }*/

	      }else{//last block

			  if(best2){
				  qrs=temp->seq2end;
				  qre=best2->seq2start;
				  dbs=temp->seq1end;
				  dbe=dbs+lenb<x?dbs+lenb:x;
			  }else{
				  qrs=temp->seq2end;
				  qre=lenb;
				  dbs=temp->seq1end;
				  dbe=dbs+lenb<x?dbs+lenb:x;
			  }
			  dbsl=dbs;
			  dbel=dbe;
			  qrsl=qrs;
	    	  fill_sub_sw_block_x(lena,lenb,x,qrs,qre,dbs,dbe);
	      }

	  }
	  /*fill y-region*/
	  for (i = 1; i < lenb + 1; i++) {
                int idx = i * (lena + 1)+x;
                swmatrix[idx].back_north = FROM_NORTH_NORTH;
                swmatrix[idx].score_north = -a_go-i*a_ge;
                swmatrix[idx].score_northwest_max =WORST_SCORE;
                swmatrix[idx].score_northwest = WORST_SCORE;
                swmatrix[idx].score_west = WORST_SCORE;
          }

	  if(best1 && best2){

		  dbe=best2->seq1start;
		  dbs=dbe-lenb>x?dbe-lenb:x;
		  qrs=qrsl;
		  qre=best2->seq2start;

	  }else if(best2){

		  dbe=best2->seq1start;
		  dbs=dbe-lenb>x?dbe-lenb:x;
		  qrs=0;
		  qre=best2->seq2start;
	  }else if (best1){
		  dbs=x;
		  dbe=lena;
		  qrs=qrsl;
		  qre=lenb;
	  }
	  fill_sub_sw_block_xy_del(lena,lenb,qrs,qre,dbs,dbe,dbsl,dbel,overlap_start,x,false);

	  for (temp = best2; temp; temp = temp->nextptr) {

#ifdef DEBUG_OUTPUT_LOG
	  	      printf("(%d %d)=",temp->seq2start, temp->seq2end);
	  	      printf("(%d %d) %f %f\n",temp->seq1start, temp->seq1end, temp->score,temp->scoreSoFar);
#endif
	  	      dbs=temp->seq1start;
	  		  dbe=temp->seq1end;
	  		  qrs=temp->seq2start;
	  		  qre=temp->seq2end;
	  		  dbel=dbe;
	  	      fill_diagonal(lena,lenb,qrs,qre,dbs,dbe);
	  	      if(switch2 && dbel<=x+overlap_len){
	  	    	  dbs-=overlap_len;
	  	    	  dbe-=overlap_len;
	  	    	  fill_diagonal(lena,lenb,qrs,qre,dbs,dbe);
	  	      }

	  	      //print_swmatrix( lena, lenb);
	  	      if(temp->nextptr){
	  			  dbs=temp->seq1end;
	  			  dbe=temp->nextptr->seq1start;
	  			  qrs=temp->seq2end;
	  			  qre=temp->nextptr->seq2start;
	  			  //fill_sub_sw_block_x(lena,lenb,qrs,qre,dbs,dbe,false);
	  			  if(switch2 && dbel<=x+overlap_len){
	  				  dbs-=overlap_len;
	  				  dbe-=overlap_len;
	  				  fill_sub_sw_block_x(lena,lenb,x,qrs,qre,dbs,dbe);
	  				  dbsl=dbs;
	  				  dbel=dbe;
	  				  dbs+=overlap_len;
	  				  dbe+=overlap_len;
	  				  fill_sub_sw_block_xy_del(lena,lenb,qrs,qre,dbs,dbe,dbsl,dbel,overlap_start,x,true);
	  			  }else{
	  				  fill_sub_sw_block_x(lena,lenb,x,qrs,qre,dbs,dbe);
	  			  }
	  			  /*
	  			  if(dbe<=x){
	  				  fill_sub_sw_block_x(lena,lenb,qrs,qre,dbs,dbe,false);
	  			  }else if(dbs>x){
	  				  fill_sub_sw_block_x(lena,lenb,qrs,qre,dbs,dbe,false);
	  			  }else{
	  				  fill_sub_sw_block_xy_del(lena,lenb,qrs,qre,dbs,dbe,overlap_start,x);
	  				  print_swmatrix( lena, lenb);
	  			  }*/

	  	      }else{//last block

	  	    	  qrs=temp->seq2end;
	  	    	  qre=lenb;
	  	    	  dbs=temp->seq1end;
	  	    	  dbe=dbs+lenb<lena?dbs+lenb:lena;

	  	    	  if(temp->seq2end<lenb){
					  if(switch2 && dbel<=x+overlap_len){
						  dbs-=overlap_len;
						  dbe-=overlap_len;

						  fill_sub_sw_block_x(lena,lenb,x,qrs,qre,dbs,dbe);
						  dbsl=dbs;
						  dbel=dbe;
						  dbs+=overlap_len;
						  dbe+=overlap_len;

						  fill_sub_sw_block_xy_del(lena,lenb,qrs,qre,dbs,dbe,dbsl,dbel,overlap_start,x,true);
					  }else{
						  fill_sub_sw_block_x(lena,lenb,x,qrs,qre,dbs,dbe);
					  }
	  	    	  }
	  	      }

	  }
	  //print_swmatrix( lena, lenb);
	  //score=find_max_score(lena, lenb,iret,jret,backtrace_pos_len,dbsl,dbel);
	  score=find_max_score(lena, lenb,iret,jret,backtrace_pos_len,dbs,dbe);
	  return score;


}

/*
 * Fill in the backtrace in order to do a pretty printout.
 *
 * Returns the beginning matrix cell (i, j) in 'sfr->read_start' and
 * 'sfr->genome_start'.
 *
 * The return value is the first valid offset in the backtrace buffer.
 */
static int
do_backtrace_ins(int lena, int i, int* backtrace_pos, int backtrace_pos_len, struct sw_full_results *sfr,  int xy_gap_size)
{
	struct swcell *cell;
	int k,j,ii,fromscore;
	int from, pos;
	int lenb;

	lenb=i;
	for(ii=0;ii<backtrace_pos_len;ii++){

		i=lenb;
		j=backtrace_pos[ii];//+1;
		gap_size=-1;
		cell = &swmatrix[i * (lena + 1) + j];

		from = (cell->back_northwest);
		fromscore = cell->score_northwest;
		if (cell->score_west > fromscore) {
			from = (cell->back_west);
			fromscore = cell->score_west;
		}
		if (cell->score_north > fromscore){
			from = (cell->back_north);
			fromscore = cell->score_north;
		}

		if(cell->score_northwest_max > fromscore){
			from = (cell->back_northwest_max);
		}
		assert(from != 0);

		/* fill out the backtrace */
		k = (dblen + qrlen) - 1;
		memset(backtrace,0,dblen+qrlen);
		while (i >= 0 && j >= 0) {
			assert(k >= 0);
			pos=cell->pos;
			//printf("%i,%i:%i,%i,%i,%i\n",i,j,cell->score_northwest,cell->score_north,cell->score_west,cell->score_northwest_max);
			cell = NULL;
			switch (from) {

			case FROM_NORTH_NORTH:
			case FROM_NORTH_NORTHWEST:
			case FROM_NORTH_NORTHWEST_MAX:

				backtrace[k] = BACK_DELETION;
				k--;
				sfr->read_start = i--;
				break;

			case FROM_WEST_WEST:
			case FROM_WEST_NORTHWEST:
			case FROM_WEST_NORTHWEST_MAX:
				backtrace[k] = BACK_INSERTION;
				k--;

				sfr->genome_start = j--;
				break;

			case FROM_NORTHWEST_NORTH:
			case FROM_NORTHWEST_NORTHWEST:
			case FROM_NORTHWEST_WEST:
			case FROM_NORTHWEST_NORTHWEST_MAX:
				backtrace[k] = BACK_MATCH_MISMATCH;
				k--;
				if (db[j] == qr[i])
					sfr->matches++;
				else
					sfr->mismatches++;
				sfr->read_start = i--;
				sfr->genome_start = j--;
				break;

			case FROM_NORTHWEST_MAX_NORTH:
			case FROM_NORTHWEST_MAX_NORTHWEST:
			case FROM_NORTHWEST_MAX_WEST:

				backtrace[k]=BACK_MATCH_MISMATCH;
				k--;
				backtrace[k] =BACK_INSERTION_N;
				gap_size = i-pos-1;

				if(i > 2*rlen - TAIL_LEN_THRESH_INS || pos < TAIL_LEN_THRESH_INS){
					//printf("%i:%i\n",2*rlen - TAIL_LEN_THRESH_INS,TAIL_LEN_THRESH_INS);
					//printf("%i:%i\n",i,pos);
					sfr->score=WORST_SCORE;
				}
				if(gap_size-rlen/2<=0){
					k=-1;
					goto nextloop;
				}
				k--;
				i = pos;
				sfr->genome_start = j--;
				break;
			default:
				fprintf(stderr, "INTERNAL ERROR: last_cell1 = %d\n", from);
				exit(0);
			}


			/* continue backtrace (nb: i and j have already been changed) */

			if(i==0 || i==lenb/2){
				break;
			}
			cell = &swmatrix[i * (lena + 1) + j];

			switch (from) {
			case FROM_NORTH_NORTH:
				from = cell->back_north;
				break;
			case FROM_NORTH_NORTHWEST:
				from = cell->back_northwest;
				break;
			case FROM_NORTH_NORTHWEST_MAX:
				from = cell->back_northwest_max;

				break;


			case FROM_WEST_WEST:
				from = cell->back_west;
				break;
			case FROM_WEST_NORTHWEST:
				from = cell->back_northwest;
				break;
			case FROM_WEST_NORTHWEST_MAX:
				from = cell->back_northwest_max;

				break;


			case FROM_NORTHWEST_NORTH:
				from = cell->back_north;
				break;
			case FROM_NORTHWEST_NORTHWEST:
				from = cell->back_northwest;
				break;
			case FROM_NORTHWEST_WEST:
				from = cell->back_west;
				break;
			case FROM_NORTHWEST_NORTHWEST_MAX:
				from = cell->back_northwest_max;
				//lscore=last_cell->score_northwest_max;
				break;


			case FROM_NORTHWEST_MAX_NORTH:
				from = cell->back_north;
				break;
			case FROM_NORTHWEST_MAX_NORTHWEST:
				from = cell->back_northwest;
				break;
			case FROM_NORTHWEST_MAX_WEST:
				from = cell->back_west;
				break;

			default:
				fprintf(stderr, "INTERNAL ERROR: last_cell2 = %d\n", from);
				assert(0);
			}

			if (from == 0){
				break;
			}
		}
nextloop:
		if(gap_size<0){
			break;
		}
		if(gap_size+xy_gap_size>0){
			gap_size+=xy_gap_size;
			break;
		}
	}

	return (k + 1);
}
static int
do_backtrace(int lena, int i, int* backtrace_pos, int backtrace_pos_len, struct sw_full_results *sfr,  int xy_gap_size)
{
	struct swcell *cell;
	int k=0,j,ii,fromscore;
	int from;
	int pos, multi_left, multi_right;
	int lenb;

	lenb=i;
	for(ii=0;ii<backtrace_pos_len;ii++){

		i=lenb;
		//j=backtrace_pos[ii]+1;
		j=backtrace_pos[ii];
		gap_size=-1;
		cell = &swmatrix[i * (lena + 1) + j];
		multi_right = cell-> multi_right;
		sfr->multi_aln=multi_right;
		from = (cell->back_northwest);
		fromscore = cell->score_northwest;
		//continue;
		if (cell->score_west > fromscore) {
			from = (cell->back_west);
			fromscore = cell->score_west;

		}
		if (cell->score_north > fromscore){
			from = (cell->back_north);
			fromscore = cell->score_north;

		}

		if(cell->score_northwest_max > fromscore){
			from = (cell->back_northwest_max);
			fromscore = cell->score_northwest_max;

		}
		assert(from != 0);

		/* fill out the backtrace */
		k = (dblen + qrlen) - 1;
		memset(backtrace,0,dblen+qrlen);
		while (i >= 0 && j >= 0) {
			assert(k >= 0);
			pos=cell->pos;
			multi_left=cell->multi_left;
			cell = NULL;
			//printf("%i,%i:%i\n",i,j,from);
			/* common operations first */
			switch (from) {

			case FROM_NORTH_NORTH:
			case FROM_NORTH_NORTHWEST:
			case FROM_NORTH_NORTHWEST_MAX:

				backtrace[k] = BACK_DELETION;
				k--;
				sfr->read_start = i--;
				break;

			case FROM_WEST_WEST:
			case FROM_WEST_NORTHWEST:
			case FROM_WEST_NORTHWEST_MAX:
				backtrace[k] = BACK_INSERTION;
				k--;

				sfr->genome_start = j--;
				break;

			case FROM_NORTHWEST_NORTH:
			case FROM_NORTHWEST_NORTHWEST:
			case FROM_NORTHWEST_WEST:
			case FROM_NORTHWEST_NORTHWEST_MAX:
				backtrace[k] = BACK_MATCH_MISMATCH;
				k--;
				if (db[j] == qr[i])
					sfr->matches++;
				else
					sfr->mismatches++;
				sfr->read_start = i--;
				sfr->genome_start = j--;
				break;

			case FROM_NORTHWEST_MAX_NORTH:
			case FROM_NORTHWEST_MAX_NORTHWEST:
			case FROM_NORTHWEST_MAX_WEST:

				backtrace[k]=BACK_MATCH_MISMATCH;
				k--;

				backtrace[k] =BACK_N;
				gap_size = j-pos-1;
				if(gap_size+xy_gap_size<=0){
					k=-1;
					goto nextloop;
				}
				k--;
				j = pos;
				//printf("mleft:%i\n",multi_left);
				//printf("mright:%i\n",multi_right);
				sfr->multi_aln = multi_left * multi_right;
				sfr->read_start = i--;
				break;

			default:
				fprintf(stderr, "INTERNAL ERROR: last_cell1 = %d\n", from);
				exit(0);
			}

			/* continue backtrace (nb: i and j have already been changed) */
			if(i==0){
				break;
			}
			cell = &swmatrix[i * (lena + 1) + j];

			switch (from) {
			case FROM_NORTH_NORTH:
				from = cell->back_north;
				break;
			case FROM_NORTH_NORTHWEST:
				from = cell->back_northwest;
				break;
			case FROM_NORTH_NORTHWEST_MAX:
				from = cell->back_northwest_max;
				break;


			case FROM_WEST_WEST:
				from = cell->back_west;
				break;
			case FROM_WEST_NORTHWEST:
				from = cell->back_northwest;
				break;
			case FROM_WEST_NORTHWEST_MAX:
				from = cell->back_northwest_max;
				break;


			case FROM_NORTHWEST_NORTH:
				from = cell->back_north;
				break;
			case FROM_NORTHWEST_NORTHWEST:
				from = cell->back_northwest;
				break;
			case FROM_NORTHWEST_WEST:
				from = cell->back_west;
				break;
			case FROM_NORTHWEST_NORTHWEST_MAX:
				from = cell->back_northwest_max;
				break;


			case FROM_NORTHWEST_MAX_NORTH:
				from = cell->back_north;
				break;
			case FROM_NORTHWEST_MAX_NORTHWEST:
				from = cell->back_northwest;
				break;
			case FROM_NORTHWEST_MAX_WEST:
				from = cell->back_west;
				break;


			default:
				fprintf(stderr, "INTERNAL ERROR: last_cell2 = %d\n", from);
				assert(0);
			}

			if (from == 0){
				break;
			}
		}
nextloop:
		if(gap_size<0){
			break;
		}
		if(gap_size+xy_gap_size>0){
			gap_size+=xy_gap_size;
			break;
		}
	}

	return (k + 1);
}

/*
 * Pretty print our alignment of 'db' and 'qr' in 'dbalign' and 'qralign'.
 *
 * i, j represent the beginning cell in the matrix.
 * k is the first valid offset in the backtrace buffer.
 */
static void
pretty_print_dup(struct sw_full_results *sfr, int k, char* cigar, int x, int xy_gap_size, int end_pos)
{
	char *d, *q;
	int i,j,l,n, done;
	char tmp_cigar[200];
	char tmp_cigar_N[100];
	char tmp_cigar_left[100];
	char tmp_cigar_right[100];
	int gstart, rstart;

	d = dbalign;
	q = qralign;
	i=sfr->read_start-1;
	j=sfr->genome_start-1;
	rstart=i;
        gstart=j;

	done = 0;
	sfr->is_split_align=0;

	cigar[0]='\0';
	tmp_cigar_left[0]='\0';
	tmp_cigar_N[0]='\0';
	tmp_cigar_right[0]='\0';
	int tmp_back=-1;
	int n_m,n_d,n_i,n_n,n_left;
	int head_i,head_d,head_m;
	int total_i,total_d,total_m;
	n=0;
	n_m=0;
	n_d=0;
	n_i=0;
	n_n=0;
	total_i=0;
	total_d=0;
	total_m=0;
	head_i=0;
	head_d=0;
	head_m=0;
	n_left=0;
	int head_len=0,tail_len=0;
	for (l = k; l < (dblen + qrlen); l++) {

		switch (backtrace[l]) {
		case BACK_DELETION:

			n_d++;
			total_i++;
			strcpy(tmp_cigar,cigar);
			if(tmp_back==BACK_DELETION){

			}else{
				if(tmp_back== BACK_INSERTION){
					sprintf(cigar,"%s%i%c",tmp_cigar,n_i,'D');
					n_i=0;
				}
				if(tmp_back== BACK_MATCH_MISMATCH){
					sprintf(cigar,"%s%i%c",tmp_cigar,n_m,'M');
					n_m=0;
				}
				assert(tmp_back!=BACK_N);
			}
			tmp_back= BACK_DELETION;

			*d++ = '-';

			break;

		case BACK_INSERTION:
			n_i++;
			total_d++;
			strcpy(tmp_cigar,cigar);
			if(tmp_back==BACK_INSERTION){

			}else{
				if(tmp_back==BACK_DELETION){
					sprintf(cigar,"%s%i%c",tmp_cigar,n_d,'I');
					n_d=0;
				}
				if(tmp_back== BACK_MATCH_MISMATCH){
					sprintf(cigar,"%s%i%c",tmp_cigar,n_m,'M');
					n_m=0;
				}
				assert(tmp_back!=BACK_N);

			}
			tmp_back= BACK_INSERTION;

			*d++ = base_translate(db[j++], false);

			break;

		case BACK_N:

			strcpy(tmp_cigar,cigar);
			if(tmp_back==BACK_MATCH_MISMATCH){
				sprintf(cigar,"%s%i%c",tmp_cigar,n_m,'M');
			}else{
				if(tmp_back== BACK_INSERTION){
					sprintf(cigar,"%s%i%c",tmp_cigar,n_i,'D');

				}else if(tmp_back==BACK_DELETION){
					sprintf(cigar,"%s%i%c",tmp_cigar,n_d,'I');

				}else{

				}

			}
			strcpy(tmp_cigar_left,cigar);
			head_i=total_i;
			head_d=total_d;
			head_m=total_m;
			total_m=0;
			total_i=0;
			total_d=0;
			sfr->is_split_align=1;

			cigar[0]='\0';
			n_m=0;n_i=0;n_d=0;tmp_back=-1;


			*d++ = base_translate(db[j++], false);

			break;
		case BACK_MATCH_MISMATCH:
			n_m++;
			n_left++;
			total_m++;
			strcpy(tmp_cigar,cigar);
			if(tmp_back==BACK_MATCH_MISMATCH){

			}else{
				if(tmp_back== BACK_INSERTION){
					sprintf(cigar,"%s%i%c",tmp_cigar,n_i,'D');
					n_i=0;
				}
				if(tmp_back==BACK_DELETION){
					sprintf(cigar,"%s%i%c",tmp_cigar,n_d,'I');
					n_d=0;
				}
				assert(tmp_back!=BACK_N);


			}
			tmp_back= BACK_MATCH_MISMATCH;

			*d++ = base_translate(db[j++], false);

			break;

		default:
			done = 1;
		}

		if (done)
			break;
	}


	*d='\0';
	strcpy(tmp_cigar,cigar);
	if(tmp_back==BACK_MATCH_MISMATCH){
		sprintf(cigar,"%s%i%c",tmp_cigar,n_m,'M');
	}else{
		if(tmp_back== BACK_INSERTION){
			sprintf(cigar,"%s%i%c",tmp_cigar,n_i,'D');

		}
		if(tmp_back==BACK_DELETION){
			sprintf(cigar,"%s%i%c",tmp_cigar,n_d,'I');

		}
		assert(tmp_back!=BACK_N);


	}

	sprintf(tmp_cigar_N,"%i%c",gap_size,'N');
        //strcpy(tmp_cigar_right,cigar);
        sprintf(tmp_cigar,"%s%s%s",tmp_cigar_left,tmp_cigar_N,cigar);
        realign_cigar_inv(tmp_cigar,gstart,rstart,end_pos,xy_gap_size,tmp_cigar_left,cigar,&head_m,&total_m,DUP);
        //printf("%i,%i %s %s\n",head_m,total_m,tmp_cigar_left,cigar);

	head_len=head_m+head_i;
	tail_len=total_m+total_i;
	int a;
        a=2*x-end_pos + sfr->genome_start+xy_gap_size;
	gap_size=a-2;
	sprintf(tmp_cigar_N,"%i%c",gap_size,'N');
	int m;
	for(m=0;m<tail_len;m++){
		qralign[m]=base_translate((qr[m+head_len]), false);
	}
	for(m=tail_len;m<rlen;m++){
			qralign[m]=base_translate(qr[m-tail_len], false);
	}

	sfr->genome_start= end_pos - x -total_m-total_d+2;
	qralign[m]='\0';


	strcpy(tmp_cigar_right,cigar);
        sprintf(cigar,"%s%s%s",tmp_cigar_right,tmp_cigar_N,tmp_cigar_left);

}
static void
pretty_print_inv(struct sw_full_results *sfr, int k, char* cigar, int x, int inver_type, int end_pos,int xy_gap_size)
{
	char *d, *q;
	int i,j,l, done;
	char tmp_cigar[200];
	char tmp_cigar_N[100];
	char tmp_cigar_left[100];
	char tmp_cigar_right[100];
	int gstart, rstart;

	d = dbalign;
	q = qralign;
	i=sfr->read_start-1;
	j=sfr->genome_start-1;
	rstart=i;
        gstart=j;

	done = 0;
	sfr->is_split_align=0;

	cigar[0]='\0';
	tmp_cigar_left[0]='\0';
	tmp_cigar_N[0]='\0';
	tmp_cigar_right[0]='\0';
	int tmp_back=-1;
	int n_m,n_d,n_i,n_n,n_left;//num for match/mismatch deletion insertion
	int total_m,total_d,total_i;
	n_m=0;
	n_d=0;
	n_i=0;
	n_n=0;
	total_m=0;
	total_d=0;
	total_i=0;
	n_left=0;
	int head_m=0,head_d=0,head_i=0,head_len=0,tail_len=0;
	for (l = k; l < (dblen + qrlen); l++) {
		
		switch (backtrace[l]) {
		case BACK_DELETION:

			n_d++;
			total_i++;

			strcpy(tmp_cigar,cigar);
			if(tmp_back==BACK_DELETION){

			}else{
				if(tmp_back== BACK_INSERTION){
					sprintf(cigar,"%s%i%c",tmp_cigar,n_i,'D');
					n_i=0;
				}
				if(tmp_back== BACK_MATCH_MISMATCH){
					sprintf(cigar,"%s%i%c",tmp_cigar,n_m,'M');
					n_m=0;
				}
				assert(tmp_back!=BACK_N);


			}
			tmp_back= BACK_DELETION;

			*d++ = '-';

			break;

		case BACK_INSERTION:
			n_i++;
			total_d++;

			strcpy(tmp_cigar,cigar);
			if(tmp_back==BACK_INSERTION){

			}else{
				if(tmp_back==BACK_DELETION){
					sprintf(cigar,"%s%i%c",tmp_cigar,n_d,'I');
					n_d=0;
				}
				if(tmp_back== BACK_MATCH_MISMATCH){
					sprintf(cigar,"%s%i%c",tmp_cigar,n_m,'M');
					n_m=0;
				}
				assert(tmp_back!=BACK_N);

			}
			tmp_back= BACK_INSERTION;

			*d++ = base_translate(db[j++], false);

			break;

		case BACK_N:

			strcpy(tmp_cigar,cigar);
			if(tmp_back==BACK_MATCH_MISMATCH){
				sprintf(cigar,"%s%i%c",tmp_cigar,n_m,'M');
			}else{
				if(tmp_back== BACK_INSERTION){
					sprintf(cigar,"%s%i%c",tmp_cigar,n_i,'D');

				}else if(tmp_back==BACK_DELETION){
					sprintf(cigar,"%s%i%c",tmp_cigar,n_d,'I');

				}else{

				}

			}

			sfr->is_split_align=1;
			strcpy(tmp_cigar_left,cigar);

			head_m=total_m;
			head_d=total_d;
			head_i=total_i;

			total_m=0;
			total_i=0;
			total_d=0;

			cigar[0]='\0';
			n_m=0;n_i=0;n_d=0;tmp_back=-1;


			*d++ = base_translate(db[j++], false);

			break;
		case BACK_MATCH_MISMATCH:
			n_m++;
			total_m++;

			strcpy(tmp_cigar,cigar);
			if(tmp_back==BACK_MATCH_MISMATCH){

			}else{
				if(tmp_back== BACK_INSERTION){
					sprintf(cigar,"%s%i%c",tmp_cigar,n_i,'D');
					n_i=0;
				}
				if(tmp_back==BACK_DELETION){
					sprintf(cigar,"%s%i%c",tmp_cigar,n_d,'I');
					n_d=0;
				}
				assert(tmp_back!=BACK_N);

			}
			tmp_back= BACK_MATCH_MISMATCH;

			*d++ = base_translate(db[j++], false);

			break;

		default:
			done = 1;
		}

		if (done)
			break;
	}
	if(!sfr->is_split_align){
		return;
	}

	*d='\0';
	strcpy(tmp_cigar,cigar);
	if(tmp_back==BACK_MATCH_MISMATCH){
		sprintf(cigar,"%s%i%c",tmp_cigar,n_m,'M');
	}else{
		if(tmp_back== BACK_INSERTION){
			sprintf(cigar,"%s%i%c",tmp_cigar,n_i,'D');

		}
		if(tmp_back==BACK_DELETION){
			sprintf(cigar,"%s%i%c",tmp_cigar,n_d,'I');

		}
		assert(tmp_back!=BACK_N);

	}

	sprintf(tmp_cigar_N,"%i%c",gap_size,'N');
        //strcpy(tmp_cigar_right,cigar);
        sprintf(tmp_cigar,"%s%s%s",tmp_cigar_left,tmp_cigar_N,cigar);
        realign_cigar_inv(tmp_cigar,gstart,rstart,end_pos,xy_gap_size,tmp_cigar_left,cigar,&head_m,&total_m,inver_type);
        //printf("%i,%i %s %s\n",head_m,total_m,tmp_cigar_left,cigar);
	
	head_len=head_m+head_i;
	tail_len=total_m+total_i;
	int m,a;
	char rcigar[100];
	memset(rcigar,0,100);
	switch(inver_type){
        case INVER_FW_OUT:
		a=x+2*x-end_pos- sfr->genome_start+xy_gap_size;

		gap_size=a-(head_m + head_d)+1;
                for(m=0;m<head_len;m++){
                	qralign[m]=base_translate(qr[m], false);
                }
                for(m=head_len;m<rlen;m++){
                	qralign[m]=base_translate((3-qr[rlen-m+head_len-1]), false);
                }

		reverse_cigar(cigar,tmp_cigar_right);
		sprintf(tmp_cigar_N,"%i%c",gap_size,'N');
		sprintf(cigar,"%s%s%s",tmp_cigar_left,tmp_cigar_N,tmp_cigar_right);
		break;
	case INVER_FW_IN:
		a=end_pos-x+sfr->genome_start+xy_gap_size;

		gap_size=a-(total_m+total_d)-1;
                for(m=0;m<tail_len;m++){
                	qralign[m]=base_translate((3-qr[rlen-m-1]), false);
                }
                for(m=tail_len;m<rlen;m++){
                	qralign[m]=base_translate(qr[m-tail_len], false);
                }
                sfr->genome_start=2*x - end_pos;

		reverse_cigar(cigar,tmp_cigar_right);

		sprintf(tmp_cigar_N,"%i%c",gap_size,'N');
		sprintf(cigar,"%s%s%s",tmp_cigar_right,tmp_cigar_N,tmp_cigar_left);

		break;
	case INVER_BW_IN:
		a=2*x-end_pos+x-sfr->genome_start+xy_gap_size;

		gap_size=a-(head_m+head_d)+1;
		for(m=0;m<tail_len;m++){
			qralign[m]=base_translate(qr[head_len+m], false);
		}
		for(m=tail_len;m<rlen;m++){
			qralign[m]=base_translate((3-qr[head_len-(m-tail_len)-1]), false);

		}

		sfr->genome_start=end_pos-x-(total_m+total_d)+2;

		reverse_cigar(tmp_cigar_left,rcigar);
		strcpy(tmp_cigar_right,cigar);
		sprintf(tmp_cigar_N,"%i%c",gap_size,'N');
		sprintf(cigar,"%s%s%s",tmp_cigar_right,tmp_cigar_N,rcigar);

		break;
	case INVER_BW_OUT:
		a=sfr->genome_start+xy_gap_size+end_pos-x;

		gap_size=a-(total_m+total_d)-1;
		for(m=0;m<head_len;m++){
			qralign[m]=base_translate((3-qr[head_len-m-1]), false);
		}
		for(m=head_len;m<rlen;m++){
			qralign[m]=base_translate(qr[m], false);
		}

		sfr->genome_start=x-sfr->genome_start-(head_m+head_d)+2;
		reverse_cigar(tmp_cigar_left,rcigar);
		strcpy(tmp_cigar_right,cigar);
		sprintf(tmp_cigar_N,"%i%c",gap_size,'N');
                sprintf(cigar,"%s%s%s",rcigar,tmp_cigar_N,tmp_cigar_right);
		break;
	case SMALL_INVER_FW:
		a=sfr->genome_start+xy_gap_size+end_pos-x;

		gap_size=a-(total_m+total_d)-1;
		for(m=0;m<head_len;m++){
			qralign[m]=base_translate((3-qr[head_len-m-1]), false);
		}
		for(m=head_len;m<rlen;m++){
			qralign[m]=base_translate(qr[m], false);
		}

		sfr->genome_start=x-sfr->genome_start-(head_m+head_d)+2;
		reverse_cigar(tmp_cigar_left,rcigar);
		strcpy(tmp_cigar_right,cigar);
		sprintf(tmp_cigar_N,"%i%c",gap_size,'N');
		sprintf(cigar,"%s%s%s",rcigar,tmp_cigar_N,tmp_cigar_right);
		break;

	case SMALL_INVER_BW:
		a=x+2*x-end_pos- sfr->genome_start+xy_gap_size;

		gap_size=a-(head_m + head_d)+1;
		for(m=0;m<head_len;m++){
				qralign[m]=base_translate(qr[m], false);
		}
		for(m=head_len;m<rlen;m++){
				qralign[m]=base_translate((3-qr[rlen-m+head_len-1]), false);
		}

		reverse_cigar(cigar,tmp_cigar_right);
		sprintf(tmp_cigar_N,"%i%c",gap_size,'N');
		sprintf(cigar,"%s%s%s",tmp_cigar_left,tmp_cigar_N,tmp_cigar_right);
		break;

	default:
		abort();
	}
	qralign[m]='\0';

}
static void
pretty_print(int i, int j, int k, char* cigar, int x, int xy_gap_size, bool *is_split_align, int sv_type)
{
	char *d, *q;
	int l, done;
	int gstart, rstart;
	char tmp_cigar[1024];
	d = dbalign;
	q = qralign;

	done = 0;
	*is_split_align=0;
	gstart=j;
	rstart=i;
	cigar[0]='\0';
	int tmp_back=-1;
	int n_m,n_d,n_i,n_n,n_left;//num for match/mismatch deletion insertion
	n_m=0;
	n_d=0;
	n_i=0;
	n_n=0;
	n_left=0;
	for (l = k; l < (dblen + qrlen); l++) {
		switch (backtrace[l]) {
		case BACK_DELETION:
			if(sv_type==SMALL_INDEL){
				*is_split_align=1;
			}
			n_d++;
			n_left++;
			strcpy(tmp_cigar,cigar);
			if(tmp_back==BACK_DELETION){

			}else{
				if(tmp_back== BACK_INSERTION){
					sprintf(cigar,"%s%i%c",tmp_cigar,n_i,'D');
					n_i=0;
				}
				if(tmp_back== BACK_MATCH_MISMATCH){
					sprintf(cigar,"%s%i%c",tmp_cigar,n_m,'M');
					n_m=0;
				}
				assert(tmp_back!=BACK_N);
			}
			tmp_back= BACK_DELETION;

			*d++ = '-';
			*q++ = base_translate(qr[i++], false);
			break;

		case BACK_INSERTION:
			if(sv_type==SMALL_INDEL){
				*is_split_align=1;
			}
			n_i++;
			strcpy(tmp_cigar,cigar);
			if(tmp_back==BACK_INSERTION){

			}else{
				if(tmp_back==BACK_DELETION){
					sprintf(cigar,"%s%i%c",tmp_cigar,n_d,'I');
					n_d=0;
				}
				if(tmp_back== BACK_MATCH_MISMATCH){
					sprintf(cigar,"%s%i%c",tmp_cigar,n_m,'M');
					n_m=0;
				}
				assert(tmp_back!=BACK_N);
			}
			tmp_back= BACK_INSERTION;

			*d++ = base_translate(db[j++], false);
			*q++ = '-';
			break;
		case BACK_INSERTION_N:
			*is_split_align=1;
			strcpy(tmp_cigar,cigar);
			if(tmp_back==BACK_MATCH_MISMATCH){
				sprintf(cigar,"%s%i%c",tmp_cigar,n_m,'M');
			}else{
				if(tmp_back== BACK_INSERTION){
					sprintf(cigar,"%s%i%c",tmp_cigar,n_i,'D');

				}else if(tmp_back==BACK_DELETION){
					sprintf(cigar,"%s%i%c",tmp_cigar,n_d,'I');

				}else{

				}

			}

			strcpy(tmp_cigar,cigar);
			sprintf(cigar,"%s%i%c",tmp_cigar,gap_size,'I');
			n_m=0;n_i=0;n_d=0;tmp_back=-1;

			*d++ = base_translate(db[j++], false);
			*q++ = '*';
			break;
		case BACK_N:

			strcpy(tmp_cigar,cigar);
			if(tmp_back==BACK_MATCH_MISMATCH){
				sprintf(cigar,"%s%i%c",tmp_cigar,n_m,'M');
			}else{
				if(tmp_back== BACK_INSERTION){
					sprintf(cigar,"%s%i%c",tmp_cigar,n_i,'D');

				}else if(tmp_back==BACK_DELETION){
					sprintf(cigar,"%s%i%c",tmp_cigar,n_d,'I');

				}else{

				}

			}

			*is_split_align=1;
			strcpy(tmp_cigar,cigar);
			sprintf(cigar,"%s%i%c",tmp_cigar,gap_size,'N');
			n_m=0;n_i=0;n_d=0;tmp_back=-1;

			*d++ = base_translate(db[j++], false);
			*q++ = '*';
			break;
		case BACK_MATCH_MISMATCH:
			n_m++;
			n_left++;
			strcpy(tmp_cigar,cigar);
			if(tmp_back==BACK_MATCH_MISMATCH){

			}else{
				if(tmp_back== BACK_INSERTION){
					sprintf(cigar,"%s%i%c",tmp_cigar,n_i,'D');
					n_i=0;
				}
				if(tmp_back==BACK_DELETION){
					sprintf(cigar,"%s%i%c",tmp_cigar,n_d,'I');
					n_d=0;
				}
				assert(tmp_back!=BACK_N);

			}
			tmp_back= BACK_MATCH_MISMATCH;

			*d++ = base_translate(db[j++], false);
			*q++ = base_translate(qr[i++], false);
			break;

		default:
			done = 1;
		}

		if (done)
			break;
	}

	*d = *q = '\0';
	strcpy(tmp_cigar,cigar);
	if(tmp_back==BACK_MATCH_MISMATCH){
		sprintf(cigar,"%s%i%c",tmp_cigar,n_m,'M');
	}else{
		if(tmp_back== BACK_INSERTION){
			sprintf(cigar,"%s%i%c",tmp_cigar,n_i,'D');

		}
		if(tmp_back==BACK_DELETION){
			sprintf(cigar,"%s%i%c",tmp_cigar,n_d,'I');

		}
		assert(tmp_back!=BACK_N);
	}
	realign_cigar(cigar,gstart,rstart,xy_gap_size);
}

int
sw_split_ls_setup(int _dblen, int _qrlen, int _a_gap_open, int _a_gap_ext,  int _b_gap_open,
		int _b_gap_ext, int _match, int _mismatch,int _split_penalty_ins,
		int _split_penalty_del_large, int _split_penalty_del_double,int _kmer_len,int min_rlen)
{
	if(initialised){
		fprintf(stderr, "initialised already.\n");
		return (0);
	}
	dblen = _dblen;

	if(db==NULL){
		db = (int8_t *)malloc(dblen * sizeof(db[0]));
		if (db == NULL)
			return (1);
	}

	qrlen = _qrlen;

	qr = (int8_t *)malloc(qrlen * sizeof(qr[0]));
	if (qr == NULL)
		return (1);

	swmatrix = (struct swcell *)malloc((dblen + 1) * (qrlen + 1) * sizeof(swmatrix[0]));
	if (swmatrix == NULL)
		return (1);

	//swmatrix

	backtrace_pos = (int *)malloc((dblen + qrlen) * sizeof(backtrace_pos[0]));
	backtrace = (int8_t *)malloc((dblen + qrlen) * sizeof(backtrace[0]));
	if (backtrace == NULL)
		return (1);

	dbalign = (char *)malloc((dblen + qrlen + 1) * sizeof(dbalign[0]));
	if (dbalign == NULL)
		return (1);

	qralign = (char *)malloc((dblen + qrlen + 1) * sizeof(dbalign[0]));
	if (qralign == NULL)
		return (1);
	max_w_cur= (struct max_t *) calloc((qrlen+1), sizeof(struct max_t));
	if(max_w_cur==NULL)
		return (1);
	max_w1= (struct max_t *) calloc((qrlen+1), sizeof(struct max_t));
	if(max_w1==NULL)
		return (1);
	max_w2= (struct max_t *) calloc((qrlen+1), sizeof(struct max_t));
	if(max_w2==NULL)
		return (1);
	max_n_cur= (struct max_t *) calloc((dblen+1), sizeof(struct max_t));
	if(max_n_cur==NULL)
		return (1);
	max_n1= (struct max_t *) calloc((dblen+1), sizeof(struct max_t));
	if(max_n1==NULL)
		return (1);
	max_n2= (struct max_t *) calloc((dblen+1), sizeof(struct max_t));
	if(max_n2==NULL)
		return (1);
	a_gap_open = -(_a_gap_open);
	a_gap_ext = -(_a_gap_ext);
	b_gap_open = -(_b_gap_open);
	b_gap_ext = -(_b_gap_ext);
	//split_penalty_double = -(_split_penalty_double);
	split_penalty_del_double=_split_penalty_del_double;
	split_penalty_del_large=_split_penalty_del_large;
	split_penalty_ins=_split_penalty_ins;
	match = _match;
	mismatch = _mismatch;
	if(min_rlen>50){
		kmer_len_array[0]=36;
		kmer_len_array[1]=25;
		kmer_len_array[2]=17;
		kmer_len_array_len=3;
	}else{
		kmer_len_array[0]=17;
		kmer_len_array[1]=11;
		kmer_len_array_len=2;
	}
	initialised = 1;
	//memset(hashtable_kmer,0,HASHSIZE);
	
	hash_setup(dblen);
	return (0);
}
static int
split_sw_del(int lena, int lenb, int *iret, int *jret, int *backtrace_pos_len, bool revcmpl, int x, int overlap_start, bool cluster_supported)
{
  int i, j, m;
  int q=0;

  int score, ms, a_go, a_ge, b_go, b_ge, tmp;
  int multi_aln_pos_num=0;
  int split_penalty;
  int8_t tmp2;

  if(cluster_supported){
  	split_penalty_del=split_penalty_del_large;
  }else{
	split_penalty_del = split_penalty_del_double;
  }
  score = WORST_SCORE;
  a_go = a_gap_open;
  a_ge = a_gap_ext;
  b_go = b_gap_open;
  b_ge = b_gap_ext;
/*
  for(i=0;i<(lena+1)*(lenb+1);i++){
  	  swmatrix[i].score_northwest_max =WORST_SCORE;
  	  swmatrix[i].score_northwest = WORST_SCORE;
  	  swmatrix[i].score_north = WORST_SCORE;
  	  swmatrix[i].score_west = WORST_SCORE;
  }
*/
  //initialize the first row

  for (j = 0; j < lena + 1; j++) {
    init_cell(j);
  }

  //initialize the first column
  for (i = 1; i < lenb + 1; i++) {
    int idx = i * (lena + 1);
    swmatrix[idx].back_north = FROM_NORTH_NORTH;
    swmatrix[idx].score_north = -a_go-i*a_ge;
    swmatrix[idx].score_northwest_max =WORST_SCORE;
    swmatrix[idx].score_northwest = WORST_SCORE;
    swmatrix[idx].score_west = WORST_SCORE;
  }

  /*
   * Figure out our band.
   *   We can actually skip computation of a significant number of
   *   cells, which could never be part of an alignment corresponding
   *   to our threshhold score.
   */

  for (i = 0; i < lenb; i++) {
  	  for(j=0;j<x;j++){

  		  	struct swcell *cell_nw, *cell_n, *cell_w, *cell_cur;
  			cell_nw  = &swmatrix[i * (lena + 1) + j];
  			cell_n   = cell_nw + 1;
  			cell_w   = cell_nw + (lena + 1);
  			cell_cur = cell_w + 1;

  			ms = (db[j] == qr[i]) ? match : mismatch;

  			/*
  			 * northwest
  			 */

  			if (!revcmpl) {

  				tmp  = cell_nw->score_northwest + ms;
  				tmp2 = FROM_NORTHWEST_NORTHWEST;

  				if (cell_nw->score_north + ms > tmp) {
  					tmp  = cell_nw->score_north + ms;
  					tmp2 = FROM_NORTHWEST_NORTH;

  				}

  				if (cell_nw->score_west + ms > tmp) {
  					tmp  = cell_nw->score_west + ms;
  					tmp2 = FROM_NORTHWEST_WEST;

  				}
  			} else {

  				  tmp  = cell_nw->score_west + ms;
  				  tmp2 = FROM_NORTHWEST_WEST;


  				if (cell_nw->score_north + ms > tmp) {
  				  tmp  = cell_nw->score_north + ms;
  				  tmp2 = FROM_NORTHWEST_NORTH;

  				}

  				if (cell_nw->score_northwest + ms > tmp) {
  				  tmp  = cell_nw->score_northwest + ms;
  				  tmp2 = FROM_NORTHWEST_NORTHWEST;

  				}
  			}

  			cell_cur->score_northwest = tmp;
  			cell_cur->back_northwest  = tmp2;
  			cell_cur->pos = -1;

  			/**
  			 * norhtwest max
  			 */
  			cell_cur->score_northwest_max = WORST_SCORE;
  			cell_cur->back_northwest_max  = 0;

  			/*
  			 * north
  			 */

  			if (!revcmpl) {
  				tmp  = cell_n->score_northwest - b_go - b_ge;
  				tmp2 = FROM_NORTH_NORTHWEST;

  				if (cell_n->score_north - b_ge > tmp) {
  				  tmp  = cell_n->score_north - b_ge;
  				  tmp2 = FROM_NORTH_NORTH;

  				}
  			} else {
  				tmp  = cell_n->score_north - b_ge;
  				tmp2 = FROM_NORTH_NORTH;


  				if (cell_n->score_northwest - b_go - b_ge > tmp) {
  				  tmp  = cell_n->score_northwest - b_go - b_ge;
  				  tmp2 = FROM_NORTH_NORTHWEST;

  				}
  			}


  			cell_cur->score_north = tmp;
  			cell_cur->back_north  = tmp2;
  			cell_cur->pos = -1;


  			/*
  			 * west
  			 */

  			if (!revcmpl) {
  				tmp  = cell_w->score_northwest - a_go - a_ge;
  				tmp2 = FROM_WEST_NORTHWEST;

  				if (cell_w->score_west - a_ge > tmp) {
  				  tmp  = cell_w->score_west - a_ge;
  				  tmp2 = FROM_WEST_WEST;

  				}
  			} else {
  				tmp  = cell_w->score_west - a_ge;
  				tmp2 = FROM_WEST_WEST;

  				if (cell_w->score_northwest - a_go - a_ge > tmp) {
  				  tmp  = cell_w->score_northwest - a_go - a_ge;
  				  tmp2 = FROM_WEST_NORTHWEST;
  				}
  			}

  			cell_cur->score_west = tmp;
  			cell_cur->back_west  = tmp2;
  			cell_cur->pos = -1;

  	  }
    }



  	/**
  	 * compute max score of each row
  	 * when jump happens, it always jump from the left most pos, because the max score is left most
  	 */


  	//initial max_w_cur matritx
/*
  	for(i=0;i<lenb+1;i++){
		max_w_cur[i].score=WORST_SCORE;//max score of left part
		max_w_cur[i].pos=0;//max cell position
  	}

  	for(i=JUMP_LEN_THRESH;i<lenb+1;i++){
		for(m=0;m<x+1;m++){
			if(max_w_cur[i].score<(&swmatrix[(i) * (lena + 1) + m])->score_northwest){
				max_w_cur[i].score=(&swmatrix[(i) * (lena + 1) + m])->score_northwest;
				max_w_cur[i].pos=m;
			}
		}
  	}
*/
  	for (i = 1; i < lenb + 1; i++) {
		int idx = i * (lena + 1)+x;
		swmatrix[idx].back_north = FROM_NORTH_NORTH;
		swmatrix[idx].score_north = -a_go-i*a_ge;
		swmatrix[idx].score_northwest = WORST_SCORE;
		swmatrix[idx].score_west = WORST_SCORE;

  	}
  	//j>x
  	struct max_t max_w;
  	for (i = 0; i < lenb; i++) {
  	  max_w.score=WORST_SCORE;
  	  max_w.pos=-1;
  	  max_w.multi=1;
	  //for(m=0;m<x+1;m++){
	  for(m=0;m<overlap_start-1;m++){
		if(max_w.score<(&swmatrix[(i) * (lena + 1) + m])->score_northwest){
			max_w.score=(&swmatrix[(i) * (lena + 1) + m])->score_northwest;
			max_w.pos=m;
		}
	  }
      for (j = x; j < lena; j++) {
        struct swcell *cell_nw, *cell_n, *cell_w, *cell_cur, *cell_max_w, *cell_return;
        cell_nw  = &swmatrix[i * (lena + 1) + j];
        cell_n   = cell_nw + 1;
        cell_w   = cell_nw + (lena + 1);
        cell_cur = cell_w + 1;
        ms = (db[j] == qr[i]) ? match : mismatch;
        if((j-x)+overlap_start-1>0 && (j-x)+overlap_start<=x){//in overlap region
		cell_max_w=(&swmatrix[i * (lena + 1) + (j-x)+overlap_start-1]);
		if(cell_max_w->score_northwest >= max_w.score){
			//max_w.score=(&swmatrix[(i) * (lena + 1) + (j-x)+overlap_start-2])->score_northwest;
			//max_w.pos=j-x+overlap_start-2;
			if(cell_max_w->score_northwest == max_w.score){
				max_w.multi++;
			}else{
				max_w.multi=1;
			}
			max_w.score=(&swmatrix[(i) * (lena + 1) + (j-x)+overlap_start-1])->score_northwest;
			max_w.pos=j-x+overlap_start-1;
		}
        }
      	/*
  		 * northwest
  		 */
  		if (!revcmpl) {
  			tmp  = cell_nw->score_northwest + ms;
  			tmp2 = FROM_NORTHWEST_NORTHWEST;
  			if ((i<lenb-JUMP_LEN_THRESH) && (cell_nw->score_northwest_max + ms > tmp)) {
  				tmp  = cell_nw->score_northwest_max + ms;
				tmp2 = FROM_NORTHWEST_NORTHWEST_MAX;

			}
  			if (cell_nw->score_north + ms > tmp) {
  				tmp  = cell_nw->score_north + ms;
  				tmp2 = FROM_NORTHWEST_NORTH;
  			}
  			if (cell_nw->score_west + ms > tmp) {
  				tmp  = cell_nw->score_west + ms;
  				tmp2 = FROM_NORTHWEST_WEST;
  			}
  		} else {

  			tmp  = cell_nw->score_west + ms;
  			tmp2 = FROM_NORTHWEST_WEST;

			if (cell_nw->score_north + ms > tmp) {
			  tmp  = cell_nw->score_north + ms;
			  tmp2 = FROM_NORTHWEST_NORTH;

			}
			if (cell_nw->score_northwest_max + ms > tmp) {
				tmp  = cell_nw->score_northwest_max + ms;
				tmp2 = FROM_NORTHWEST_NORTHWEST_MAX;

			}

			if (cell_nw->score_northwest + ms > tmp) {
			  tmp  = cell_nw->score_northwest + ms;
			  tmp2 = FROM_NORTHWEST_NORTHWEST;

			}

  		}

  		cell_cur->score_northwest = tmp;
  		cell_cur->back_northwest  = tmp2;
  		cell_cur->pos = -1;

  		/*
  		 * northwest_max
  		 */
  		  if (!revcmpl) {
			if(i<lenb-JUMP_LEN_THRESH && i>JUMP_LEN_THRESH){
  				//tmp  = max_w_cur[i].score + ms + split_penalty;//+ 1;
				tmp = max_w.score + ms + split_penalty_del;
  			  	tmp2 = FROM_NORTHWEST_MAX_NORTHWEST;
			}else{
				tmp =WORST_SCORE;
			}

  		  } else {//waiting for modify
  			 tmp  = max_w_cur[i].score + ms + split_penalty_del;//1;//3*(mismatch-match);
  			 tmp2 = FROM_NORTHWEST_MAX_NORTHWEST;
  		  }


  		  cell_cur->score_northwest_max = tmp;
  		  cell_cur->back_northwest_max  = tmp2;
  		  //cell_cur->pos = max_w_cur[i].pos;
  		  cell_cur->pos = max_w.pos;
  		  cell_cur->multi_left = max_w.multi;


  		  /*
  		   * north
  		   */
  		  if (!revcmpl) {

  			  tmp  = cell_n->score_northwest - b_go - b_ge;
  			  tmp2 = FROM_NORTH_NORTHWEST;


  			  if (cell_n->score_north - b_ge > tmp) {
  				  tmp  = cell_n->score_north - b_ge;
  				  tmp2 = FROM_NORTH_NORTH;

  			  }


  		  }else {
  			  tmp  = cell_n->score_north - b_ge;
  			  tmp2 = FROM_NORTH_NORTH;


  			  if (cell_n->score_northwest - b_go - b_ge > tmp) {
  				  tmp  = cell_n->score_northwest - b_go - b_ge;
				  tmp2 = FROM_NORTH_NORTHWEST;
  			  }


  		  }

  		  cell_cur->score_north = tmp;
  		  cell_cur->back_north  = tmp2;

  		  /*
  		   * west
  		   */
  		 if (!revcmpl) {

  			tmp  = cell_w->score_northwest - a_go - a_ge;
  			tmp2 = FROM_WEST_NORTHWEST;


  			if (cell_w->score_west - a_ge > tmp) {
  			  tmp  = cell_w->score_west - a_ge;
  			  tmp2 = FROM_WEST_WEST;

  			}

  		 } else {
  			tmp  = cell_w->score_west - a_ge;
			tmp2 = FROM_WEST_WEST;


			if (cell_w->score_northwest - a_go - a_ge > tmp) {
			  tmp  = cell_w->score_northwest - a_go - a_ge;
			  tmp2 = FROM_WEST_NORTHWEST;

			}

  		 }
  		 cell_cur->score_west = tmp;
  		 cell_cur->back_west  = tmp2;
        /*
		  * max score
		  */

		 if(i==lenb-1){

			if(cell_cur->score_west>score){// && cell_cur->w_jumped){
				  score=cell_cur->score_west;
				  q=0;
				  jret[q++]=j;

				  multi_aln_pos_num=1;
				  cell_return=cell_cur;
			}else if (cell_cur->score_west==score){
				  multi_aln_pos_num++;

			}

			if(cell_cur->score_northwest>score){//  && cell_cur->nw_jumped){
				  score=cell_cur->score_northwest;
				  q=0;
				  jret[q++]=j;

				  multi_aln_pos_num=1;
				  cell_return=cell_cur;
			}else if (cell_cur->score_northwest==score){
				multi_aln_pos_num++;
			}

			if(cell_cur->score_north>score){//  && cell_cur->n_jumped){
				  score=cell_cur->score_north;
				  q=0;
				  jret[q++]=j;

				  multi_aln_pos_num=1;
				  cell_return=cell_cur;
			}else if (cell_cur->score_north==score){
				multi_aln_pos_num++;

			}
			cell_return->multi_right=multi_aln_pos_num;
		 }

      }// for j

    }// for i

    *backtrace_pos_len=q;
    *iret = lenb;//for full sw, iret is rlen
    *jret+=1;
    if(multi_aln_pos_num>1){
    	//score=WORST_SCORE;
    }


    return score;

}
static int
split_sw_indel(int lena,int lenb, int *iret, int *jret, int *backtrace_pos_len, bool revcmpl, int x, int overlap_start,int cluster_supported){

	  int i, j;

	  int score_ins,score_del,a_go, a_ge, b_go, b_ge;

	  //split_penalty=split_penalty_ins;
	  split_penalty_del = split_penalty_del_double;
	  score_ins = WORST_SCORE;
	  score_del = WORST_SCORE;
	  a_go = a_gap_open;
	  a_ge = a_gap_ext;
	  b_go = b_gap_open;
	  b_ge = b_gap_ext;
/*
	  for(i=0;i<(lena+1)*(2*lenb+1);i++){
	  	  swmatrix[i].score_northwest_max =WORST_SCORE;
	  	  swmatrix[i].score_northwest = WORST_SCORE;
	  	  swmatrix[i].score_north = WORST_SCORE;
	  	  swmatrix[i].score_west = WORST_SCORE;
	  }
*/
	   //clean up the last row of del_matrix
          for(i=x+1;i<=lena;i++){
                int idx;
                idx=lenb*(lena+1)+i;
                swmatrix[idx].score_northwest_max =WORST_SCORE;
                swmatrix[idx].score_northwest = WORST_SCORE;
                swmatrix[idx].score_north = WORST_SCORE;
                swmatrix[idx].score_west = WORST_SCORE;
          }
          //clean up the last row of del_matrix
          for(i=1;i<x;i++){
                 int idx;
                 idx=2*lenb*(lena+1)+i;
                 swmatrix[idx].score_northwest_max =WORST_SCORE;
                 swmatrix[idx].score_northwest = WORST_SCORE;
                 swmatrix[idx].score_north = WORST_SCORE;
                 swmatrix[idx].score_west = WORST_SCORE;
           }

	  //initialize the first row

	  for (j = 0; j < lena + 1; j++) {
		  init_cell(j);
	  }

	  //initialize the first column
	  for (i = 1; i < lenb; i++) {

		int idx = i * (lena + 1);
		swmatrix[idx].back_north = FROM_NORTH_NORTH;
		swmatrix[idx].score_north = -b_go-i*b_ge;
		swmatrix[idx].score_northwest = WORST_SCORE;
		swmatrix[idx].score_northwest_max = WORST_SCORE;
		swmatrix[idx].score_west = WORST_SCORE;
	  }
/*
	  for(j = 0; j < lena + 1; j++){
		  init_cell(lenb*(lena+1)+j);
	  }
	  for(i = 1; i < lenb + 1; i++){
		int idx = i * (lena + 1)+x;
		swmatrix[idx].back_north = FROM_NORTH_NORTH;
		swmatrix[idx].score_north = -b_go-i*b_ge;
		swmatrix[idx].score_northwest_max =WORST_SCORE;
		swmatrix[idx].score_northwest = WORST_SCORE;
		swmatrix[idx].score_west = WORST_SCORE;
	  }
	  for(i = lenb+1; i < 2*lenb+1; i++){
		  int idx = i * (lena + 1);
		  swmatrix[idx].back_north = FROM_NORTH_NORTH;
		  swmatrix[idx].score_northwest_max = WORST_SCORE;
		  swmatrix[idx].score_north = -b_go-(i-lenb)*b_ge;
		  swmatrix[idx].score_northwest = WORST_SCORE;
		  swmatrix[idx].score_west = WORST_SCORE;
	  }
*/
	  //return 0;
	  /*
	   * Figure out our band.
	   *   We can actually skip computation of a significant number of
	   *   cells, which could never be part of an alignment corresponding
	   *   to our threshhold score.
	   */
	  hll* temp;
	  int qrs,dbs,qre,dbe,qrs_ins,dbs_del,qre_ins,dbe_del;
	  int dbsl,dbel,qrsl,qrel;

	  /*fill part1*/

	  qrs=0;
	  qre=lenb;
	  dbs=0;
	  dbe=x;
	  fill_sub_sw_block_x(lena,lenb,x,qrs,qre,dbs,dbe);

	  qrsl=qrs;
	  qrel=qre;
	  dbsl=dbs;
	  dbel=dbe;
		
	  for(j = 0; j < lena + 1; j++){
                  init_cell(lenb*(lena+1)+j);
          }

	  for(i = 1; i < lenb + 1; i++){
                int idx = i * (lena + 1)+x;
                swmatrix[idx].back_north = FROM_NORTH_NORTH;
                swmatrix[idx].score_north = -b_go-i*b_ge;
                swmatrix[idx].score_northwest_max =WORST_SCORE;
                swmatrix[idx].score_northwest = WORST_SCORE;
                swmatrix[idx].score_west = WORST_SCORE;
          }
          for(i = lenb+1; i < 2*lenb+1; i++){
                  int idx = i * (lena + 1);
                  swmatrix[idx].back_north = FROM_NORTH_NORTH;
                  swmatrix[idx].score_northwest_max = WORST_SCORE;
                  swmatrix[idx].score_north = -b_go-(i-lenb)*b_ge;
                  swmatrix[idx].score_northwest = WORST_SCORE;
                  swmatrix[idx].score_west = WORST_SCORE;
          }
		

	  /*fill part2*/
	  qrs_ins=qrs+lenb;
	  qre_ins=qre+lenb;
	  fill_sub_sw_block_xy_ins(lena,lenb,qrs_ins,qre_ins,dbs,dbe,qrsl,qrel,overlap_start,x);
	  /*fill part3*/
	  dbs_del=dbs+x;
	  dbe_del=dbe+x;
	  fill_sub_sw_block_xy_del(lena,lenb,qrs,qre,dbs_del,dbe_del,dbsl,dbel,overlap_start,x,true);


	  //print_swmatrix( lena, lenb);
	  int iret_ins=0, jret_ins=0, iret_del=0, jret_del=0, bpl_ins=0,bpl_del=0;
	  score_ins=find_max_score(lena, 2*lenb,&iret_ins,&jret_ins,&bpl_ins,dbs,dbe);
	  score_del=find_max_score(lena, lenb,&iret_del,&jret_del,&bpl_del,dbs_del,dbe_del);
	  if(score_ins>score_del){
		  *iret=iret_ins;
		  *jret=jret_ins;
		  *backtrace_pos_len=bpl_ins;
	  }else{
		  *iret=iret_del;
		  *jret=jret_del;
		  *backtrace_pos_len=bpl_del;
	  }

	  return score_ins>score_del?score_ins:score_del;
}
void
sw_split_ls(int8_t *genome, uint32_t glen, uint32_t *read, int _rlen, int threshscore_ins, int threshscore_del,
		struct sw_full_results *sfr, bool revcmpl, int x, char* cigar, int xy_gap_size, int sv_type, int overlap_start,bool cluster_supported)
{
	llint before;
	int i,j,k, backtrace_pos_len;
	int8_t* q=db;
	int numfreed=0;
	int score_del=0;
	hll *best,*bestp,*best1first,*best1last,*best2first;
	hll *mylist;
	bool do_full_alignment=false;
	before = rdtsc();

	if (!initialised){
		fprintf(stderr,"not initialised!\n");
		abort();
	}
	revcmpl=0;
	swinvocs++;

	assert(glen > 0 && glen <= dblen);
	assert(rlen > 0 && rlen <= qrlen);

	memset(backtrace, 0, (dblen + qrlen) * sizeof(backtrace[0]));
	memset(backtrace_pos, 0, (_rlen) * sizeof(backtrace_pos[0]));

	db=genome;

	rlen=_rlen;

	for (i = 0; i < rlen; i++){
		qr[i] = (int8_t)EXTRACT(read, i);

	}


	sfr->is_split_align=0;
	sfr->multi_aln=0;
	sfr->qralign=NULL;
	sfr->score=WORST_SCORE;

	/*build hash table*/
	for(j=0;j<kmer_len_array_len;j++){
		int kmer_len=kmer_len_array[j];
		switch(sv_type){
		case SMALL_INDEL:
			q=db;
			for(i=0;i<x-kmer_len+1;i++){
				//memcpy(tmp,q++,kmer_len);
				if(!hashtable_insert(q, i+1,kmer_len)){
					return;
				}
				q++;
			}
			break;
		case LARGE_DEL:
		case INVER_FW_OUT:
       		case SMALL_INVER_BW:
	        case INVER_FW_IN:
	        case INVER_BW_IN:
	        case INVER_BW_OUT:
	        case SMALL_INVER_FW:
		case DUP:
			q=db;
                	for(i=0;i<overlap_start-kmer_len;i++){
	                        //memcpy(tmp,q++,kmer_len);
        	                if(!hashtable_insert(q, i+1,kmer_len)){
					return;
				}
				q++;
        	        }
                	q=&db[x];
	                for(i=x;i<glen-kmer_len+1;i++){
        	                //memcpy(tmp,q++,kmer_len);
                	        if(!hashtable_insert(q, i+1,kmer_len)){
					return;
				}
				q++;
                	}

			break;
		default:
			fprintf(stderr,"unknown sv type:%i\n",sv_type);
			abort();
		}

		//print_hashtable();

		/*merge squares and find best chain*/
		//hll *mylist;
		best=laganAnchors(qr,rlen, kmer_len,&mylist);
		hashtable_clean();
		if(best){
			if(best->scoreSoFar<0){
				do_full_alignment=true;
			}
			break;
		}

	}
	if(!best){
		return;
	}
	if(do_full_alignment){

	}else{
		best1first=NULL;
		best1last=NULL;
		best2first=NULL;
	#ifdef DEBUG_OUTPUT_LOG
		bestp=best;
		while(bestp){
			//printf("best:%i\n",bestp->seq1start);
			printf("0(%d %d)=",bestp->seq2start, bestp->seq2end);
			printf("(%d %d) %f %f\n",bestp->seq1start, bestp->seq1end, bestp->score,bestp->scoreSoFar);
			bestp=bestp->bkptr;
		}
	#endif
		bestp=best;
		bool best2first_fixed=false;
		while(bestp){
			bestp->nextptr=best1first;
			if(!best2first_fixed && bestp->seq1end <=x){
				best2first=bestp->nextptr;
				bestp->nextptr=NULL;
				best2first_fixed=true;
				best1last=bestp;
			}
			best1first=bestp;
			bestp=bestp->bkptr;
		}

		best=best1first;
		if(best1first->seq1start>x){
			best2first=best1first;
			best1first=NULL;
			best1last=NULL;
		}

	#ifdef DEBUG_OUTPUT_LOG
		bestp=best1first;
		while(bestp){
			//printf("best:%i\n",bestp->seq1start);
			printf("1(%d %d)=",bestp->seq2start, bestp->seq2end);
			printf("(%d %d) %f %f\n",bestp->seq1start, bestp->seq1end, bestp->score,bestp->scoreSoFar);
			bestp=bestp->nextptr;
		}
		bestp=best2first;
		while(bestp){
			//printf("best:%i\n",bestp->seq1start);
			printf("2(%d %d)=",bestp->seq2start, bestp->seq2end);
			printf("(%d %d) %f %f\n",bestp->seq1start, bestp->seq1end, bestp->score,bestp->scoreSoFar);
			bestp=bestp->nextptr;
		}
	#endif
	}

	if(!do_full_alignment){
		//return;
	}

	switch(sv_type){
	case SMALL_INDEL:
		for (i = rlen; i < 2*rlen+1; i++){
			qr[i] = qr[i-rlen];
		}
		if(do_full_alignment){
			score_del = split_sw_indel(glen, rlen, &i, backtrace_pos, &backtrace_pos_len,revcmpl, x, overlap_start,cluster_supported);
		}else{
			score_del = split_sw_indel_square(best1first,glen,rlen, &i, backtrace_pos, &backtrace_pos_len,revcmpl, x, overlap_start);
		}
		if(score_del>threshscore_del ){
			sfr->score=score_del;
			sfr->type=sv_type;
			if(i==rlen){
				xy_gap_size=-x;
				k = do_backtrace(glen, i, backtrace_pos, backtrace_pos_len, sfr, xy_gap_size);
			}else if(i==2*rlen){
				gap_size=0;
				xy_gap_size=-1*rlen;
				k = do_backtrace_ins(glen, i, backtrace_pos, backtrace_pos_len, sfr, xy_gap_size);
			}else{
				abort();
			}

			pretty_print(sfr->read_start-1, sfr->genome_start-1, k, cigar, x-sfr->genome_start+1, xy_gap_size, &sfr->is_split_align,sv_type);


		}else{
			//sfr->type=NOT_PRINT;
		}
		break;

	/*large del*/
	case LARGE_DEL:
		//score_del = split_sw_del(glen, rlen, &i, backtrace_pos, &backtrace_pos_len,revcmpl, x, overlap_start,cluster_supported);//i is  rlen, j is genome pos
		if(do_full_alignment){
			score_del = split_sw_del(glen, rlen, &i, backtrace_pos, &backtrace_pos_len,revcmpl, x, overlap_start,cluster_supported);
		}else{
			score_del = split_sw_del_square(best1first,best2first,glen,rlen, &i, backtrace_pos, &backtrace_pos_len,revcmpl, x, overlap_start,cluster_supported);
		}
		if(score_del>threshscore_del){
			sfr->score=score_del;
			k = do_backtrace(glen, i, backtrace_pos, backtrace_pos_len, sfr, xy_gap_size);
			pretty_print(sfr->read_start-1, sfr->genome_start-1, k, cigar, x-sfr->genome_start+1, xy_gap_size, &sfr->is_split_align,sv_type);
			sfr->type=LARGE_DEL;

		}else{
			//sfr->type=NOT_PRINT;
		}
		break;

	/*inver*/
	case INVER_FW_OUT:
	case SMALL_INVER_BW:
	case INVER_FW_IN:
	case INVER_BW_IN:
	case INVER_BW_OUT:
	case SMALL_INVER_FW:
		int inver_type;

		inver_type=sv_type;

		switch(inver_type){
		case INVER_FW_OUT:
		case SMALL_INVER_BW:
			revcmpl=0;
			break;
		case INVER_FW_IN:
			revcmpl=1;
			break;
		case INVER_BW_IN:
			revcmpl=0;
			break;
		case INVER_BW_OUT:
		case SMALL_INVER_FW:
			revcmpl=1;
			break;
		default:
			fprintf(stderr,"invalid inver type: %i\n",inver_type);
			abort();
		}
		//score_del = split_sw_del(glen, rlen, &i, backtrace_pos, &backtrace_pos_len,revcmpl, x, overlap_start, cluster_supported);//i is  rlen, j is genome pos
		if(do_full_alignment){
			score_del = split_sw_del(glen, rlen, &i, backtrace_pos, &backtrace_pos_len,revcmpl, x, overlap_start, cluster_supported);
		}else{
			score_del = split_sw_del_square(best1first,best2first,glen,rlen, &i, backtrace_pos, &backtrace_pos_len,revcmpl, x, overlap_start,cluster_supported);
		}
		if(score_del>threshscore_del){
			sfr->score=score_del;
			k = do_backtrace(glen, i, backtrace_pos, backtrace_pos_len, sfr, 0);
			pretty_print_inv(sfr, k, cigar, x, inver_type,backtrace_pos[0]-1, xy_gap_size);
			sfr->qralign = qralign;
			sfr->type=sv_type;
			//to prevent the alignment in which head and tail overlaps is complicated,simply remove them here
			if (gap_size<=0){
				sfr->is_split_align=0;
			}else{

			}
		}else{
			//sfr->type=NOT_PRINT;
		}
		break;
	/*dup*/
	case DUP:
		//score_del = split_sw_del(glen, rlen, &i, backtrace_pos, &backtrace_pos_len,revcmpl, x, overlap_start, cluster_supported);//i is  rlen, j is genome pos
		if(do_full_alignment){
			score_del = split_sw_del(glen, rlen, &i, backtrace_pos, &backtrace_pos_len,revcmpl, x, overlap_start, cluster_supported);
		}else{
			score_del = split_sw_del_square(best1first,best2first,glen,rlen, &i, backtrace_pos, &backtrace_pos_len,revcmpl, x, overlap_start,cluster_supported);
		}
		if(score_del>threshscore_del){

			sfr->score=score_del;
			k = do_backtrace(glen, i, backtrace_pos, backtrace_pos_len, sfr, 0);
			pretty_print_dup(sfr, k, cigar, x, xy_gap_size, backtrace_pos[0]-1);

			sfr->qralign = qralign;
			sfr->type=sv_type;
			if (gap_size<=0){
				sfr->is_split_align=0;
			}

		}else{
			//sfr->type=NOT_PRINT;
		}
		break;
	default:
		fprintf(stderr,"invalid sv type: %i\n",sv_type);
				abort();
	}

	bestp=mylist;
	while(bestp){
		numfreed++;
		best1first=bestp->next;
		free(bestp);
		bestp=best1first;
	}

	swticks += (rdtsc() - before);
}



