
#include <assert.h>
#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <inttypes.h>
#include <math.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>
#include <zlib.h>
#include <omp.h>	// OMP multithreading
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>

#include "time.h"
#include "../common/fasta.h"
#include "../common/util.h"
#include "../smapper/smapper.h"
#include "../common/version.h"
#include "../common/sw_split_ls.h"
#include "../common/sw_split_cs.h"
#include "../common/sw-vector.h"
#include "../common/anchors.h"
#define DEBUG_OUTPUT_LOG
/* Scores */
static int	match_score		= DEF_MATCH_VALUE;
static int	mismatch_score		= DEF_MISMATCH_VALUE;
static int	a_gap_open_score	= DEF_A_GAP_OPEN;
static int	a_gap_extend_score	= DEF_A_GAP_EXTEND;
static int	b_gap_open_score	= DEF_B_GAP_OPEN;
static int	b_gap_extend_score	= DEF_B_GAP_EXTEND;
static int	crossover_score		= DEF_XOVER_PENALTY;
static uint32_t cur_1_thresh	= CUR_1_THRESH;
static uint32_t cur_2_thresh	= CUR_2_THRESH;
static uint32_t cur_3_thresh	= CUR_3_THRESH;
static uint32_t cur_4_thresh	= CUR_4_THRESH;
static int min_ins_match_len = MIN_INS_MATCH_LEN;
static int split_penalty_ins = SPLIT_PENALTY_INS;
static int split_penalty_del_double = SPLIT_PENALTY_DEL_DOUBLE;
static int split_penalty_del_large = SPLIT_PENALTY_DEL_LARGE;
static int kmer_len = 11;
/* files to use when saving and loading genome maps */
char *		save_file = NULL;
char *		load_file = NULL;

/* offset info for genome contigs */
static uint32_t *contig_offsets;
static char **contig_names = NULL;
static int	num_contigs;

/* Genomic sequence, stored in 32-bit words, first is in the LSB */
static uint32_t **genome_contigs;			/* genome -- always in letter */
static uint32_t	 *genome_len;
static int cur_contig_num;

static uint32_t xy_contig_size;
static uint32_t xy_contig_len;
static int8_t *xy_contig;

static bool      genome_is_rna = false;		/* is genome RNA (has uracil)?*/
/* read sequence */
struct read_entry * read_set;

/* seed management */
int			n_seeds = 0;
struct seed_type *	seed = NULL;
uint32_t * *		seed_hash_mask = NULL;
int			max_seed_span = 0;
int			min_seed_span = MAX_SEED_SPAN;
int			avg_seed_span = 0;

/*map split*/
static FILE *file_indel,*file_cluster_del,*file_dup,*file_inv,*file_unsplit,*file_header;

static struct sam_entry *tmp_sam;
static struct sam_entry *cur_1, *cur_2, *cur_3, *cur_4;
static uint32_t	cur_1_len, cur_2_len,cur_3_len,cur_4_len;
static int rlen,devi,mean,delta,max_rlen,max_cigar_len;//delta may be 3*devi or 5 devi depends on the distribution
static uint32_t min_dis, max_dis,test_region_len;
static uint32_t max_win_size, cur_1_size,cur_2_size,cur_3_size,cur_4_size;
static struct cnv_breakpoint *indel_bp, *inver_bp, *dup_bp;
static struct sw_full_results *sfr;
static int indel_bp_len, indel_bp_size,inver_bp_len, inver_bp_size,dup_bp_len, dup_bp_size;
//win_size is the size of extended cur window, win_size is not must larger than max_win_size, max_win_size is
//compared with real pos, while cur_2[win_size-1]-cur_2[win_size-2] may be larger than max_win_size

static uint32_t last_1_pos, cur_pos;// last_1_pos, last 1-mapped read's pos; cur_pos, current cur_1 cell's mapping pos
static int longest_read_len = 200;
static int max_xy_contig_size=2000;

static char *out_file_name, *cmd_line[50];
static int cmd_num;
static fasta_t fasta;
static int min_rlen;
//static int sv_type[5];
static bool find_del_clusters, find_inv_clusters, find_dup_clusters;
static int space_flag;//1 for letter, 2 for color

static bool align_small_indel, align_large_del, align_inver, align_dup;

/* statistics*/
static int single_read_num=0;
static llint total_time, plus_strand_time, minus_strand_time;
//TODO move this to utils
void reverse(char* s, char* t) {
       int l=strlen(s);
       int i;
       for (i=0; i<l; i++) {
               switch (s[i]) {
			   case 'A':
					   t[l-i-1]='T';
					   break;
			   case 'a':
					   t[l-i-1]='t';
					   break;
			   case 'C':
					   t[l-i-1]='G';
					   break;
			   case 'c':
					   t[l-i-1]='g';
					   break;
			   case 'G':
					   t[l-i-1]='C';
					   break;
			   case 'g':
					   t[l-i-1]='c';
					   break;
			   case 'T':
					   t[l-i-1]='A';
					   break;
			   case 't':
					   t[l-i-1]='a';
					   break;
			   case '-':
					   t[l-i-1]='-';
					   break;
			   default:
					   fprintf(stderr,"There has been a error in getting reverse complement of %s\n",s);
					   exit(1);
               }
       }
       t[l]='\0';
       strcpy(s,t);
}
/*
 * index the kmers in the genome contained in the file.
 * This can then be used to align reads against.
 */
static bool
load_genome(char **files, int nfiles)
{
	fasta_t fasta;
	size_t seqlen;
	uint32_t *read;
	char *seq, *name;
	//uint32_t *kmerWindow;
	//int sn;
	char *file;
	bool is_rna;




	num_contigs = 0;
	u_int i = 0;
	int cfile;
	for(cfile = 0; cfile < nfiles; cfile++){

		file = files[cfile];
		//open the fasta file and check for errors
		fasta = fasta_open(file,LETTER_SPACE);
		if (fasta == NULL) {
			fprintf(stderr,"error: failded to open genome file [%s]\n",file);
			return (false);
		} else {
			fprintf(stderr,"- Processing genome file [%s]\n",file);
		}

		//Read the contigs and record their sizes

		while(fasta_get_next(fasta, &name, &seq, &is_rna)){
			genome_is_rna = is_rna;
			num_contigs++;
			contig_offsets = (uint32_t *)xrealloc(contig_offsets,sizeof(uint32_t)*num_contigs);
			contig_offsets[num_contigs - 1] = i;
			contig_names = (char **)xrealloc(contig_names,sizeof(char *)*num_contigs);
			contig_names[num_contigs - 1] = name;

			fprintf(stderr,"- Processing contig %s\n",name);

			if (strchr(name, '\t') != NULL || strchr(seq, '\t') != NULL) {
				fprintf(stderr, "error: tabs are not permitted in fasta names "
						"or sequences. Tag: [%s].\n", name);
				return false;
			}

			seqlen = strlen(seq);
			if (seqlen == 0) {
				fprintf(stderr, "error: genome [%s] had no sequence!\n",
						name);
				return false;
			}


			read = fasta_sequence_to_bitfield(fasta,seq);

			if (read == NULL) {
				fprintf(stderr, "error: invalid sequence; tag: [%s]\n", name);
				return false;
			}
			genome_contigs = (uint32_t **)xrealloc(genome_contigs,sizeof(uint32_t *)*num_contigs);
			genome_contigs[num_contigs-1] = read;

			//genome_contigs_rc = (uint32_t **)xrealloc(genome_contigs_rc,sizeof(uint32_t *)*num_contigs);
			//genome_contigs_rc[num_contigs-1] = reverse_complement_read_ls(read,seqlen,is_rna);

			genome_len = (uint32_t *)xrealloc(genome_len,sizeof(uint32_t)*num_contigs);
			genome_len[num_contigs -1] = seqlen;



			free(seq);
			seq = name = NULL;


		}
		fasta_close(fasta);
	}

	fprintf(stderr,"Loaded Genome\n");

	return (true);

}

void
print_split(struct sam_entry cur_1, struct sw_full_results sfr){

	FILE *file;
	switch(sfr.type){
	case SMALL_INDEL:
		file=file_indel;
		break;
	case LARGE_DEL:
		file=file_cluster_del;
		break;
	//case SMALL_INS:
        //file=file_ins;
        break;
	case INVER_FW_OUT:
	case INVER_FW_IN:
	case INVER_BW_IN:
	case INVER_BW_OUT:
	case SMALL_INVER_FW:
	case SMALL_INVER_BW:
		file=file_inv;
		break;
	case DUP:
		file=file_dup;
		break;
	case UNSPLIT:
		file=file_unsplit;
		break;
	case NOT_PRINT:
		return;
	default:
		fprintf(stderr,"sv type:%i\n",sfr.type);
		abort();

	}
	if(file==NULL){
		fprintf(stderr,"output file open failed");
		exit(0);
	}


	if(sfr.type==UNSPLIT){
		fprintf(file,"%s\t",cur_1.rname);

		//flag
		fprintf(file,"%i\t",cur_1.strand);

		//genome name
		fprintf(file,"%s\t",cur_1.genome_name);

		//pos
		fprintf(file,"%i\t",cur_1.pos);

		//mapq
		fprintf(file,"%i\t",cur_1.mapq);

		//cigar
		fprintf(file,"%s\t",cur_1.cigar);

		//mate ref name
		fprintf(file,"%s\t",cur_1.mate_ref_name);

		//mpos
		fprintf(file,"%i\t",cur_1.mpos);

		//isize
		fprintf(file,"%i\t",cur_1.isize);

		//seq
		fprintf(file,"%s\t", cur_1.seq);

		//qval
		fprintf(file,"%s\t", cur_1.qval);
		//mseq
		fprintf(file,"R2:Z:%s\n",cur_1.mseq);


	}else{
		fprintf(file,"%s\t",cur_1.rname);
		//flag
		fprintf(file,"%i\t",(cur_1.strand & 0x10) == 0? 0x10:0x0);
		//genome name
		fprintf(file,"%s\t",cur_1.genome_name);
		//pos
		fprintf(file,"%i\t",sfr.genome_start);
		//mapq
		fprintf(file,"%i\t",255);
		//cigar
		fprintf(file,"%s\t",sfr.cigar);
		//mate ref name
		fprintf(file,"%s\t","=");
		//mpos
		fprintf(file,"%i\t",cur_1.pos);
		//isize
		fprintf(file,"%i\t",0);
		//seq
		fprintf(file,"%s\t", sfr.qralign);
		//qval
		int t;
		for(t=0;t<rlen;t++){
			fprintf(file,"%c",'I');
		}
		fprintf(file,"\t");
		//score
		fprintf(file,"AS:i:%i\t",sfr.score);
		//sv type
		fprintf(file,"ST:i:%i\t", sfr.type);
		//mate MAPQ
		fprintf(file,"MQ:i:%i\t", cur_1.mapq);
		//multi aln pos
		fprintf(file,"X0:i:%i", sfr.multi_aln);
		//mseq
		//fprintf(file,"R2:Z:%s",cur_1.seq);
		fprintf(file,"\n");

	}
	//fclose(file);
}

int
map_mate(int strand, uint32_t x, uint32_t goff_x, uint32_t goff_y, int min_gap_size, int type,bool cluster_supported){

	uint32_t i;

	int thresh_ins, thresh_del;
	uint32_t *read,*read_fw=NULL,*read_rc=NULL;
	char cigar[max_cigar_len];
	struct sw_full_results scratch, *cur_sfr;
	cur_sfr=&scratch;

	//memset(sfr,0,1*sizeof(*sfr));


	for(i=0;i<cur_1_len;i++){

		/*randomly change N to "ACGT"*/
		int p;
		int n_N=0;
		srand((unsigned)time(0));
		while((p = strcspn(cur_1[i].mseq,"N."))!=strlen(cur_1[i].mseq) && n_N<5){

			int a = rand()%4;

			switch(a){
			case 0:
				if(space_flag==LETTER_SPACE){
					cur_1[i].mseq[p] = 'A';
				}else{
					cur_1[i].mseq[p] = '0';
				}
				break;
			case 1:
				if(space_flag==LETTER_SPACE){
					cur_1[i].mseq[p] = 'C';
				}else{
					cur_1[i].mseq[p] = '1';
				}
                		break;
			case 2:
				if(space_flag==LETTER_SPACE){
                			cur_1[i].mseq[p] = 'G';
				}else{
					cur_1[i].mseq[p] = '2';
				}
                		break;
			case 3:
				if(space_flag==LETTER_SPACE){
                			cur_1[i].mseq[p] = 'T';
				}else{
					cur_1[i].mseq[p] = '3';
				}
                		break;	
			default:
				fprintf(stderr,"rand error:%i\n",a);
				abort();
			}
			n_N++;
		}
		if(n_N==4){
			continue;
		}
		char *s=xstrdup(cur_1[i].mseq);
		rlen=strlen(cur_1[i].mseq);
		thresh_ins = rlen*match_score-split_penalty_ins-(rlen-min_ins_match_len)*match_score;
		if(type==SMALL_DEL){
			thresh_del = (rlen*match_score-split_penalty_del_double)*0.6;//=599
		}else{
			thresh_del = (rlen*match_score-split_penalty_del_large)*0.6;//=599
		}
		int overlap_start;
		if(min_gap_size<0){
				overlap_start=x+min_gap_size+1;
		}else{
				overlap_start=x+1;
		}
		if(space_flag==MODE_LETTER_SPACE){//letter space
			read_fw=fasta_sequence_to_bitfield(fasta, cur_1[i].mseq);
			read_rc=reverse_complement_read_ls(read_fw, rlen, 0);
			if((cur_1[i].strand & 0x10) == 0){
					read=read_rc;
					reverse_complement_read_ls_text(cur_1[i].mseq,s);
			}else{
					read=read_fw;
			}

			sw_split_ls(xy_contig, xy_contig_len, read, rlen, thresh_ins, thresh_del, cur_sfr, 1-strand, x, cigar, min_gap_size, type, overlap_start,cluster_supported);
		}else if(space_flag==MODE_COLOUR_SPACE){//color space

			int initbp;
			rlen--;
			initbp=fasta_get_initial_base(fasta,cur_1[i].mseq);
			//printf("cs read:%s\n",cur_1[i].mseq);

			read_fw= fasta_sequence_to_bitfield(fasta, cur_1[i].mseq);
			read_rc=reverse_complement_read_cs(read_fw, (int8_t)initbp, (int8_t)initbp, rlen, 0);
			if((cur_1[i].strand & 0x10) == 0){
					//read_rc=reverse_complement_read_cs(read, (int8_t)initbp, (int8_t)initbp, rlen, 0);
					read=read_rc;
			}else{
					read=read_fw;
			}
			sw_split_cs(xy_contig, xy_contig_len, read, rlen, initbp, thresh_ins, thresh_del,cur_sfr, 1-strand, x, cigar, min_gap_size, type, overlap_start);

		}

		if(cur_sfr->genome_start>x){
                        cur_sfr->genome_start+=(goff_y-x);
                }else{
                        cur_sfr->genome_start+=goff_x;
                }		
#ifdef DEBUG_OUTPUT_LOG
        printf("name:%s\n",cur_1[i].rname);
        printf("min gap size:%i\n",min_gap_size);
        printf("over lap start:%i\n",overlap_start);
        printf("read to be aligned:%s\n",s);
	printf("%s score %i:%i\tcigar:%s\tpos:%i\n",cur_1[i].rname,type,cur_sfr->score,cigar, cur_sfr->genome_start);
#endif

		if(cur_sfr->is_split_align && cur_sfr->score>=sfr[i].score){
			sfr[i].is_split_align=cur_sfr->is_split_align;
			sfr[i].multi_aln=cur_sfr->multi_aln;
			sfr[i].score=cur_sfr->score;
			sfr[i].type=cur_sfr->type;
			sfr[i].genome_start=cur_sfr->genome_start;
			strcpy(sfr[i].cigar,cigar);
			if(type/10 ==INVER || type==DUP || space_flag==MODE_COLOUR_SPACE){
				strcpy(sfr[i].qralign,cur_sfr->qralign);
			}else{
				strcpy(sfr[i].qralign,s);
			}
		}
		free(read_rc);
                free(read_fw);
		free(s);

	}

	return 0;
}
int
process_mate(int strand){
	uint32_t pos1, pos2, goff_x, goff_y;
	uint32_t mid_pos_1;
	int min_gap_size;
	uint32_t i=0,j=0;
	uint32_t x,y;
	int type=0;
	//memset(sfr,0,cur_1_thresh*(sizeof(sfr[0])));
	for(i=0;i<cur_1_thresh;i++){
		sfr[i].score=0;
		sfr[i].is_split_align=false;
	}
	mid_pos_1=cur_1[0].pos;
	//rlen=max_rlen;//strlen(cur_1[0].mseq);
	if(align_small_indel){

		if(strand){
			pos1 = mid_pos_1-max_dis;//2*(mean+delta);
			pos2 = mid_pos_1-min_dis;//mid_pos_1-(mean-delta-rlen)+rlen;
		}else{
			pos1 = mid_pos_1+max_rlen+min_dis;//+(mean-delta)-rlen;
			pos2 = mid_pos_1+max_rlen+max_dis;//2*(mean+delta);
		}
		//generate xy_contig
		min_gap_size=-1*(pos2-pos1);
		x=pos2-pos1;
		xy_contig_len= x + x;
		if(pos1 < 0 || pos2 > genome_len[0]-1){
                	return 0;
  		}

		while(xy_contig_size<2*(pos2-pos1)){
			xy_contig=(int8_t *)xrealloc(xy_contig, sizeof(int8_t) * 2*(pos2-pos1));
			xy_contig_size=2*(pos2-pos1);
		}



		for(i=pos1,j=0;i<pos2;i++,j++){
			xy_contig[j] = (int8_t)EXTRACT(genome_contigs[cur_contig_num],i);
		}

		for(i=pos1;i<pos2;i++,j++){
			xy_contig[j] = (int8_t)EXTRACT(genome_contigs[cur_contig_num],i);
		}
#ifdef DEBUG_OUTPUT_LOG
	printf("pos1:%i\tpos2:%i\n",pos1,pos2);
#endif

		goff_x=pos1;
		goff_y=pos1;//in fact this should never be used, because x & y are the same here
		/*
		type=SMALL_DEL;//small del
		map_mate(strand, x, goff_x, goff_y, min_gap_size, type,false);
		type=SMALL_INS;//small ins
		xy_contig_len/=2;
		min_gap_size=0;
		map_mate(strand, x, goff_x, goff_y, min_gap_size, type,false);
		*/
		type=SMALL_INDEL;
		map_mate(strand, x, goff_x, goff_y, min_gap_size, type,false);


	}
	/*large del*/
	if(align_large_del && find_del_clusters){//large del
		type=LARGE_DEL;
		uint32_t k;
		for(k=0;k<cur_2_len;k++){
			pos1 = cur_2[k].pos;
			pos2 = cur_2[k].mpos+rlen;

			//if(1 || pos2-pos1 >= 2*(mean+delta)){//3*delta + mean + rlen){//large del, do not need double
			if(pos1 < test_region_len || pos2 + test_region_len> genome_len[0]-1){
				//goto nextloop;'
				continue;
			}

			pos1-=test_region_len;//mean+delta;
			pos2+=test_region_len;//mean+delta;
#ifdef DEBUG_OUTPUT_LOG
	printf("pos1:%i\tpos2:%i\t",pos1,pos2);
	printf("cluster pos1:%i\t cluster pos2:%i\tcluster region len:%i\n",cur_2[k].pos,cur_2[k].mpos,cur_2[k].mpos-cur_2[k].pos+1);
#endif
			x = 2*test_region_len;//2*(mean + delta) ;//2*delta+rlen;
			y = 2*test_region_len;//2*(mean + delta) ;
			min_gap_size = pos2 - pos1 - x - y;
			xy_contig_len = x + y;

			assert(xy_contig_len <= max_xy_contig_size);

			if(xy_contig_size < xy_contig_len){
				xy_contig=(int8_t *)xrealloc(xy_contig, sizeof(int8_t) * xy_contig_len);
				xy_contig_size=xy_contig_len;
			}

			for(i=pos1,j=0;i<pos1+x;i++,j++){
				xy_contig[j] = (int8_t)EXTRACT(genome_contigs[cur_contig_num],i);
			}

			for(i=pos2-y;i< pos2;i++,j++){
				xy_contig[j] = (int8_t)EXTRACT(genome_contigs[cur_contig_num],i);
			}
			goff_x=pos1;
			goff_y=pos2-y;

			map_mate(strand, x, goff_x, goff_y, min_gap_size, type,true);


			//}
		}
	}
	/*inver*/
	if(align_inver && find_inv_clusters){

		uint32_t k;
		for(k=0;k<cur_3_len;k++){
			pos1 = cur_3[k].pos;
			pos2 = cur_3[k].mpos;
			if(pos1 < test_region_len || pos2 + test_region_len> genome_len[0]-1){
				//goto nextloop;'
				continue;
			}
			
			x = 2*test_region_len;//2*(mean + delta) ;//2*delta+rlen;
			y = 2*test_region_len;//2*(mean + delta) ;

			xy_contig_len = x + y;

			assert(xy_contig_len <= max_xy_contig_size);


			if(xy_contig_size < xy_contig_len){
				xy_contig=(int8_t *)xrealloc(xy_contig, sizeof(int8_t) * xy_contig_len);
				xy_contig_size=xy_contig_len;
			}
			if(!strand){//+strand
				/*
				if(pos2-pos1<2*test_region_len){
					x = pos2 - pos1;
					y = pos2 - pos1;
					//min_gap_size = pos2 - pos1 - x ;
					xy_contig_len = x + y;
				}else{

					pos1 -= test_region_len;//mean + delta;
					pos2 -= test_region_len;//mean + delta;

				}*/
				pos1 -= test_region_len;//mean + delta;
				pos2 -= test_region_len;//mean + delta;
#ifdef DEBUG_OUTPUT_LOG
	printf("pos1:%i\tpos2:%i\n",pos1,pos2);
#endif
				min_gap_size = pos2 - pos1 - x ;
				if( cur_3[k].type == 1){//cur read is in bp1 region
					for(i=pos1,j=0;i<pos1+x;i++,j++){
						xy_contig[j] = (int8_t)EXTRACT(genome_contigs[cur_contig_num],i);

					}

					for(i=pos2+x;i> pos2;i--,j++){

						xy_contig[j] = (int8_t)(3-(int8_t)EXTRACT(genome_contigs[cur_contig_num],i));

					}
					goff_x=pos1;
					goff_y=pos2+1;
					type=INVER_FW_OUT;
				}else if(cur_3[k].type == 2){//cur read is in bp2 region
					for(i=pos2,j=0;i< pos2+x;i++,j++){
						xy_contig[j] = (int8_t)EXTRACT(genome_contigs[cur_contig_num],i);
					}
					for(i=pos1+x;i>pos1;i--,j++){
						xy_contig[j] = (int8_t)(3-(int8_t)EXTRACT(genome_contigs[cur_contig_num],i));
					}
					goff_x=pos1+1;
					goff_y=pos2;
					type=INVER_FW_IN;

					/*	
					if(pos2 - pos1 > 2*test_region_len){//large inver
						for(i=pos2,j=0;i< pos2+x;i++,j++){
							xy_contig[j] = (int8_t)EXTRACT(genome_contigs[cur_contig_num],i);
						}
						for(i=pos1+x;i>pos1;i--,j++){
							xy_contig[j] = (int8_t)(3-(int8_t)EXTRACT(genome_contigs[cur_contig_num],i));
						}
						goff_x=pos1+1;
						goff_y=pos2;
						type=INVER_FW_IN;
					}else{//small inver

						for(i=pos1+x,j=0;i>pos1;i--,j++){
							xy_contig[j] = (int8_t)(3-(int8_t)EXTRACT(genome_contigs[cur_contig_num],i));
						}
						for(i=pos2;i<pos2+x;i++,j++){
							xy_contig[j] = (int8_t)EXTRACT(genome_contigs[cur_contig_num],i);
						}
						goff_x=pos1+1;
						goff_y=pos2;
						type=SMALL_INVER_FW;
					}*/

				}else{
					fprintf(stderr,"invalid inversion type %i.\n",cur_3[k].type);
					abort();
				}
			}else{//-strand
				/*
				if(0 && pos2-pos1<2*test_region_len){
					pos1+=rlen;
					pos2+=rlen;
					x = pos2 - pos1;
					y = pos2 - pos1;
					//min_gap_size = pos2 - pos1 - x ;
					xy_contig_len = x + y;
				}else{
					//pos1+=test_region_len;//mean+delta;
					//pos2+=test_region_len;//mean+delta;
				}*/
				pos1+=test_region_len;//mean+delta;
				pos2+=test_region_len;//mean+delta;
#ifdef DEBUG_OUTPUT_LOG
	printf("pos1:%i\tpos2:%i\tx:%i\n",pos1,pos2,x);
#endif
				min_gap_size = pos2 - pos1 - x ;
				if(cur_3[k].type == 1){//cur read is in bp1 region
					if(1 || pos2 - pos1 > test_region_len){
						for(i=pos2,j=0;i>pos2-x;i--,j++){
							xy_contig[j] = (int8_t)(3-(int8_t)EXTRACT(genome_contigs[cur_contig_num],i));
						}

						for(i=pos1-x;i< pos1;i++,j++){
							xy_contig[j] = (int8_t)EXTRACT(genome_contigs[cur_contig_num],i);
						}
						goff_x=pos1-x;
						goff_y=pos2-x+1;
						type=INVER_BW_IN;
					}else{

						for(i=pos1-x,j=0;i<pos1;i++,j++){
								xy_contig[j] = (int8_t)EXTRACT(genome_contigs[cur_contig_num],i);
						}
						for(i=pos2;i> pos2-x;i--,j++){
								xy_contig[j] = (int8_t)(3-(int8_t)EXTRACT(genome_contigs[cur_contig_num],i));
						}
						goff_x=pos1-x;
						goff_y=pos2-x+1;
						type=SMALL_INVER_BW;
					}

				}else if(cur_3[k].type == 2){//cur read is in bp2 region
					for(i=pos1,j=0;i>pos1-x;i--,j++){
						xy_contig[j] = (int8_t)(3-(int8_t)EXTRACT(genome_contigs[cur_contig_num],i));
					}
					for(i=pos2-x;i<pos2;i++,j++){
						xy_contig[j] = (int8_t)EXTRACT(genome_contigs[cur_contig_num],i);

					}
					goff_x=pos1-x+1;
					goff_y=pos2-x;
					type=INVER_BW_OUT;
				}else{
					fprintf(stderr,"invalid inversion type %i.\n",cur_3[k].type);
					abort();
				}
			}
			map_mate(strand, x, goff_x, goff_y, min_gap_size, type,true);

			//pos1 = cur_3[k].pos;
			//pos2 = cur_3[k].mpos;
			if(  pos2 - pos1 < 2*test_region_len){// a small inver
				//x = test_region_len + (pos2-pos1);
				//pos2 = pos1;
				//pos1 -= test_region_len;
				//min_gap_size = pos2 - pos1 - x ;
				if(type==INVER_FW_IN){
					for(i=pos1+x,j=0;i>pos1;i--,j++){
						xy_contig[j] = (int8_t)(3-(int8_t)EXTRACT(genome_contigs[cur_contig_num],i));
					}
					for(i=pos2;i<pos2+x;i++,j++){
						xy_contig[j] = (int8_t)EXTRACT(genome_contigs[cur_contig_num],i);
					}
					goff_x=pos1+1;
					goff_y=pos2;
					type=SMALL_INVER_FW;
					map_mate(strand, x, goff_x, goff_y, min_gap_size, type,true);
				}
				if(type==INVER_BW_IN){

					for(i=pos1-x,j=0;i<pos1;i++,j++){
							xy_contig[j] = (int8_t)EXTRACT(genome_contigs[cur_contig_num],i);
					}
					for(i=pos2;i> pos2-x;i--,j++){
							xy_contig[j] = (int8_t)(3-(int8_t)EXTRACT(genome_contigs[cur_contig_num],i));
					}
					goff_x=pos1-x;
					goff_y=pos2-x+1;
					type=SMALL_INVER_BW;
					map_mate(strand, x, goff_x, goff_y, min_gap_size, type,true);
				}
			}
		}
	}
	/*dup*/
	if(align_dup && find_dup_clusters){
		type=DUP;
		uint32_t k;
		for(k=0;k<cur_4_len;k++){

			
			pos1 = cur_4[k].pos+rlen;
			pos2 = cur_4[k].mpos;
			if(pos1 < test_region_len || pos2 + test_region_len> genome_len[0]-1){
				//goto nextloop;'
				continue;
			}
#ifdef DEBUG_OUTPUT_LOG
        printf("pos1:%i\tpos2:%i\n",pos1,pos2);
#endif
			if(pos2-pos1 >= 2*test_region_len){
				pos1 += test_region_len;
				pos2 -= test_region_len;
				x = 2*test_region_len;

				min_gap_size=pos2-pos1;
				xy_contig_len=x + x;
				if(xy_contig_size<xy_contig_len){
						xy_contig=(int8_t *)xrealloc(xy_contig, sizeof(int8_t) * xy_contig_len);
						xy_contig_size=xy_contig_len;
				}

				for(i=pos2,j=0;i< pos2+x;i++,j++){

						xy_contig[j] = (int8_t)EXTRACT(genome_contigs[cur_contig_num],i);

				}
				for(i=pos1-x;i<pos1;i++,j++){
						xy_contig[j] = (int8_t)EXTRACT(genome_contigs[cur_contig_num],i);

				}
				goff_x=pos1-x;
				goff_y=pos2;
				map_mate(strand, x, goff_x, goff_y, min_gap_size, type,true);
			}
		}
	}
	/*print alignment*/
	for(i=0;i<cur_1_len;i++){
		if(sfr[i].is_split_align){
			if(sfr[i].type/10 ==INVER || sfr[i].type==DUP){
				print_split(cur_1[i], sfr[i]);
			}else{
				print_split(cur_1[i], sfr[i]);
			}
		}else{
			sfr[i].type=UNSPLIT;
			print_split(cur_1[i], sfr[i]);

		}
	}
	return 1;
}


void
map_split_setup(){

	tmp_sam=(struct sam_entry*)malloc(sizeof(struct sam_entry));
	memset(tmp_sam,0,sizeof(sam_entry));

	//TODO cover and rlen should be from args
	max_rlen=100;
	max_cigar_len=200;
	//split_penalty=5*(DEF_MATCH_VALUE-DEF_MISMATCH_VALUE)+1;
	split_penalty_ins= SPLIT_PENALTY_INS;
	split_penalty_del_large=SPLIT_PENALTY_DEL_LARGE;
	split_penalty_del_double=SPLIT_PENALTY_DEL_DOUBLE;

	//delta=5*devi;//the range of length between two mates

	max_win_size=2*delta;//(rlen+2*delta);// there is  no 1-mapped reads in a range of max_win_size after the last 1-mapped read we met in current round

	cur_1_size=1000;//the amount of mem we alloc for current window
	cur_2_size=1000;
	cur_3_size=1000;
	cur_4_size=1000;

	cur_1_len=0;// the number of cells of cur1 array those are wrote.
	cur_2_len=0;// the number of cells of cur2 array those are wrote.
	cur_3_len=0;
	cur_4_len=0;

	cur_1=(struct sam_entry*)xcalloc((cur_1_size)* sizeof(struct sam_entry));
	cur_2=(struct sam_entry*)xcalloc((cur_2_size)* sizeof(struct sam_entry));
	cur_3=(struct sam_entry*)xcalloc((cur_3_size)* sizeof(struct sam_entry));
	cur_4=(struct sam_entry*)xcalloc((cur_4_size)* sizeof(struct sam_entry));


	xy_contig_size=2000;//(mean+delta);//2*(3*delta+mean+max_rlen);
	xy_contig=(int8_t*)calloc(1,xy_contig_size*sizeof(int8_t));

	cur_contig_num=0;
	out_file_name=(char*)xcalloc(1024*sizeof(char));

	space_flag=MODE_LETTER_SPACE;

	indel_bp_size=10000;
	indel_bp_len=0;
	indel_bp=(struct cnv_breakpoint*)xcalloc(indel_bp_size*(sizeof(struct cnv_breakpoint)));
	inver_bp_size=10000;
	inver_bp_len=0;
	inver_bp=(struct cnv_breakpoint*)xcalloc(inver_bp_size*(sizeof(struct cnv_breakpoint)));
	dup_bp_size=10000;
	dup_bp_len=0;
	dup_bp=(struct cnv_breakpoint*)xcalloc(dup_bp_size*(sizeof(struct cnv_breakpoint)));

	align_small_indel=true;
	align_large_del=true;
	align_inver=true;
	align_dup=true;

	//fasta=fasta_open2(space_flag);

	sfr=(struct sw_full_results*)xcalloc(cur_1_thresh*(sizeof(struct sw_full_results)));
	uint32_t i;
	for(i=0;i<cur_1_thresh;i++){
		sfr[i].qralign= (char *)xcalloc(max_rlen* sizeof(char));
		sfr[i].cigar= (char *)xcalloc(max_cigar_len* sizeof(char));
	}

	min_dis=0;
	test_region_len=500;
	min_rlen=100;
}

void
save_sam_entry(struct sam_entry* cur, int cur_len, sam_t sam, int sam_field_len){
	int i;

	i=cur_len;

	strcpy(cur[i].rname, sam->sam_field[0]);

	cur[i].strand = atoi(sam->sam_field[1]);

	strcpy(cur[i].genome_name, sam->sam_field[2]);

	cur[i].pos = atoi(sam->sam_field[3]);

	cur[i].mapq = atoi(sam->sam_field[4]);

	strcpy(cur[i].cigar, sam->sam_field[5]);

	strcpy(cur[i].mate_ref_name, sam->sam_field[6]);

	cur[i].mpos = atoi(sam->sam_field[7]);

	cur[i].isize = atoi(sam->sam_field[8]);

	strcpy(cur[i].seq, sam->sam_field[9]);

	strcpy(cur[i].qval, sam->sam_field[10]);

	//strcpy(cur[i].mseq, sam->sam_field[sam_field_len-1]);
	int j;
        char *tok;
        tok=NULL;
        for(j=11;j<sam_field_len;j++){
		
        	tok=strstr(sam->sam_field[j],"R2");
                if(tok!=NULL){
			tok+=5;
			char* p;
			p=strchr(sam->sam_field[j],'\n');
			if(p!=NULL){
				*p='\0';
			}
			strcpy(cur[i].mseq, tok);
			break;				
                }
        }
	if(tok==NULL){
		fprintf(stderr,"No R2 field in the sam line:%s\n",sam->buffer1);
		abort();
	}


}
int
get_sam_1(sam_t sam, int strand){

	int sam_field_len;

	bool read=false;
#ifdef DEBUG_OUTPUT_LOG
	printf("cur pos:%i\n",last_1_pos);
#endif
	
	while(read_sam(sam)){

		if(sam->buffer1[0]=='@'){
			continue;
		}
		if(sam->buffer1[0]=='\0'){
			continue;
		}
		parse_sam_field(sam->buffer1, sam->sam_field, &sam_field_len);
		if((atoi(sam->sam_field[1]) & 0x10 )== 0x10*(1-strand)){
			continue;
		}
		if((atoi(sam->sam_field[1]) & 0x04)==0x04){
			continue;
		}

		single_read_num++;
		read=true;
		if(single_read_num>0 && single_read_num  % 100000 == 0){
			fprintf(stderr,"%i reads processed.\n",single_read_num);
		}
/*
		tok=strtok(sam->sam_field[sam_field_len-1],":");
		while(tok!=NULL){
			strcpy(sam->sam_field[sam_field_len],tok);
			tok = strtok (NULL, ":");
			sam_field_len++;
		}
		sam->sam_field[sam_field_len-1]=strtok(sam->sam_field[sam_field_len-1],"\n");
*/
		if(atoi(sam->sam_field[3])==last_1_pos){
			//save current 1-read to cur_1
			save_sam_entry(cur_1, cur_1_len, sam,sam_field_len);
			if(cur_1_len < cur_1_thresh){
				cur_1_len++;
			}
			if(cur_1_len==cur_1_size){
				cur_1_size*=2;
				cur_1=(struct sam_entry*)xrealloc(cur_1, (cur_1_size)*sizeof(struct sam_entry));
				fprintf(stderr,"cur 1 extended. size:%i\tpos:%s\n",cur_1_size,sam->sam_field[3]);
			}

		}else{

			save_sam_entry(tmp_sam, 0, sam, sam_field_len);
			cur_pos=last_1_pos;
			last_1_pos=atoi(sam->sam_field[3]);
			
			break;

		}

	}
	return read;
}
int
get_discordant_break_point(int strand){//sv_type:0-indel,1-ins,2-del,3-inver,4-dup
	//example: RANDOM_GENOME_1010000   7682    1007776 0       53      527     514     1.0007e+06      1       EDGE
	int i;
	uint32_t bp1,bp2,left_bound,right_bound;

	find_del_clusters=false;
	find_inv_clusters=false;
	find_dup_clusters=false;

	cur_2_len=0;
	cur_3_len=0;
	cur_4_len=0;
	rlen=strlen(cur_1[0].mseq);
	/*get del bp*/
	for(i=0;i<indel_bp_len;i++){
		bp1=indel_bp[i].bp1 ;
		bp2=indel_bp[i].bp2 ;
		if(!strand){
			left_bound = bp1-max_dis;//bp1-2*(mean+delta);
			right_bound= bp1-min_dis;//(bp1+rlen)-(mean-rlen-delta);
			if(bp1<max_dis || bp2<max_dis){
				continue;	
			}
			if(cur_pos < left_bound){
				break;
			}else if(cur_pos <= right_bound){
				//printf("links bp1:%i\tlinks bp2:%i\n",bp1,bp2);
				cur_2[cur_2_len].pos=bp1;
				cur_2[cur_2_len].mpos=bp2;
				cur_2_len++;
				find_del_clusters=true;

			}
		}else{
			left_bound = bp2+min_dis;//bp2+rlen;//(bp2-rlen)+(mean-rlen-delta);
			right_bound= bp2+max_dis;//bp2+2*(mean+delta);
			if(cur_pos < left_bound){
				break;
			}else if(cur_pos <= right_bound){
				//printf("links bp1:%i\tlinks bp2:%i\n",bp1,bp2);
				cur_2[cur_2_len].pos=bp1;
				cur_2[cur_2_len].mpos=bp2;
				cur_2_len++;
				find_del_clusters=true;

			}
		}
		if(cur_2_len>cur_2_thresh){
			cur_2_len=cur_2_thresh;
		}


	}
	/*get inver bp*/
	for(i=0;i<inver_bp_len;i++){
		bp1=inver_bp[i].bp1 ;
		bp2=inver_bp[i].bp2 ;

		if(!strand){
			left_bound = bp1-max_dis;//bp1-2*(mean+delta);
			right_bound= bp1-min_dis;//bp1;//(bp1+rlen)-(mean-rlen-delta);
			if(bp1<max_dis || bp2<max_dis){
				continue;	
			}
			if(cur_pos < left_bound){
				break;
			}else if(cur_pos <= right_bound){

				cur_3[cur_3_len].pos=bp1;
				cur_3[cur_3_len].mpos=bp2;
				cur_3[cur_3_len].type=1;//cur read in bp1 region
				cur_3_len++;
				find_inv_clusters=true;

			}
			left_bound = bp2-max_dis;//bp2-2*(mean+delta);
			right_bound= bp2-min_dis;//bp2;//(bp1+rlen)-(mean-rlen-delta);

			if(cur_pos < left_bound){
				//break;
			}else if(cur_pos <= right_bound){

				cur_3[cur_3_len].pos=bp1;
				cur_3[cur_3_len].mpos=bp2;
				cur_3[cur_3_len].type=2;//cur read in bp2 region
				cur_3_len++;
				find_inv_clusters=true;
			}

		}else{
			left_bound = bp1+min_dis;//bp1+rlen;//(bp2-rlen)+(mean-rlen-delta);
			right_bound= bp1+max_dis;//bp1+2*(mean+delta);
			if(cur_pos < left_bound){
				break;
			}else if(cur_pos <= right_bound){

				cur_3[cur_3_len].pos=bp1;
				cur_3[cur_3_len].mpos=bp2;
				cur_3[cur_3_len].type=1;//cur read in bp2 region
				cur_3_len++;
				find_inv_clusters=true;
			}
			left_bound = bp2+min_dis;//bp2+rlen;//(bp2-rlen)+(mean-rlen-delta);
			right_bound= bp2+max_dis;//bp2+2*(mean+delta);
			if(cur_pos < left_bound){
				//break;
			}else if(cur_pos <= right_bound){

				cur_3[cur_3_len].pos=bp1;
				cur_3[cur_3_len].mpos=bp2;
				cur_3[cur_3_len].type=2;//cur read in bp2 region
				cur_3_len++;
				find_inv_clusters=true;
			}

		}
		if(cur_3_len>cur_3_thresh){
                        cur_3_len=cur_3_thresh;
                }

	}
	/*dup*/
	for(i=0;i<dup_bp_len;i++){
		bp1=dup_bp[i].bp1;
		bp2=dup_bp[i].bp2;

		if(strand){
			left_bound = bp1+min_dis;//bp1+rlen;//(bp2-rlen)+(mean-rlen-delta);
			right_bound= bp1+max_dis;//bp1+2*(mean+delta);
			if(bp1<max_dis || bp2<max_dis){
				continue;	
			}

			if(cur_pos < left_bound){
				break;
			}else if(cur_pos <= right_bound){

				cur_4[cur_4_len].pos=bp1;
				cur_4[cur_4_len].mpos=bp2;
				cur_4_len++;
				find_dup_clusters=true;
				//discordant=true;
				//break;
			}
		}else{
			left_bound = bp2-max_dis;//bp2-2*(mean+delta);
			right_bound= bp2-min_dis;//bp2;//(bp1+rlen)-(mean-rlen-delta);
			if(cur_pos < left_bound){
				break;
			}else if(cur_pos <= right_bound){

				cur_4[cur_4_len].pos=bp1;
				cur_4[cur_4_len].mpos=bp2;
				cur_4_len++;
				find_dup_clusters=true;

			}
		}
		if(cur_4_len>cur_4_thresh){
                        cur_4_len=cur_4_thresh;
                }
	}
	return 0;
}

void
map_split(char* single_file, int strand){
	sam_t sam1;
	int i,j;

	sam1=sam_open(single_file);
	if(sam1 == NULL){
		fprintf(stderr,"singleton read file open failed.\n");
		abort();
	}
	/*initialize*/
	i=0;
	j=0;
	cur_1_len=0;
	cur_2_len=0;
	cur_3_len=0;

	cur_pos=0;//this is a coordinate of the genome, it is from the map pos in SAM file

	last_1_pos=0;//it is always seemed that last_1_pos is just 1 base before the first pos to check at the beginning

	//main loop
	while(get_sam_1(sam1,strand)){//read singleton read

		if(cur_1_len>cur_1_thresh){
			cur_1_len=cur_1_thresh;
		}

		get_discordant_break_point(strand);

		/*cur_1 used to be a window, but is 1 coordinate at present*/

		/*single_read_num+=cur_1_len;
		if(single_read_num % 100000 == 0){
			fprintf(stderr,"%i reads processed.\n",single_read_num);
		}*/

		process_mate(strand);

		/*prepair for the next loop*/
		memcpy(cur_1,tmp_sam,sizeof(struct sam_entry));
		cur_1_len=1;
		//TODO the read in tmp_sam should be dealed with when it is the end of sam1
	}

	/*process the last 1*/
	if(last_1_pos>cur_pos){
		get_discordant_break_point(strand);

		//is_ls: 0 for letter space 1 for color space
		//single_read_num+=cur_1_len;
		process_mate(strand);
		//map_mate(max_win_start, max_win_end, x, goff_x,goff_y,goff_1,goff_2, min_gap_size, strand);
	}

}

void
load_breakpoint_list(char *large_gap_read_file, char *inver_read_file,char *dup_read_file){

	sam_t sam;
	int sam_field_len;
	indel_bp_len=0;
	inver_bp_len=0;
	dup_bp_len=0;
	uint32_t bp1,bp2;

	/*indel bp list*/
	sam = sam_open(large_gap_read_file);

	if(sam != NULL){
		while(read_sam(sam)){
			parse_sam_field(sam->buffer1, sam->sam_field, &sam_field_len);
			bp1=atoi(sam->sam_field[1]);
			bp2=atoi(sam->sam_field[2]);
			if(bp2>bp1){
				indel_bp[indel_bp_len].bp1=bp1;//=atoi(sam->sam_field[1]);
				indel_bp[indel_bp_len].bp2=bp2;//=atoi(sam->sam_field[2]);
				indel_bp_len++;
				if(indel_bp_len == indel_bp_size){
					indel_bp_size*=2;
					indel_bp=(struct cnv_breakpoint*)xrealloc(indel_bp,indel_bp_size*sizeof(struct cnv_breakpoint));
				}
			}

		}
		sam_close(sam);
	}else{
		fprintf(stderr,"No indel bp list file specified.\n");
	}

	/*inver bp list*/
	sam = sam_open(inver_read_file);
	if(sam != NULL){
		while(read_sam(sam)){
			parse_sam_field(sam->buffer1, sam->sam_field, &sam_field_len);
			bp1=atoi(sam->sam_field[1]);
                        bp2=atoi(sam->sam_field[2]);
                        if(bp2>bp1){
				inver_bp[inver_bp_len].bp1=bp1;//=atoi(sam->sam_field[1]);
				inver_bp[inver_bp_len].bp2=bp2;//=atoi(sam->sam_field[2]);
				inver_bp_len++;
				if(inver_bp_len == inver_bp_size){
					inver_bp_size*=2;
					inver_bp=(struct cnv_breakpoint*)xrealloc(inver_bp,inver_bp_size*sizeof(struct cnv_breakpoint));
				}
			}

		}
		sam_close(sam);
	}else{
		fprintf(stderr,"No inver bp list file specified.\n");
	}
	/*dup bp list*/
	sam = sam_open(dup_read_file);
	if(sam != NULL){
		while(read_sam(sam)){
			parse_sam_field(sam->buffer1, sam->sam_field, &sam_field_len);
			bp1=atoi(sam->sam_field[1]);
                        bp2=atoi(sam->sam_field[2]);
                        if(bp2>bp1){
				dup_bp[dup_bp_len].bp1=bp1;//atoi(sam->sam_field[1]);
				dup_bp[dup_bp_len].bp2=bp2;//atoi(sam->sam_field[2]);
				dup_bp_len++;
				if(dup_bp_len == dup_bp_size){
					dup_bp_size*=2;
					dup_bp=(struct cnv_breakpoint*)xrealloc(dup_bp,dup_bp_size*sizeof(struct cnv_breakpoint));
				}
			}
		}
		sam_close(sam);
	}else{
		fprintf(stderr,"No dup bp list file specified.\n");
	}

}
int
smapper(char *single_read_file,char *large_gap_read_file_fw,char *large_gap_read_file_bw,char *inver_read_file_fw,
		char *inver_read_file_bw, char *dup_read_file_fw, char *dup_read_file_bw, int g_file_num, char** g_file_name){
	uint64_t before;
	//fprintf(stderr,"genome file num:%i\n",g_file_num);
	//fprintf(stderr,"genome file name:%s\n",g_file_name[0]);

	/*open output file*/
	char out[1024];
	char *tok;
	//tok = strtok(out_file_name,".");
	tok=out_file_name;
	fprintf(stderr,"\nOutput:\n\n");
	fprintf(stderr,"\tprefix:%s\n",tok);

	sprintf(out,"%s_indel.sam",tok);
	file_indel=fopen(out,"w");
	fprintf(stderr,"\tout:%s\n",out);
	if(file_indel == NULL){
		fprintf(stderr,"ins out put file %s  open failed.\n",out);
        	exit(0);
	}
		
	sprintf(out,"%s_cluster_del.sam",tok);
        file_cluster_del=fopen(out,"w");
	fprintf(stderr,"\tout:%s\n",out);
	if(file_cluster_del== NULL){
		fprintf(stderr,"del out put file %s  open failed.\n",out);
		exit(0);
	}

	sprintf(out,"%s_dup.sam",tok);
        file_dup=fopen(out,"w");
	fprintf(stderr,"\tout:%s\n",out);
	if(file_dup== NULL){
			fprintf(stderr,"dup out put file %s  open failed.\n",out);
			exit(0);
	}
	sprintf(out,"%s_inv.sam",tok);
        file_inv=fopen(out,"w");
	fprintf(stderr,"\tout:%s\n",out);
	if(file_inv == NULL){
			fprintf(stderr,"inv out put file %s  open failed.\n",out);
			exit(0);
	}

	sprintf(out,"%s_unsplit.sam",tok);
        file_unsplit=fopen(out,"w");
	fprintf(stderr,"\tout:%s\n",out);
	if(file_unsplit == NULL){
			fprintf(stderr,"unsplit out put file %s  open failed.\n",out);
			exit(0);
	}

	sprintf(out,"%s_header.sam",tok);
        file_header=fopen(out,"w");
	fprintf(stderr,"\tout:%s\n",out);
	if(file_header == NULL){
			fprintf(stderr,"header out put file %s  open failed.\n",out);
			exit(0);
	}
	fprintf(stderr,"\n-------------------------------------------------------------\n\n");

	/*load the genome*/

    load_genome(g_file_name, g_file_num);

	/*Print sam header*/
	fprintf(file_header,"@HD\tVN:%i\tSO:%s\n",1,"unsorted");

	int i;
	for(i = 0; i < num_contigs; i++){
		fprintf(file_header,"@SQ\tSN:%s\tLN:%u\n",contig_names[i],genome_len[i]);
	}
	fprintf(file_header,"@PG\tID:%s\tVN:%s\t","smapper",PRISM_VERSION_STRING);
	fprintf(file_header,"CL:");
	for(i=0;i<cmd_num;i++){
		fprintf(file_header,"%s ",cmd_line[i]);
	}
	fprintf(file_header,"\n");

	/*load forward cnv bp list*/
	load_breakpoint_list(large_gap_read_file_fw, inver_read_file_fw, dup_read_file_bw);

	/*split mapping for + strand*/
	plus_strand_time=0;
	minus_strand_time=0;
	total_time=0;
	before = gettimeinusecs();

	map_split(single_read_file, 0);

	plus_strand_time= gettimeinusecs() - before;
	total_time+= plus_strand_time;
	fprintf(stderr,"map +strand done.\n");

	cur_1_len=0;
	cur_2_len=0;
	cur_3_len=0;
	cur_4_len=0;

	/*load backward cnv bp list*/

	load_breakpoint_list(large_gap_read_file_bw,inver_read_file_bw, dup_read_file_fw);

	/*split mapping for - strand*/
	before = gettimeinusecs();
	map_split(single_read_file, 1);
	minus_strand_time= gettimeinusecs() - before;
	total_time+= minus_strand_time;
	fprintf(stderr, "%i singleton reads processed.\n", single_read_num);
	fprintf(stderr, "+ Mapping Time:%.2f seconds\n", (double)plus_strand_time / 1.0e6);
	fprintf(stderr, "- Mapping Time:%.2f seconds\n", (double)minus_strand_time / 1.0e6);
	fprintf(stderr, "Total Mapping Time:%.2f seconds\n", (double)total_time / 1.0e6);

	fprintf(stdout, "%i singleton reads processed.\n", single_read_num);
        fprintf(stdout, "+ Mapping Time:%.2f seconds\n", (double)plus_strand_time / 1.0e6);
        fprintf(stdout, "- Mapping Time:%.2f seconds\n", (double)minus_strand_time / 1.0e6);
        fprintf(stdout, "Total Mapping Time:%.2f seconds\n", (double)total_time / 1.0e6);

	free(fasta);
	fclose(file_indel);
	fclose(file_cluster_del);
	fclose(file_dup);
	fclose(file_inv);
	fclose(file_unsplit);
	fclose(file_header);
	return 0;
}


int
rm_overlaped_sv(char *del_file_name){//del_location file generated by detect_gap
	sam_t sam1;
	char output_buffer[1024];
	int sam_field_len=0,strand_supp_num=0;
	sam1 = sam_open(del_file_name);
	FILE *file;
	int cur_gap_start, cur_gap_size, cur_cnt, bound;
	int score1,score2,score3;
	char out[1000];
	sprintf(out,"%s_RO",del_file_name);
	file=fopen(out,"w");
	read_sam(sam1);
	parse_sam_field(sam1->buffer1, sam1->sam_field, &sam_field_len);
	strand_supp_num=atoi(sam1->sam_field[10]);
	cur_gap_start = atoi(sam1->sam_field[1]);
	bound = atoi(sam1->sam_field[2]);
	cur_gap_size = atoi(sam1->sam_field[3]);
	cur_cnt= atoi(sam1->sam_field[5]);
	strcpy(output_buffer,sam1->buffer1);
	while(read_sam(sam1)){
		

		parse_sam_field(sam1->buffer1, sam1->sam_field, &sam_field_len);
		if(atoi(sam1->sam_field[1])>bound){//not overlap
			cur_gap_start = atoi(sam1->sam_field[1]);
			cur_gap_size = atoi(sam1->sam_field[3]);
			bound = atoi(sam1->sam_field[2]);
	
			cur_cnt=atoi(sam1->sam_field[5]);
			score1=atoi(sam1->sam_field[8]);
			score2=atoi(sam1->sam_field[9]);
			score3=atoi(sam1->sam_field[10]);
			strand_supp_num=atoi(sam1->sam_field[11]);

			fprintf(file,"%s",output_buffer);
			strcpy(output_buffer,sam1->buffer1);
		}else{//overlap

			//if(strand_supp_num > atoi(sam1->sam_field[10])){
				//continue;
			//}

			if(cur_cnt>=atoi(sam1->sam_field[5])){

                        }else{
				cur_gap_start = atoi(sam1->sam_field[1]);
                                cur_gap_size = atoi(sam1->sam_field[3]);
				strand_supp_num=atoi(sam1->sam_field[11]);
				bound = atoi(sam1->sam_field[2]);
				cur_cnt= atoi(sam1->sam_field[5]);
				strcpy(output_buffer,sam1->buffer1);
                        }
		}
	}
	fprintf(file,"%s",output_buffer);
	fclose(file);
	sam_close(sam1);
	return 0;
}
/**
 * input file should be sorted by gap location and gap size
 */
int
detect_sv(int left_len_thresh, int right_len_thresh, int aln_score_thresh, int read_num_thresh, char *del_file_name){
	sam_t sam1,sam2;
	int sam_field_len=0,cnt=1;
	char **gap_seq;
	char voted_seq[1024];
	char *chr,*sv_type;
	int sv_start,sv_end,sv_size;
	int i,j;
	int na=0,nc=0,ng=0,nt=0;
	int gap_seq_size=100,gap_seq_len=0,gap_size;
	int strand_support_num=0,strand=0;
	bool plus_supported=false, minus_supported=false;
	gap_seq=(char**)xcalloc(gap_seq_size*sizeof(char*));

	for(i=0;i<gap_seq_size;i++){
		gap_seq[i]=(char*)xcalloc(1024*sizeof(char));
	}

	sam1 = sam_open(del_file_name);
	sam2 = sam_open(del_file_name);
	if(sam1==NULL || sam2==NULL){
		fprintf(stderr,"sv location file open failed.\n");
		exit(0);
	}
	int left_len=0, right_len=0, score=1,max_score1=0, max_score2=0, max_score3=0;

	FILE *file;
	char out[1000];
	sprintf(out,"%s_sv",del_file_name);
	file=fopen(out,"w");
	while(read_sam(sam1)){
		if(sam1->buffer1[0]!='@'){
			break;
		}
	}
	parse_sam_field(sam1->buffer1, sam1->sam_field, &sam_field_len);
	strcpy(gap_seq[gap_seq_len++],sam1->sam_field[6]);
	sv_start=atoi(sam1->sam_field[1]);
	sv_size=atoi(sam1->sam_field[2]);
	sv_type=sam1->sam_field[5];
	chr=sam1->sam_field[9];
	char *p;
	p=strchr(chr,'\n');
	if(p){
		*p='\0';	
	}
	if (strcmp(sv_type,"INS")){
		sv_end=sv_start+sv_size-1;
	}else{
		sv_end=sv_start;
	}
	//fprintf(indel_location_file,"@read start\tgap start\tgap size\tleft_contig_length\tright_contig_length\tgap_type\tgap seq\tscore\tstrand\tchr\n");
	fprintf(file,"@CHR\tSTART\tEND\tSIZE\tTYPE\tSUPP_READ_NUM\tLEFT_LEN\tRIGHT_LEN\tMAX_SOCRE\t2ST_SCORE\t3ST_SCORE\tSUPP_STRAND_NUM\n");
	while(read_sam(sam2)){
		parse_sam_field(sam2->buffer1, sam2->sam_field, &sam_field_len);
		
		if(strcmp(sam1->sam_field[1],sam2->sam_field[1])==0 && strcmp(sam1->sam_field[2],sam2->sam_field[2])==0
				&& strcmp(sam1->sam_field[5],sam2->sam_field[5])==0){
			strand=atoi(sam2->sam_field[8]);
			
			score=atoi(sam2->sam_field[7]);
	
			if(score>aln_score_thresh && left_len>left_len_thresh && right_len >right_len_thresh){
				if(strand==0){
        	                        plus_supported=true;
	                        }else{
                	                minus_supported=true;
                       		}

			}else{
				score=0;
			}
			if(max_score1<score){
				max_score3=max_score2;
				max_score2=max_score1;
				max_score1=score;
				
			}else if(max_score2<score){
				max_score3=max_score2;
				max_score2=score;
			}else if(max_score3<score){
				max_score3=score;
			}
			if(left_len<atoi(sam2->sam_field[3])){
				left_len=atoi(sam2->sam_field[3]);
			}
			if(right_len<atoi(sam2->sam_field[4])){
				right_len=atoi(sam2->sam_field[4]);
			}
			if(gap_seq_len==gap_seq_size){
				gap_seq=(char**)xrealloc(gap_seq,2*gap_seq_len*sizeof(char*));
				for(i=gap_seq_size;i<2*gap_seq_len;i++){
					gap_seq[i]=(char*)xcalloc(1024*sizeof(char));
				}
				gap_seq_size=2*gap_seq_len;
			}
			

			strcpy(gap_seq[gap_seq_len++],sam2->sam_field[6]);

			cnt++;

		}else{


			if(cnt>read_num_thresh && left_len>left_len_thresh && right_len>right_len_thresh && max_score1>aln_score_thresh){
				if(0 && strcmp(sam1->sam_field[5],"INS")==0){
					na=0,nc=0,ng=0,nt=0;
					gap_size=atoi(sam1->sam_field[2]);

					for(j=0;j<gap_size;j++){
						na=0,nc=0,ng=0,nt=0;
						for(i=0;i<gap_seq_len;i++){
							switch(gap_seq[i][j]){
							case 'A':
							case 'a':
								na++;
								break;
							case 'C':
							case 'c':
								nc++;
								break;
							case 'G':
							case 'g':
								ng++;
								break;
							case 'T':
							case 't':
								nt++;
								break;

							default:
								fprintf(stderr, "%s\tinvalid base:%s\n",sam2->sam_field[0],gap_seq[i]);
								fprintf(stderr, "i,j %i\t:%i\n",i,j);
								fprintf(stderr, "gap_seq_len:%i\n",gap_seq_len);
								fprintf(stderr, "%s,%s,%s,%s\n",sam2->sam_field[1],sam2->sam_field[2],sam2->sam_field[3],sam2->sam_field[6]);
								abort();
								break;
							}
						}
						int max;
						max=na;
						voted_seq[j]='a';
						if(nc>max){
							max=nc;
							voted_seq[j]='c';
						}
						if(ng>max){
							max=ng;
							voted_seq[j]='g';
						}
						if(nt>max){
							max=nt;
							voted_seq[j]='t';
						}

					}
					voted_seq[j]='\0';
				}else{
					strcpy(voted_seq,"*");
				}
				if(plus_supported ){
					strand_support_num++;
				}
				if(minus_supported){
					strand_support_num++;
				}
				//read num, gap start, gap size, left length, right length seq
				//fprintf(file,"%i\t%i\t%s\t%i\t%i\t%s\t%s\t%i\t%i\t%i\t%i\n",cnt,atoi(sam1->sam_field[1]),sam1->sam_field[2],
				//		left_len, right_len,sam1->sam_field[5],voted_seq,max_score1,max_score2,max_score3,strand_support_num);
				fprintf(file,"%s\t%i\t%i\t%i\t%s\t%i\t%i\t%i\t%i\t%i\t%i\t%i\n",chr,sv_start,sv_end,sv_size,sv_type,cnt,
						left_len, right_len,max_score1,max_score2,max_score3,strand_support_num);
			}
			
			
			gap_seq_len=0;
			strcpy(sam1->buffer1,sam2->buffer1);
			parse_sam_field(sam1->buffer1, sam1->sam_field, &sam_field_len);
			score=atoi(sam2->sam_field[7]);
			strand=atoi(sam1->sam_field[8]);
			sv_start=atoi(sam1->sam_field[1]);
		        sv_size=atoi(sam1->sam_field[2]);
		        sv_type=sam1->sam_field[5];
		        chr=sam1->sam_field[9];
			p=strchr(chr,'\n');
		        if(p){
		                *p='\0';
        		}
			if (strcmp(sv_type,"INS")){
		                sv_end=sv_start+sv_size-1;
		        }else{
		                sv_end=sv_start;
		        }

			plus_supported=false;
			minus_supported=false;
			strand_support_num=0;

                        strcpy(gap_seq[gap_seq_len++],sam1->sam_field[6]);
			left_len=atoi(sam1->sam_field[3]);
			right_len=atoi(sam1->sam_field[4]);
			cnt=1;
			if(score>aln_score_thresh && left_len>left_len_thresh && right_len>right_len_thresh){
				max_score1=score;
				if(strand==0){
					plus_supported=true;
				}else{
					minus_supported=true;
				}

			}else{
				max_score1=0;
			}
			
			max_score2=0;
			max_score3=0;
		}


	}

	if(cnt>read_num_thresh && left_len>left_len_thresh && right_len>right_len_thresh && max_score1>aln_score_thresh){
		if(strcmp(sam1->sam_field[5],"INS")==0){
			na=0,nc=0,ng=0,nt=0;
			gap_size=atoi(sam1->sam_field[2]);

			for(j=0;j<gap_size;j++){
				na=0,nc=0,ng=0,nt=0;
				for(i=0;i<gap_seq_len;i++){
					switch(gap_seq[i][j]){
					case 'A':
					case 'a':
						na++;
						break;
					case 'C':
					case 'c':
						nc++;
						break;
					case 'G':
					case 'g':
						ng++;
						break;
					case 'T':
					case 't':
						nt++;
						break;

					default:
						fprintf(stderr, "%s\tinvalid base:%s\n",sam1->sam_field[0],gap_seq[i]);
						exit(1);
						break;
					}
				}
				int max;
				max=na;
				voted_seq[j]='a';
				if(nc>max){
					max=nc;
					voted_seq[j]='c';
				}
				if(ng>max){
					max=ng;
					voted_seq[j]='g';
				}
				if(nt>max){
					max=nt;
					voted_seq[j]='t';
				}

			}
			voted_seq[j]='\0';
		}else{
			strcpy(voted_seq,"*");
		}

		//fprintf(file,"%i\t%i\t%s\t%i\t%i\t%s\t%s\t%i\t%i\t%i\t%i\n",cnt,atoi(sam1->sam_field[1]),sam1->sam_field[2],
		//		left_len, right_len,sam1->sam_field[5],voted_seq,max_score1,max_score2,max_score3,strand_support_num);
		fprintf(file,"%s\t%i\t%i\t%i\t%s\t%i\t%i\t%i\t%i\t%i\t%i\t%i\n",chr,sv_start,sv_end,sv_size,sv_type,cnt,
				left_len, right_len,max_score1,max_score2,max_score3,strand_support_num);
	}
	fclose(file);
	sam_close(sam1);
	sam_close(sam2);
	return 0;
}
int
format_sv(char *split_file_name){
	FILE *file, *indel_location_file;
	file=fopen(split_file_name,"r");
	char out[200];
	sprintf(out,"%s_cigar",split_file_name);
	indel_location_file=fopen(out,"w");
	if(file==NULL){
		fprintf(stderr,"split file open failed.%s\n",split_file_name);
		exit(1);
	}
	if(indel_location_file==NULL){
		fprintf(stderr,"gap location file open failed.\n");
		exit(1);
	}
	int sam_field_len=0;
	int read_start=0, read_len=0;
	int score;
	sam_t sam;
	char *p,*q, *gap_seq, *seq,*chr;
	char *tok;
	int type=0;
	char *st_pos, *as_pos;
	int left_len,right_len;
	int tmp_gap_size,tmp_gap_start;	
	int flag=0,strand=0;
	gap_seq=(char*)xcalloc(1024*sizeof(char));
	seq=(char*)xcalloc(1024*sizeof(char));
	sam = sam_open(split_file_name);
	fprintf(indel_location_file,"@read start\tgap start\tgap size\tleft_contig_length\tright_contig_length\tgap_type\tgap seq\tscore\tstrand\tchr\n");
	int gap_start=0,gap_size=0;
	int del_size;
	while(fgets(sam->buffer1,1024,file)){
		if(sam->buffer1[0]=='@'){
			continue;
		}
		parse_sam_field(sam->buffer1, sam->sam_field, &sam_field_len);
		chr=sam->sam_field[2];
		del_size=0;
		flag=atoi(sam->sam_field[1]);
		strand=flag & 0x10;
		read_start=atoi(sam->sam_field[3]);
		gap_start=atoi(sam->sam_field[3]);
		read_len=strlen(sam->sam_field[9]);
		//strcpy(gap_seq,sam->sam_field[9]);
		strcpy(seq,sam->sam_field[9]);
		tok=sam->sam_field[11];
		//score=atoi(tok+5);//AS:I:SCORE
		st_pos=strstr(sam->buffer1,"ST:i:");
		if(st_pos){
			type=atoi(st_pos+5);
		}else{
			fprintf(stderr, "ST field is absent. line:%s\n",sam->buffer1);
			continue;
			abort();
		}
		as_pos=strstr(sam->buffer1,"AS:i:");
		if(as_pos){
                        score=atoi(as_pos+5);
                }else{
                        fprintf(stderr, "AS field is absent. line:%s\n",sam->buffer1);
                        continue;
                        abort();
                }
	
		p=sam->sam_field[5];//p point to CIGAR field
		q=p;
		while(*p!='\0'){
			while(*p>='0' && *p<= '9'){
				//printf("p:%c\n",*p);
				p++;
			}
			//printf("p:%c\n",*p);
			if(*p=='M'){
				*p='\0';
				gap_start+=atoi(q);
			}else if(*p=='D'){
				*p='\0';
				gap_size=atoi(q);
				left_len= gap_start-read_start-del_size;
				right_len=read_len - (gap_start-read_start-del_size);
				fprintf(indel_location_file,"%s\t%i\t%i\t%i\t%i\t%s\t%s\t%i\t%i\t%s\n",sam->sam_field[3],gap_start, gap_size,left_len ,right_len,"DEL","*",score,strand,chr);
				gap_start+=gap_size;
				del_size+=gap_size;

			}else if( *p=='N'){
				*p='\0';
				gap_size=atoi(q);
				left_len= gap_start-read_start-del_size;
				right_len=read_len - (gap_start-read_start-del_size);
				switch(type){
				case SMALL_INDEL:
				case LARGE_DEL:
					fprintf(indel_location_file,"%s\t%i\t%i\t%i\t%i\t%s\t%s\t%i\t%i\t%s\n",sam->sam_field[3],gap_start, gap_size,left_len ,right_len,"DEL","*",score,strand,chr);
					//fprintf(indel_location_file,"%s\t%i\t%i\t%i\t%i\t%s\t%s\t%i\t%i\n",sam->sam_field[3],gap_start, gap_size,left_len ,right_len,"DEL","*",score,strand);
					gap_start+=gap_size;
					del_size+=gap_size;
					break;
				case INVER_FW_OUT:
				case INVER_FW_IN:
				case SMALL_INVER_BW:
					tmp_gap_size=gap_size+read_len-(gap_start-read_start);
					//fprintf(indel_location_file,"%s\t%i\t%i\t%i\t%i\t%s\t%s\t%i\t%i\n",sam->sam_field[3],gap_start, tmp_gap_size,left_len ,right_len,"INV","*",score,strand);
					fprintf(indel_location_file,"%s\t%i\t%i\t%i\t%i\t%s\t%s\t%i\t%i\t%s\n",sam->sam_field[3],gap_start, tmp_gap_size,left_len ,right_len,"INV","*",score,strand,chr);
					gap_start+=gap_size;
					del_size+=gap_size;
					break;
				case INVER_BW_IN:
				case INVER_BW_OUT:
				case SMALL_INVER_FW:
					tmp_gap_start=read_start;
					tmp_gap_size=gap_size+gap_start-read_start;
					//fprintf(indel_location_file,"%s\t%i\t%i\t%i\t%i\t%s\t%s\t%i\t%i\n",sam->sam_field[3],tmp_gap_start, tmp_gap_size,left_len ,right_len,"INV","*",score,strand);
					fprintf(indel_location_file,"%s\t%i\t%i\t%i\t%i\t%s\t%s\t%i\t%i\t%s\n",sam->sam_field[3],tmp_gap_start, tmp_gap_size,left_len ,right_len,"INV","*",score,strand,chr);
					gap_start+=gap_size;
					del_size+=gap_size;
					break;
				case DUP:
					tmp_gap_start=read_start;
					tmp_gap_size=gap_size+read_len;
					//fprintf(indel_location_file,"%s\t%i\t%i\t%i\t%i\t%s\t%s\t%i\t%i\n",sam->sam_field[3],tmp_gap_start, tmp_gap_size,left_len ,right_len,"DUP","*",score,strand);
					fprintf(indel_location_file,"%s\t%i\t%i\t%i\t%i\t%s\t%s\t%i\t%i\t%s\n",sam->sam_field[3],tmp_gap_start, tmp_gap_size,left_len ,right_len,"DUP","*",score,strand,chr);
					gap_start+=gap_size;
					del_size+=gap_size;
					break;
				default:
					fprintf(stderr, "invalid type:%i in read %s\n",type,sam->sam_field[0]);
					abort();	
					
				}

			}else if(*p=='I'){
				*p='\0';
				gap_size=atoi(q);
				memcpy(gap_seq,seq+(gap_start-read_start-del_size),gap_size);
				gap_seq[gap_size]='\0';
				//fprintf(indel_location_file,"%s\t%i\t%i\t%i\t%i\t%s\t%s\t%i\t%i\n",sam->sam_field[3],gap_start, gap_size, gap_start-read_start-del_size,read_len - (gap_start-read_start-del_size)-gap_size,"INS", gap_seq,score,strand);
				fprintf(indel_location_file,"%s\t%i\t%i\t%i\t%i\t%s\t%s\t%i\t%i\t%s\n",sam->sam_field[3],gap_start, gap_size, gap_start-read_start-del_size,read_len - (gap_start-read_start-del_size)-gap_size,"INS", gap_seq,score,strand,chr);

			}else{
				fprintf(stderr,"unknown CIGAR.%s\n",sam->buffer1);
				break;
				//exit(1);
			}

			//printf("p:%c\n",*p);
			//printf("q:%s\n",q);

			p++;
			q=p;
		}
	}

	sam_close(sam);
	free(seq);
	free(gap_seq);
	fclose(file);
	fclose(indel_location_file);
	return 0;
}

static int usage(int cmd_id)
{
	fprintf(stderr, "\n");
        fprintf(stderr, "Program: PRISM (Pair Read informed Split Mapper) \n");
        fprintf(stderr, "Version: %s%s\n", "PRISM ",PRISM_VERSION_STRING);
        fprintf(stderr, "Contact: \n\n");

	switch(cmd_id){
	case 1:
		fprintf(stderr, "mapsplit letter space split alignment\n");
        	fprintf(stderr, "Usage:  smapper mapsplit [options] <reference.fa>\n\n");

	        fprintf(stderr, "\t-T split penalty for non-cluster-supported deletion(default:-74)\n");
	        fprintf(stderr, "\t-R split penalty for non-cluster-supported insertion(default:-49)\n");
	        fprintf(stderr, "\t-Q split penalty for cluster-supported deletion,duplication and inversion(default:0)\n");
	        fprintf(stderr, "\t-m mapping distance mean\n");
       		fprintf(stderr, "\t-e mapping distance std\n\n");

        	fprintf(stderr, "\t-S singleton read file (A read pair that only one read aligned.)\n");
	        fprintf(stderr, "\t-o output file (A group of files with the sam prefix will be generated.)\n");
        	fprintf(stderr, "\t-f min testing distance for non-cluster-supported read(default:0)\n");
       		fprintf(stderr, "\t-g testing region length for cluster-supported read\n");
       		fprintf(stderr, "\t-x max read length(default:100)\n\n");
		
        	fprintf(stderr, "\t-L forward Deletion cluster file name\n");
	        fprintf(stderr, "\t-l backward Deletion cluster file name\n");
        	fprintf(stderr, "\t-I forward Inversion cluster file name\n");
	        fprintf(stderr, "\t-i backward Inversion cluster file name\n");
        	fprintf(stderr, "\t-D forward Duplication cluster file name\n");
	        fprintf(stderr, "\t-d backward Dduplication cluster file name\n\n");

        	fprintf(stderr, "\t-f min testing distance (default:0)\n");
        	fprintf(stderr, "\t-c only align reads around cluster points\n\n");
        	fprintf(stderr, "\t-r minimal read lengths, used to descide kmer length (default:100)\n\n");

		break;
	case 2:
		
		fprintf(stderr, "bwasam2tracks generate singleton reads file and discordant reads file from bwa output sam file\n");
                fprintf(stderr, "Usage:  smapper bwasam2tracks <bwa sam file> <mean of isize> <std of isize> <output folder>\n\n");
		break;
	case 3:
		fprintf(stderr, "detectsv detect sv from unique sorted sv location file\n");
                fprintf(stderr, "Usage:  smapper detectsv [options] <sv_format_sorted file>\n\n");
		fprintf(stderr, "\t-n min supported reads num thresh for a sv call\n");
	        fprintf(stderr, "\t-l left half length thresh for a sv call\n");
       		fprintf(stderr, "\t-r right half length thresh for a sv call\n");
	        fprintf(stderr, "\t-s thresh of best score for a sv call\n\n");
		break;
	case 4:
		fprintf(stderr, "formatsv format CIGAR of split mapping result\n");
                fprintf(stderr, "Usage:  smapper formatsv <split_mapping_sam_file>\n\n");
		break;
	case 5:
		fprintf(stderr, "rmoverlap remove overlaped sv, keep the one supported by most reads\n");
                fprintf(stderr, "Usage:  smapper rmoverlap <sv_loci_file generated by \"detectsv\">\n\n");
		break;
	case 100:
		fprintf(stderr, "Command:\n");
		fprintf(stderr, "mapsplit\tletter space split alignment\n");
		fprintf(stderr, "bwasam2tracks\tgenerate singleton reads file and discordant reads file from bwa output sam file\n");
		fprintf(stderr, "detectsv\tdetect sv from unique sorted sv location file\n");
		fprintf(stderr, "formatsv\tformat CIGAR of split mapping result\n");
		fprintf(stderr, "rmoverlap\tremove overlaped sv, keep the one supported by most reads\n");
		break;
	default:
		fprintf(stderr, "unknown command id:%i\n",cmd_id);
		abort();
	}
	exit(1);
}
int
smapper_main(int argc, char *argv[]){
	char c;
	char space_string[100];
	char *single_file_name=NULL,*inver_file_name_fw=NULL, *inver_file_name_bw=NULL, *dup_file_name_fw=NULL, *dup_file_name_bw=NULL,
			*large_gap_file_name_fw=NULL, *large_gap_file_name_bw=NULL;
	mean=-1;
	devi=-1;
	map_split_setup();

	while ((c = getopt(argc, argv, "Cco:e:i:d:l:k:m:t:NM:O:E:q:f:g:T:S:D:I:L:B:R:Q:x:k:r:")) >= 0) {
		switch (c) {
		case 'n':

			break;
		case 'T': split_penalty_del_double = atoi(optarg);break;
		case 'Q': split_penalty_del_large = atoi(optarg);break;
		case 'R': split_penalty_ins = atoi(optarg);break;
		case 'C': space_flag = MODE_COLOUR_SPACE;break;
		case 'o': strcpy(out_file_name,optarg); break;
		case 'm': mean = atoi(optarg); break;
		case 'e': devi = atoi(optarg);delta=3*devi;max_dis=mean+delta; break;
		case 'f': min_dis = atoi(optarg); break;
		case 'x': max_rlen = atoi(optarg);break;
		case 'g': test_region_len = atoi(optarg);break;
		case 'S': single_file_name=optarg;break;
		case 'L': large_gap_file_name_fw=optarg; break;
		case 'l': large_gap_file_name_bw=optarg; break;
		case 'D': dup_file_name_fw=optarg; break;
		case 'd': dup_file_name_bw=optarg; break;
		case 'I': inver_file_name_fw=optarg; break;
		case 'i': inver_file_name_bw=optarg; break;
		case 'c': align_small_indel=false; break;
		//case 'k': kmer_len = atoi(optarg); break;
		case 'r': min_rlen = atoi(optarg); break;
		default:
			fprintf(stderr,"unknown option:-%c\n",c);
			usage(1);
		}

	}
	if(single_file_name==NULL){
		fprintf(stderr,"singleton read file (sorted) is not specified.\n");
		usage(1);
	}
	switch(space_flag){
        case MODE_LETTER_SPACE:
		strcpy(space_string,"LETTER SPACE");
                break;
        case MODE_COLOUR_SPACE:
		strcpy(space_string,"COLOR SPACE");
                break;
        default:
                fprintf(stderr,"unsupported space flag:%i\n",space_flag);
                usage(1);
        }

	fprintf(stderr,"-------------------------------------------------------------\n");
	fprintf(stderr, "smapper: %s.\nPRISM %s\n[%s]\n", space_string,
			PRISM_VERSION_STRING, get_compiler());
	fprintf(stderr,"-------------------------------------------------------------\n");
	fprintf(stderr,"\nClusters:\n\n");
	if(dup_file_name_fw==NULL){
		//dup_file_name_fw="";
		fprintf(stderr,"\tduplication cluster file (leftbound sorted) is not specified.\n");
	}else{
		fprintf(stderr,"\tDuplication cluster for + strand.\n");
	}
	
	if(dup_file_name_bw==NULL){
		//dup_file_name_bw="";
		fprintf(stderr,"\tduplication cluster file (rightbound sorted) is not specified.\n");

	}else{
		fprintf(stderr,"\tDuplication cluster for - strand.\n");
	}
	if(inver_file_name_fw==NULL){
		//inver_file_name_fw="";
		fprintf(stderr,"\tinversion cluster file (leftbound sorted) is not specified.\n");

	}else{
		fprintf(stderr,"\tInversion cluster for + strand.\n");
	}
	if(inver_file_name_bw==NULL){
		//inver_file_name_bw="";
		fprintf(stderr,"\tinversion cluster file (rightbound sorted) is not specified.\n");

	}else{
		fprintf(stderr,"\tInversion cluster for - strand.\n");
	}
	if(large_gap_file_name_fw==NULL){
		//large_gap_file_name_fw="";
		fprintf(stderr,"\tdeletion cluster file (leftbound sorted) is not specified.\n");

	}else{
		fprintf(stderr,"\tDeletion cluster for + strand.\n");
	}
	if(large_gap_file_name_bw==NULL){
		//large_gap_file_name_bw="";
		fprintf(stderr,"deletion cluster file (rightbound sorted) is not specified.\n");

	}else{
		fprintf(stderr,"\tDeletion cluster for - strand.\n");
	}
	if(min_rlen<1){
		fprintf(stderr,"invalid minimal read length.\n");
		usage(1);
	}
	if(mean<1){
		fprintf(stderr,"mean value of mapping distance is invalid or not specified.\n");
		usage(1);
	}
	if(kmer_len<1){
		fprintf(stderr,"kmer length must be longer than 1bp (see -k option).\n");
		usage(1);
	}
	if(devi<0){
		fprintf(stderr,"deviation value of mapping distance is invalid or not specified.\n");
		usage(1);
	}
	if(argc-optind<0){
		//fprintf(stderr,"usage:\n");
		usage(1);
	}

	if(space_flag==MODE_LETTER_SPACE){

		if(sw_split_ls_setup(max_xy_contig_size, longest_read_len, a_gap_open_score, a_gap_extend_score,
				b_gap_open_score, b_gap_extend_score, match_score, mismatch_score,split_penalty_ins,
				split_penalty_del_large,split_penalty_del_double,kmer_len,min_rlen)){
			fprintf(stderr, "ls-split-mapper initialization failed\n");
			abort();
		}
		if(anchors_setup(max_xy_contig_size, longest_read_len)){
			fprintf(stderr, "anchors initialization failed\n");
			abort();
		}
	}else if(space_flag==MODE_COLOUR_SPACE){
		if(sw_split_cs_setup(max_xy_contig_size, longest_read_len, a_gap_open_score, a_gap_extend_score,
	                  match_score, mismatch_score, crossover_score,split_penalty_ins,
                                split_penalty_del_large,split_penalty_del_double)){
                        fprintf(stderr, "cs-split-mapper initialization failed\n");
                        abort();
                }

	}
	fasta=fasta_open2(space_flag);	
	fprintf(stderr,"\nParameters:\n\n");
	fprintf(stderr,"\tmean:%i\n",mean);
	fprintf(stderr,"\tstdev:%i\n",devi);
	fprintf(stderr,"\tmin mapping distance:%i\n",min_dis);
	fprintf(stderr,"\ttest region length:%i\n",test_region_len);

	fprintf(stderr,"\tsplit penalty for insertion:%i\n",split_penalty_ins);
	fprintf(stderr,"\tsplit penalty for small del:%i\n",split_penalty_del_double);
	fprintf(stderr,"\tsplit penalty for large del:%i\n",split_penalty_del_large);
	smapper(single_file_name, large_gap_file_name_fw, large_gap_file_name_bw,inver_file_name_fw, inver_file_name_bw,
			dup_file_name_fw, dup_file_name_bw, 1, &argv[optind]);//read file1,read file2, out file,  genome file num, genome file[]

	return 0;
}
int
detect_sv_main(int argc, char *argv[]){
	char c;
	int left_len_thresh=15,right_len_thresh=15, score_thresh=0, read_num_thresh=5;
	/*initialize*/
	if(sw_split_ls_setup(max_xy_contig_size, longest_read_len, a_gap_open_score, a_gap_extend_score,
			b_gap_open_score, b_gap_extend_score, match_score, mismatch_score,split_penalty_ins,
			split_penalty_del_large,split_penalty_del_double,0,min_rlen)){
		fprintf(stderr, "initialization failed");
		abort();
	}
	map_split_setup();
	while ((c = getopt(argc, argv, "l:r:s:n:")) >= 0) {
		switch (c) {

		case 'l': left_len_thresh = atoi(optarg);break;
		case 'r': right_len_thresh = atoi(optarg); break;
		case 's': score_thresh = atoi(optarg); break;
		case 'n': read_num_thresh = atoi(optarg); break;

		default: return 1;
		}

	}
	if(argc-optind+1 <1 ){
		usage(3);
	}

	detect_sv(left_len_thresh,right_len_thresh,score_thresh,read_num_thresh,argv[optind]);//read file1,read file2, out file,  genome file num, genome file[]
	return 0;
}
int
get_discordant_from_bwa_sam(char *bwa_sam_file_name,int mean, int devi, char *output_folder){//, int inver_isize, int dup_isize, char *cnv_cluster, int rlen){
	int large_isize=mean+3*devi;

	int inver_isize=0; int dup_isize=0;//these definations are for future use

	FILE *large_file, *inver_file, *dup_file,*single_file;
	char out[1024];
	sprintf(out,"%s/%s_large",output_folder,bwa_sam_file_name);
	large_file=fopen(out,"w");
	sprintf(out,"%s/%s_inver",output_folder,bwa_sam_file_name);
	inver_file=fopen(out,"w");
	sprintf(out,"%s/%s_dup",output_folder,bwa_sam_file_name);
	dup_file=fopen(out,"w");
	sprintf(out,"%s/%s_single",output_folder,bwa_sam_file_name);
	single_file=fopen(out,"w");

	if(large_file==NULL || inver_file==NULL || dup_file==NULL|| single_file==NULL){
		fprintf(stderr,"out put files open failed.\n");
		exit(1);
	}
	int sam_field_len=0;
	sam_t sam;
	int flag1,isize1,flag2,isize2;
	uint32_t pos1, pos2;
	char cigar1[100],cigar2[100],seq1[1024],seq2[1024],tmp_seq[1024],name1[200],name2[200];
	sam = sam_open(bwa_sam_file_name);
	if (sam == NULL){
		fprintf(stderr,"%s open failed.\n",bwa_sam_file_name);
		abort();
	}
	while(gzgets(sam->fp,sam->buffer1,1024)){
		if(sam->buffer1[0]=='@'){
			continue;
		}
		if(strlen(sam->buffer1)==0){
			continue;
		}
read_1_line:
		gzgets(sam->fp,sam->buffer2,1024);
		if(sam->buffer2[0]=='@'){
				abort();
		}
		if(strlen(sam->buffer2)==1){
				abort();
		}

		parse_sam_field(sam->buffer1, sam->sam_field, &sam_field_len);
		strcpy(name1,sam->sam_field[0]);
		flag1=atoi(sam->sam_field[1]);
		isize1=atoi(sam->sam_field[8]);
		pos1=atoi(sam->sam_field[3]);
		strcpy(cigar1,sam->sam_field[5]);
		strcpy(seq1,sam->sam_field[9]);
	
		parse_sam_field(sam->buffer2, sam->sam_field, &sam_field_len);
		strcpy(name2,sam->sam_field[0]);
		flag2=atoi(sam->sam_field[1]);
		isize2=atoi(sam->sam_field[8]);
		pos2=atoi(sam->sam_field[3]);
		strcpy(cigar2,sam->sam_field[5]);
		strcpy(seq2,sam->sam_field[9]);
		if(strcmp(name1,name2)){

			strcpy(sam->buffer1,sam->buffer2);
			goto read_1_line;
			//abort();
		}
		//code for detect and output large inv dup single
		if((flag1 & 0x04)==0 && (flag2 & 0x04)==0 ){//both mapped
			/*large*/
			if((isize1<0 && (flag1 & 0x10)==0x10 && (flag1 & 0x20)==0)||
			   (isize1>0 && (flag1 & 0x10)==0 && (flag1 & 0x20)==0x20)){//opo-in for large

				if(abs(isize1)>large_isize){//large
					fprintf(large_file,"%s",sam->buffer1);

					fprintf(large_file,"%s",sam->buffer2);
					if((strchr(cigar1,'S')||strchr(cigar1,'I')||strchr(cigar1,'D')||strchr(cigar1,'N'))){//mate1 is difficultly mapped
						//generate a visual single read from mate1
						sam->buffer2[strlen(sam->buffer2)-1]='\0';
						fprintf(single_file,"%s\t",sam->buffer2);
						if((flag1 & 0x10) == 0x10){//mate1 is mapped to -strand
								reverse_complement_read_ls_text(seq1,tmp_seq);
								strcpy(seq1,tmp_seq);
								pos1-=mean;
						}else{//+strand
								pos1+=mean;
						}
						fprintf(single_file,"VC:i:%i\t", pos1);
						fprintf(single_file,"R2:Z:%s\n",seq1);

					}
					if((strchr(cigar2,'S' && strlen(seq1)>50)||strchr(cigar2,'I')||strchr(cigar2,'D')||strchr(cigar2,'N')) && (flag1 & 0x04)==0){//mate2 diffcult
							sam->buffer1[strlen(sam->buffer1)-1]='\0';
							fprintf(single_file,"%s\t",sam->buffer1);
							if((flag2 & 0x10) == 0x10){//mate2 -
									reverse_complement_read_ls_text(seq2,tmp_seq);
									strcpy(seq2,tmp_seq);
									pos2-=mean;
							}else{//mate1 -
									pos2+=mean;
							}
							fprintf(single_file,"VC:i:%i\t", pos1);
							fprintf(single_file,"R2:Z:%s\n",seq2);
					}


				}else{/*normal*/
					if(((strchr(cigar1,'S') && strlen(seq1)>50)||strchr(cigar1,'I')||strchr(cigar1,'D')||strchr(cigar1,'N')) && (flag2 & 0x04)==0){//cigar1 includes S, and its mate is mapped

							sam->buffer2[strlen(sam->buffer2)-1]='\0';
							fprintf(single_file,"%s",sam->buffer2);
							if((flag1 & 0x10) == 0x10){
									reverse_complement_read_ls_text(seq1,tmp_seq);
									strcpy(seq1,tmp_seq);
							}
							fprintf(single_file,"\tR2:Z:%s\n",seq1);
					}
					if(((strchr(cigar2,'S') && strlen(seq1)>50)||strchr(cigar2,'I')||strchr(cigar2,'D')||strchr(cigar2,'N')) && (flag1 & 0x04)==0){
							sam->buffer1[strlen(sam->buffer1)-1]='\0';
							fprintf(single_file,"%s",sam->buffer1);
							if((flag2 & 0x10) == 0x10){
									reverse_complement_read_ls_text(seq2,tmp_seq);
									strcpy(seq2,tmp_seq);
							}
							fprintf(single_file,"\tR2:Z:%s\n",seq2);

					}
				}
			}
			/*dup*/
			if((isize1<0 && (flag1 & 0x10)==0 && (flag1 & 0x20)==0x20) ||
			   (isize1>0 && (flag1 & 0x10)==0x10 && (flag1 & 0x20)==0)){//opp-out for dup
				if(abs(isize1)>dup_isize){

					fprintf(dup_file,"%s",sam->buffer1);
					fprintf(dup_file,"%s",sam->buffer2);
					if(((strchr(cigar1,'S') && strlen(seq1)>50)||strchr(cigar1,'I')||strchr(cigar1,'D')||strchr(cigar1,'N')) && (flag2 & 0x04)==0){//cigar1 includes S, and its mate is mapped
						sam->buffer2[strlen(sam->buffer2)-1]='\0';
						fprintf(single_file,"%s",sam->buffer2);
						if((flag1 & 0x10) == 0x10){
										reverse_complement_read_ls_text(seq1,tmp_seq);
										strcpy(seq1,tmp_seq);
						}
						fprintf(single_file,"\tR2:Z:%s\n",seq1);
					}
					if(((strchr(cigar2,'S') && strlen(seq1)>50)||strchr(cigar2,'I')||strchr(cigar2,'D')||strchr(cigar2,'N')) && (flag1 & 0x04)==0){
						sam->buffer1[strlen(sam->buffer1)-1]='\0';
						fprintf(single_file,"%s",sam->buffer1);
						if((flag2 & 0x10) == 0x10){
										reverse_complement_read_ls_text(seq2,tmp_seq);
										strcpy(seq2,tmp_seq);
						}
						fprintf(single_file,"\tR2:Z:%s\n",seq2);
					}

				/*VISUAL COORDINATE IS TO BEGENERATED FOR DUP & INVER*/
				}else{
					if(((strchr(cigar1,'S') && strlen(seq1)>50)||strchr(cigar1,'I')||strchr(cigar1,'D')||strchr(cigar1,'N')) && (flag2 & 0x04)==0){//cigar1 includes S, and its mate is mapped
						sam->buffer2[strlen(sam->buffer2)-1]='\0';
						fprintf(single_file,"%s",sam->buffer2);
						if((flag1 & 0x10) == 0x10){
								reverse_complement_read_ls_text(seq1,tmp_seq);
								strcpy(seq1,tmp_seq);
						}
						fprintf(single_file,"\tR2:Z:%s\n",seq1);
					}
					if(((strchr(cigar2,'S') && strlen(seq1)>50)||strchr(cigar2,'I')||strchr(cigar2,'D')||strchr(cigar2,'N')) && (flag1 & 0x04)==0){
							sam->buffer1[strlen(sam->buffer1)-1]='\0';
							fprintf(single_file,"%s",sam->buffer1);
							if((flag2 & 0x10) == 0x10){
									reverse_complement_read_ls_text(seq2,tmp_seq);
									strcpy(seq2,tmp_seq);
							}
							fprintf(single_file,"\tR2:Z:%s\n",seq2);
					}

				}
			}

			/*inver*/
			if((flag1 & 0x10)== (flag2 & 0x10)){//same strand for inver
				if(abs(isize1)>inver_isize){
					fprintf(inver_file,"%s",sam->buffer1);

					fprintf(inver_file,"%s",sam->buffer2);
					if(((strchr(cigar1,'S') && strlen(seq1)>50)||strchr(cigar1,'I')||strchr(cigar1,'D')||strchr(cigar1,'N')) && (flag2 & 0x04)==0){//cigar1 includes S, and its mate is mapped
						sam->buffer2[strlen(sam->buffer2)-1]='\0';
						fprintf(single_file,"%s",sam->buffer2);
						if((flag1 & 0x10) == 0x10){
							reverse_complement_read_ls_text(seq1,tmp_seq);
							strcpy(seq1,tmp_seq);
						}
						fprintf(single_file,"\tR2:Z:%s\n",seq1);
					}
					if(((strchr(cigar2,'S') && strlen(seq1)>50)||strchr(cigar2,'I')||strchr(cigar2,'D')||strchr(cigar2,'N')) && (flag1 & 0x04)==0){
						sam->buffer1[strlen(sam->buffer1)-1]='\0';
						fprintf(single_file,"%s",sam->buffer1);
						if((flag2 & 0x10) == 0x10){
										reverse_complement_read_ls_text(seq2,tmp_seq);
										strcpy(seq2,tmp_seq);
						}
						fprintf(single_file,"\tR2:Z:%s\n",seq2);

					}
                /*VIRTUAL COORDINATE IS TO BEGENERATED FOR DUP & INVER*/
				}else{
					if(((strchr(cigar1,'S') && strlen(seq1)>50)||strchr(cigar1,'I')||strchr(cigar1,'D')||strchr(cigar1,'N')) && (flag2 & 0x04)==0){//cigar1 includes S, and its mate is mapped

							sam->buffer2[strlen(sam->buffer2)-1]='\0';
							fprintf(single_file,"%s",sam->buffer2);

							if((flag1 & 0x10) == 0x10){
									reverse_complement_read_ls_text(seq1,tmp_seq);
									strcpy(seq1,tmp_seq);
							}
							fprintf(single_file,"\tR2:Z:%s\n",seq1);
					}
					if(((strchr(cigar2,'S') && strlen(seq1)>50)||strchr(cigar2,'I')||strchr(cigar2,'D')||strchr(cigar2,'N')) && (flag1 & 0x04)==0){
							sam->buffer1[strlen(sam->buffer1)-1]='\0';
							fprintf(single_file,"%s",sam->buffer1);
							if((flag2 & 0x10) == 0x10){
									reverse_complement_read_ls_text(seq2,tmp_seq);
									strcpy(seq2,tmp_seq);
							}
							fprintf(single_file,"\tR2:Z:%s\n",seq2);
					}
				}
			}
		}else if((flag1 & 0x04)==0x04 && (flag1 & 0x08)==0){//mate1 not mapped
			sam->buffer2[strlen(sam->buffer2)-1]='\0';
			fprintf(single_file,"%s",sam->buffer2);
			if((flag1 & 0x10) == 0x10){
				reverse_complement_read_ls_text(seq1,tmp_seq);
				strcpy(seq1,tmp_seq);
	    	}
			fprintf(single_file,"\tR2:Z:%s\n",seq1);
		}else if((flag2 & 0x04)==0x04 && (flag2 & 0x08)==0){//mate2 not mapped
			sam->buffer1[strlen(sam->buffer1)-1]='\0';
			fprintf(single_file,"%s",sam->buffer1);
			if((flag2 & 0x10) == 0x10){
				reverse_complement_read_ls_text(seq2,tmp_seq);
				strcpy(seq2,tmp_seq);
			}
			fprintf(single_file,"\tR2:Z:%s\n",seq2);
		}

	}


	sam_close(sam);

	fclose(large_file);
	fclose(inver_file);
	fclose(dup_file);
	fclose(single_file);

	return 0;

}

int
main(int argc, char **argv){
	int i;
	cmd_num = argc;
	for(i=0;i<cmd_num;i++){
		cmd_line[i] = xstrdup(argv[i]);
	}
	if(argc==1){
 	        usage(100);
	}

	if (strcmp(argv[1], "mapsplit") == 0){
		 return smapper_main(argc-2, argv+1);//read file1,read file2, out file,  genome file num, genome file[]
	}else if (strcmp(argv[1], "bwasam2tracks") == 0){ 	
		if(argc==6){
			return get_discordant_from_bwa_sam(argv[2], atoi(argv[3]),atoi(argv[4]),argv[5]);
		}else{
			usage(2);
		}
	}else if (strcmp(argv[1], "detectsv") == 0){
		 return detect_sv_main(argc-2, argv+1);//split reads num, deletion start, deletion size
	}else if (strcmp(argv[1], "rmoverlap") == 0){
		if(argc==3){
			return rm_overlaped_sv(argv[2]);
		}else{
			usage(5);
		}
	}else if (strcmp(argv[1], "formatsv") == 0){
		if(argc==3){
			return format_sv(argv[2]);
                }else{
			usage(4);
		}
	}else {
		fprintf(stderr, "[main] unrecognized command '%s'\n", argv[1]);
		usage(100);
	}
	for(i=0;i<cmd_num;i++){
		free(cmd_line[i]);
	}
	return 0;

}
