#/usr/bin/env bash
set -eu
set -o pipefail

prism_sam="_none"
supp_read_num=4
best_score_thresh=874
second_score_thresh=500
third_score_thresh=500
sort_mem="1g"
usage()
{
        echo "usage:"
        echo "  -i PRiSM output sam file."
        echo "  -n minmal support read number for each SV.[default:5]"
        echo "  -1 1st score threshold.[default:874] "
        echo "  -2 2st score threshold.[default:500] "
        echo "  -3 3st score threshold.[default:500] "
        exit 0
}

export PRISM_PATH

if test ! -d ${PRISM_PATH} || [ -z ${PRISM_PATH} ];then
        echo  "PRISM_PATH(${PRISM_PATH}) is invalid."
        exit 0;
fi
if test ! -e "${PRISM_PATH}/bin/smapper";then
        echo "PRiSM bin files lost."
        exit 0
fi
if test ! -e "${PRISM_PATH}/bin/cluster_matepairs";then
        echo "Cluster bin files lost."
        exit 0
fi

while getopts "i:n:1:2:3:S:" options;do
        case $options in
                i)      prism_sam=$OPTARG;echo -e "PRiSM sam:\t${prism_sam}";;
                n)      supp_read_num=$OPTARG;echo -e "Support read num:\t${supp_read_num}";;
                1)      best_score_thresh=$OPTARG;echo -e "1st score thresh:\t${best_score_thresh}";;
                2)      second_score_thresh=$OPTARG;echo -e "2st score thresh:\t${second_score_thresh}";;
                3)      third_score_thresh=$OPTARG;echo -e "3st score thresh:\t${third_score_thresh}";;
		S)	sort_mem=$OPTARG;;
                \?)     usage;;
        esac
done
if test ! -e "${PRISM_PATH}/bin/smapper";then
        echo "PRiSM bin files lost."
        exit 0
fi

if test ! -e ${prism_sam};then
        echo "input file is invalid:$prism_sam."
        usage
        exit 0
fi
echo "PLEASE NOTE: the default 1st_score_thresh,2st_score_thresh,3st_score_thresh are for 100bp long reads. If your reads have different
	length change these values.For default penalties of PRiSM match score is 10. So the maximum score of an alignment is read_length*10.
	Give a penalty for this score (e.g. -150) or multiply a coefficient (e.g. 0.85) for your need."

output_path=$(dirname "${prism_sam}")
#output_path="/tmp/tmpfile"
sort -S ${sort_mem} -k1,1 ${prism_sam} -o ${prism_sam}
${PRISM_PATH}/toolkit/rm_multi_read.py ${prism_sam}
${PRISM_PATH}/bin/smapper formatsv ${prism_sam}_rmmul 
sort -S ${sort_mem} -u -k2,2n -k3,3n -k6,6 -k4,4n ${prism_sam}_rmmul_cigar -o ${prism_sam}_rmmul_cigar_sorted
${PRISM_PATH}/bin/smapper detectsv -n $supp_read_num ${prism_sam}_rmmul_cigar_sorted

awk -v first=${best_score_thresh} -v second=${second_score_thresh} -v third=${third_score_thresh} -v srn=${supp_read_num} '$6>srn && $9>first && $10 > second && $11>third' ${prism_sam}_rmmul_cigar_sorted_sv > ${output_path}/FILTERED_LOCI
awk '($5=="DEL" || $5=="INS") && $4<=100' ${output_path}/FILTERED_LOCI > ${output_path}/FILTERED_LOCI_INDEL
awk '$5=="DEL" && $4<=500 &&$4>100' ${output_path}/FILTERED_LOCI > ${output_path}/FILTERED_LOCI_LARGE_DEL_100_500
awk '$5=="DEL" && $4<=1000 &&$4>500' ${output_path}/FILTERED_LOCI > ${output_path}/FILTERED_LOCI_LARGE_DEL_500_1000
awk '$5=="DEL" && $4<=5000 &&$4>1000' ${output_path}/FILTERED_LOCI > ${output_path}/FILTERED_LOCI_LARGE_DEL_1000_5000
awk '$5=="DEL" && $4>5000' ${output_path}/FILTERED_LOCI > ${output_path}/FILTERED_LOCI_LARGE_DEL_5000
awk '$5=="DUP"' ${output_path}/FILTERED_LOCI > ${output_path}/FILTERED_LOCI_DUP
awk '$5=="INV"' ${output_path}/FILTERED_LOCI > ${output_path}/FILTERED_LOCI_INV

${PRISM_PATH}/bin/smapper rmoverlap ${output_path}/FILTERED_LOCI_INDEL
${PRISM_PATH}/bin/smapper rmoverlap ${output_path}/FILTERED_LOCI_LARGE_DEL_100_500
${PRISM_PATH}/bin/smapper rmoverlap ${output_path}/FILTERED_LOCI_LARGE_DEL_500_1000
${PRISM_PATH}/bin/smapper rmoverlap ${output_path}/FILTERED_LOCI_LARGE_DEL_500_1000
${PRISM_PATH}/bin/smapper rmoverlap ${output_path}/FILTERED_LOCI_LARGE_DEL_1000_5000
${PRISM_PATH}/bin/smapper rmoverlap ${output_path}/FILTERED_LOCI_LARGE_DEL_5000
${PRISM_PATH}/bin/smapper rmoverlap ${output_path}/FILTERED_LOCI_INV
${PRISM_PATH}/bin/smapper rmoverlap ${output_path}/FILTERED_LOCI_DUP
