#!/usr/bin/env bash
set -eu
set -o pipefail
export PRISM_PATH

mean=-1
stdev=-1 
min_mps=5
USER_FOLDER="."
input_folder="${USER_FOLDER}/PRISM_input"
output_folder="${USER_FOLDER}/PRISM_output"
input_file="_none"
#cnver_path="/data/jiangyue/cnver-0.7.2"
#PRISM_PATH="/filer/jiangyue/PRISM_oua_debug"
cnver_folder="${PRISM_PATH}/bin"
toolkit_path="${PRISM_PATH}/toolkit"
#ref_path="/filer/jiangyue/PRISM/sample/reference"
#ref_file="${PRISM_PATH}/sample/chr100.fa"
ref_file="_none"
aligner="BWA"
aligner_flag=""
supp_read_num=4
best_score_thresh=874
second_score_thresh=500
third_score_thresh=500
sort_mem="1g"
min_rlen=100;
usage()
{
	echo "usage:"
        echo "  -m mean of insert size."
        echo "  -e standard deviation of insert size."
        echo "  -p minmal support read number of a cluster.[default:5]"
        echo "  -I input folder of PRISM, the cluster files and input sam files will be generated in this folder.[default:\"./PRISM_input\"]"
        echo "  -O output folder of PRISM, the output sam files will be generated in this folder.[default:\"./PRISM_output\"]"
        echo "  -r reference file"
        echo "  -i the original input sam file generated by read aligner."
        echo "  -a the read aligner used to align the original reads. [BWA] for BWA letter space and [SHR] for SHRiMP color space.[default:BWA]"
        echo "  -l the minimal read length.This value will descide the kmer length.[default:100]"

	echo "  following parameters are for sv calling, they are the same as in filter_and_detect.sh"
	echo "  -n minmal support read number for each SV.[default:5]"
        echo "  -1 1st score threshold.[default:874] "
        echo "  -2 2st score threshold.[default:500] "
        echo "  -3 3st score threshold.[default:500] "

        exit 0
}

if test ! -d ${PRISM_PATH} || [ -z ${PRISM_PATH} ];then
        echo  "PRISM_PATH(${PRISM_PATH}) is invalid."
        exit 0;
fi
if test ! -e "${PRISM_PATH}/bin/smapper";then
        echo "PRISM bin files lost."
        exit 0
fi
if test ! -e "${PRISM_PATH}/bin/cluster_matepairs";then
        echo "Cluster bin files lost."
        exit 0
fi

while getopts "m:e:p:r:I:O:i:a:n:1:2:3:S:l:" options;do
        case $options in
                m)      mean=$OPTARG;;
                e)      stdev=$OPTARG;;
                p)      min=$OPTARG;;
                r)      ref_file=$OPTARG;;
                I)      input_folder=$OPTARG;;
                O)      output_folder=$OPTARG;;
                i)      input_file=$OPTARG;;
		a)	aligner=$OPTARG;;
		l)	min_rlen=$OPTARG;;
		
		n)      supp_read_num=$OPTARG;;
                1)      best_score_thresh=$OPTARG;;
                2)      second_score_thresh=$OPTARG;;
                3)      third_score_thresh=$OPTARG;;
		
		S)	sort_mem=$OPTARG;;
                \?)     usage;;
        esac
done
if [ $mean -lt 0 ];then
	echo "mean of insert size is not specified. [see usage -m]"
	usage
	exit 0
fi
if [ $stdev -lt 0 ];then
	echo "standard deviation of insert size is not specified. [see usage -e ]"
	usage
	exit 0
fi
if test ! -e ${input_file};then
	echo "input file is not specified. [see usage -i]"
	usage
	exit 0
fi
if test ! -e ${ref_file};then
	echo "reference file is not specified. [see usage -r]"
	usage
	exit 0
fi

if [ "$aligner" != "BWA" -a "$aligner" != "SHR" ];then
	echo "unsupported read aligner:$aligner. [see usage -a]"
        usage
        exit 0
fi
mkdir -pv ${input_folder}
mkdir -pv ${output_folder}
echo -e "[Parameters:]"

echo -e "mean:\t$mean"
echo -e "std:\t$stdev"
echo -e "min mps:\t$min_mps"
echo -e "Reference:\t$ref_file"
echo -e "Input folder:\t$input_folder"
echo -e "Output folder:\t$output_folder"
echo -e "input file:\t$input_file"
echo -e "read aligner:\t$aligner"

echo -e "Support read num for SV calling:\t${supp_read_num}"
echo -e "1st score thresh for SV calling:\t${best_score_thresh}"
echo -e "2st score thresh for SV calling:\t${second_score_thresh}"
echo -e "3st score thresh for SV calling:\t${third_score_thresh}"


echo -e "\n[Preparing input files ... ]"
#${PRISM_PATH}/bin/smapper bwasam2tracks $input_file $mean $stdev $input_folder
if [ $aligner == "BWA" ];then
	${toolkit_path}/bwasam2prism.py ${input_file} ${input_folder} $mean $stdev
else
	echo "${toolkit_path}/shrimpsam_cs2prism.py ${input_file} ${input_folder} $mean $stdev"
	${toolkit_path}/shrimpsam_cs2prism.py ${input_file} ${input_folder} $mean $stdev
	aligner_flag="-C"
fi
prefix=$(basename "${input_file}")
sort -S ${sort_mem} -k4,4n ${input_folder}/${prefix}_single -o ${input_folder}/${prefix}_single_sorted
${toolkit_path}/cnver_cluster.sh $mean $stdev $min_mps ${input_folder}/${prefix} ${sort_mem}
echo -e "[Input files done.]"
echo -e "\n[PRISM running ... ]"
echo "PRISM CMD:${PRISM_PATH}/bin/smapper mapsplit $aligner_flag -r ${min_rlen} -f0 -g300 -m $mean -e $stdev -o ${output_folder}/split -S ${input_folder}/${prefix}_single_sorted -L ${input_folder}/${prefix}.links.del.fw -l ${input_folder}/${prefix}.links.del.bw -D ${input_folder}/${prefix}.links.dup.fw -d ${input_folder}/${prefix}.links.dup.bw -I ${input_folder}/${prefix}.links.inver.fw -i ${input_folder}/${prefix}.links.inver.bw ${ref_file} > ${output_folder}/log"
${PRISM_PATH}/bin/smapper mapsplit $aligner_flag -f0 -g300 -m $mean -e $stdev -o ${output_folder}/split -S ${input_folder}/${prefix}_single_sorted -L ${input_folder}/${prefix}.links.del.fw -l ${input_folder}/${prefix}.links.del.bw -D ${input_folder}/${prefix}.links.dup.fw -d ${input_folder}/${prefix}.links.dup.bw -I ${input_folder}/${prefix}.links.inver.fw -i ${input_folder}/${prefix}.links.inver.bw ${ref_file} > ${output_folder}/log
echo -e "[PRISM done.]"
echo -e "[Calling SV ... ]"
cat ${output_folder}/split_indel.sam ${output_folder}/split_cluster_del.sam ${output_folder}/split_inv.sam ${output_folder}/split_dup.sam > ${output_folder}/split_all.sam

${toolkit_path}/filter_and_detect.sh -S ${sort_mem} -i ${output_folder}/split_all.sam -n $supp_read_num -1 $best_score_thresh -2 $second_score_thresh -3 $third_score_thresh
echo -e "[All done.]"
